@@ -1,4 +1,4 @@
-#' @include internal.R pproto.R ConservationProblem-proto.R zones.R
+#' @include internal.R ConservationProblem-class.R zones.R
 NULL
 
 #' Add absolute targets
@@ -9,9 +9,9 @@ NULL
 #' planning units for which their summed feature values are equal to or greater
 #' than 10.
 #'
-#' @param x [problem()] (i.e., [`ConservationProblem-class`]) object.
+#' @param x [problem()] object.
 #'
-#' @param targets Object that specifies the targets for each feature.
+#' @param targets object that specifies the targets for each feature.
 #'   See the Targets format section for more information.
 #'
 #' @details
@@ -56,11 +56,14 @@ NULL
 #'   `x`, and each cell contains the target value for a given feature
 #'   that the solution needs to secure in a given zone.}
 #'
-#' \item{`targets` as a `character` vector}{containing the names of fields
-#'   (columns) in the feature data associated with the argument to `x` that
+#' \item{`targets` as a `character` vector}{containing the column name(s) in the
+#'   feature data associated with the argument to `x` that
 #'   contain targets. This format can only be used when the
-#'   feature data associated with `x` is a `data.frame`.
-#'   This argument must contain a field (column) name for each zone.}
+#'   feature data associated with `x` is a [sf::st_sf()] or `data.frame`.
+#'   For problems that contain a single zone, the argument to `targets` must
+#'   contain a single column name. Otherwise, for problems that
+#'   contain multiple zones, the argument to `targets` must
+#'   contain a column name for each zone.}
 #'
 #' }
 #'
@@ -77,40 +80,49 @@ NULL
 #' set.seed(500)
 #'
 #' # load data
-#' data(sim_pu_raster, sim_features, sim_pu_zones_stack, sim_features_zones)
+#' sim_pu_raster <- get_sim_pu_raster()
+#' sim_features <- get_sim_features()
+#' sim_zones_pu_raster <- get_sim_zones_pu_raster()
+#' sim_zones_features <- get_sim_zones_features()
 #'
-#' # create simple problem
-#' p <- problem(sim_pu_raster, sim_features) %>%
-#'      add_min_set_objective() %>%
-#'      add_binary_decisions() %>%
-#'      add_default_solver(verbose = FALSE)
+#' # create minimal problem with no targets
+#' p0 <-
+#'   problem(sim_pu_raster, sim_features) %>%
+#'   add_min_set_objective() %>%
+#'   add_binary_decisions() %>%
+#'   add_default_solver(verbose = FALSE)
 #'
 #' # create problem with targets to secure 3 amounts for each feature
-#' p1 <- p %>% add_absolute_targets(3)
+#' p1 <- p0 %>% add_absolute_targets(3)
 #'
 #' # create problem with varying targets for each feature
 #' targets <- c(1, 2, 3, 2, 1)
-#' p2 <- p %>% add_absolute_targets(targets)
+#' p2 <- p0 %>% add_absolute_targets(targets)
 #'
 #' # solve problem
-#' s <- stack(solve(p1), solve(p2))
+#' s1 <- c(solve(p1), solve(p2))
+#' names(s1) <- c("equal targets", "varying targets")
 #'
 #' # plot solution
-#' plot(s, main = c("equal targets", "varying targets"), axes = FALSE,
-#'      box = FALSE)
+#' plot(s1, axes = FALSE)
 #'
 #' # create a problem with multiple management zones
-#' p3 <- problem(sim_pu_zones_stack, sim_features_zones) %>%
-#'       add_min_set_objective() %>%
-#'       add_binary_decisions() %>%
-#'       add_default_solver(verbose = FALSE)
+#' p3 <-
+#'   problem(sim_zones_pu_raster, sim_zones_features) %>%
+#'   add_min_set_objective() %>%
+#'   add_binary_decisions() %>%
+#'   add_default_solver(verbose = FALSE)
 #'
 #' # create a problem with targets that specify an equal amount of each feature
 #' # to be represented in each zone
-#' p4_targets <- matrix(2, nrow = number_of_features(sim_features_zones),
-#'                      ncol = number_of_zones(sim_features_zones),
-#'                      dimnames = list(feature_names(sim_features_zones),
-#'                                      zone_names(sim_features_zones)))
+#' p4_targets <- matrix(
+#'   2,
+#'   nrow = number_of_features(sim_zones_features),
+#'   ncol = number_of_zones(sim_zones_features),
+#'   dimnames = list(
+#'     feature_names(sim_zones_features), zone_names(sim_zones_features)
+#'   )
+#' )
 #' print(p4_targets)
 #'
 #' p4 <- p3 %>% add_absolute_targets(p4_targets)
@@ -119,24 +131,28 @@ NULL
 #' s4 <- solve(p4)
 #'
 #' # plot solution (pixel values correspond to zone identifiers)
-#' plot(category_layer(s4), main = c("equal targets"))
+#' plot(category_layer(s4), main = "equal targets", axes = FALSE)
 #'
 #' # create a problem with targets that require a varying amount of each
 #' # feature to be represented in each zone
-#' p5_targets <- matrix(rpois(15, 1),
-#'                      nrow = number_of_features(sim_features_zones),
-#'                      ncol = number_of_zones(sim_features_zones),
-#'                      dimnames = list(feature_names(sim_features_zones),
-#'                                      zone_names(sim_features_zones)))
+#' p5_targets <- matrix(
+#'   rpois(15, 1),
+#'   nrow = number_of_features(sim_zones_features),
+#'   ncol = number_of_zones(sim_zones_features),
+#'   dimnames = list(
+#'     feature_names(sim_zones_features),
+#'     zone_names(sim_zones_features)
+#'   )
+#' )
 #' print(p5_targets)
 #'
-#' p5 <- p3 %>% add_absolute_targets(p4_targets)
+#' p5 <- p3 %>% add_absolute_targets(p5_targets)
 #'
 #' # solve problem
 #' s5 <- solve(p5)
 #'
 #' # plot solution (pixel values correspond to zone identifiers)
-#' plot(category_layer(s5), main = c("varying targets"))
+#' plot(category_layer(s5), main = "varying targets", axes = FALSE)
 #' }
 #'
 #' @aliases add_absolute_targets-method add_absolute_targets,ConservationProblem,numeric-method add_absolute_targets,ConservationProblem,matrix-method add_absolute_targets,ConservationProblem,character-method
@@ -151,7 +167,16 @@ NULL
 methods::setGeneric(
   "add_absolute_targets",
   signature = methods::signature("x", "targets"),
-  function(x, targets) standardGeneric("add_absolute_targets"))
+  function(x, targets) {
+    rlang::check_required(x)
+    rlang::check_required(targets)
+    assert(
+      is_conservation_problem(x),
+      is_inherits(targets, c("character", "numeric", "matrix"))
+    )
+    standardGeneric("add_absolute_targets")
+  }
+)
 
 #' @name add_absolute_targets
 #' @rdname add_absolute_targets
@@ -160,15 +185,27 @@ methods::setMethod(
   "add_absolute_targets",
   methods::signature("ConservationProblem", "numeric"),
   function(x, targets) {
-    assertthat::assert_that(inherits(x, "ConservationProblem"))
-    assertthat::assert_that(x$number_of_zones() == 1,
-                            msg = paste("argument to x has multiple zones,",
-                                        "and so targets must be provided as",
-                                        "a matrix"))
-    assertthat::assert_that(length(targets) %in% c(1, x$number_of_features()))
-    add_absolute_targets(x, matrix(targets, nrow = x$number_of_features(),
-                                   ncol = 1))
-})
+    assert(is_conservation_problem(x))
+    assert(
+      x$number_of_zones() == 1,
+      msg = paste(
+        "{.arg targets} must be a character vector or matrix, because",
+        "{.arg x} has multiple zones"
+      )
+    )
+    assert(
+      length(targets) %in% c(1, x$number_of_features()),
+      msg  = paste(
+        "{.arg targets} must be a single numeric value,",
+        "or a numeric vector containing ", x$number_of_features(),
+        "values (one for each feature)."
+      )
+    )
+    add_absolute_targets(
+      x, matrix(targets, nrow = x$number_of_features(), ncol = 1)
+    )
+  }
+)
 
 #' @name add_absolute_targets
 #' @rdname add_absolute_targets
@@ -178,29 +215,38 @@ methods::setMethod(
   methods::signature("ConservationProblem", "matrix"),
   function(x, targets) {
     # assert that arguments are valid
-    assertthat::assert_that(
-      inherits(x, "ConservationProblem"),
-      inherits(targets, "matrix"), is.numeric(targets),
-      isTRUE(all(is.finite(targets))),
-      isTRUE(length(targets) > 0),
+    assert(
+      is_conservation_problem(x),
+      is.matrix(targets),
+      is.numeric(targets),
+      all_finite(targets),
+      length(targets) > 0,
       nrow(targets) == x$number_of_features(),
-      ncol(targets) == x$number_of_zones())
-    verify_that(all(targets >= 0.0, na.rm = TRUE))
-    verify_that(all(targets <= x$feature_abundances_in_planning_units(),
-                    na.rm = TRUE))
+      ncol(targets) == x$number_of_zones()
+    )
+    verify(all_positive(targets))
+    verify(
+      all(targets <= x$feature_abundances_in_planning_units(), na.rm = TRUE),
+      msg = paste0(
+        "Some {.arg targets} values cannot be met even if all planning units",
+        "are selected."
+      )
+    )
     # create targets as data.frame
     if (x$number_of_zones() > 1) {
-      target_data <- expand.grid(feature = x$feature_names(),
-                                 zone = x$zone_names(),
-                                 type = "absolute")
+      target_data <- expand.grid(
+        feature = x$feature_names(),
+        zone = x$zone_names(),
+        type = "absolute"
+      )
     } else {
-      target_data <- expand.grid(feature = x$feature_names(),
-                                 type = "absolute")
+      target_data <- data.frame(feature = x$feature_names(), type = "absolute")
     }
     target_data$target <- as.numeric(targets)
     # add targets to problem
     add_manual_targets(x, target_data)
-  })
+  }
+)
 
 #' @name add_absolute_targets
 #' @rdname add_absolute_targets
@@ -210,14 +256,37 @@ methods::setMethod(
   methods::signature("ConservationProblem", "character"),
   function(x, targets) {
     # assert that arguments are valid
-    assertthat::assert_that(inherits(x, "ConservationProblem"),
+    assert(
+      is_conservation_problem(x),
       is.character(targets),
-      inherits(x$data$features, "data.frame"),
-      !anyNA(targets),
+      assertthat::noNA(targets),
+      length(targets) == number_of_zones(x)
+    )
+    assert(
+      is.data.frame(x$data$features),
+      msg = paste(
+        "{.arg targets} cannot be a character vector, because the feature data",
+        "for {.arg x} are not a data frame."
+      )
+    )
+    assert(
       all(assertthat::has_name(x$data$features, targets)),
-      length(targets) == x$number_of_zones(),
-      all(vapply(x$data$features[, targets, drop = FALSE], is.numeric,
-                logical(1))))
+      msg = paste0(
+        "{.arg targets} must contain character values that are",
+        "column names of the feature data for {.arg x}."
+      )
+    )
+    assert(
+      all_columns_inherit(
+        x$data$features[, targets, drop = FALSE],
+        "numeric"
+      ),
+      msg = paste(
+        "{.arg targets} must contain character values that",
+        "refer to numeric columns of the feature data for {.arg x}."
+      )
+    )
     # add targets to problem
     add_absolute_targets(x, as.matrix(x$data$features[, targets, drop = FALSE]))
-})
+  }
+)
