@@ -1,17 +1,17 @@
-#' @include internal.R Penalty-proto.R marxan_boundary_data_to_matrix.R
+#' @include internal.R Penalty-class.R marxan_connectivity_data_to_matrix.R
 NULL
 
 #' Add connectivity penalties
 #'
-#' Add penalties to a conservation planning [problem()] to account for
+#' Add penalties to a conservation planning problem to account for
 #' symmetric connectivity between planning units.
 #' Symmetric connectivity data describe connectivity information that is not
 #' directional. For example, symmetric connectivity data could describe which
-#' planning units are adjacent to each other (see [adjacency_matrix()],
+#' planning units are adjacent to each other (see [adjacency_matrix()]),
 #' or which planning units are within threshold distance of each other (see
 #' [proximity_matrix()]).
 #'
-#' @param x [problem()] (i.e., [`ConservationProblem-class`]) object.
+#' @param x [problem()] object.
 #'
 #' @param penalty `numeric` penalty that is used to scale the importance
 #'   of selecting planning units with strong connectivity between them compared
@@ -69,16 +69,16 @@ NULL
 #'   The default argument for `zones` is to treat planning units
 #'   allocated to different zones as having zero connectivity.}
 #'
-#' \item{`data` as a `data.frame` object}{containing the fields (columns)
+#' \item{`data` as a `data.frame` object}{containing columns that are named
 #'   `"id1"`, `"id2"`, and `"boundary"`. Here, each row
 #'   denotes the connectivity between a pair of planning units
 #'   (per values in the `"id1"` and `"id2"` columns) following the
 #'   *Marxan* format.
-#'   If the argument to `x` contains multiple zones, then the columns
-#'   `"zone1"` and `"zone2"` can optionally be provided to manually
+#'   If the argument to `x` contains multiple zones, then the
+#'   `"zone1"` and `"zone2"` columns can optionally be provided to manually
 #'   specify the connectivity values between planning units when they are
-#'   allocated to specific zones. If the columns `"zone1"` and
-#'   `"zone2"` are present, then the argument to `zones` must be
+#'   allocated to specific zones. If the `"zone1"` and
+#'   `"zone2"` columns are present, then the argument to `zones` must be
 #'   `NULL`.}
 #'
 #' \item{`data` as an `array` object}{
@@ -168,7 +168,10 @@ NULL
 #' set.seed(600)
 #'
 #' # load data
-#' data(sim_pu_sf, sim_pu_zones_stack, sim_features, sim_features_zones)
+#' sim_pu_polygons <- get_sim_pu_polygons()
+#' sim_features <- get_sim_features()
+#' sim_zones_pu_raster <- get_sim_zones_pu_raster()
+#' sim_zones_features <- get_sim_zones_features()
 #'
 #' # define function to rescale values between zero and one so that we
 #' # can compare solutions from different connectivity matrices
@@ -177,14 +180,15 @@ NULL
 #' }
 #'
 #' # create basic problem
-#' p1 <- problem(sim_pu_sf, sim_features, "cost") %>%
-#'       add_min_set_objective() %>%
-#'       add_relative_targets(0.2) %>%
-#'       add_default_solver(verbose = FALSE)
+#' p1 <-
+#'   problem(sim_pu_polygons, sim_features, "cost") %>%
+#'   add_min_set_objective() %>%
+#'   add_relative_targets(0.2) %>%
+#'   add_default_solver(verbose = FALSE)
 #'
 #' # create a symmetric connectivity matrix where the connectivity between
 #' # two planning units corresponds to their shared boundary length
-#' b_matrix <- boundary_matrix(sim_pu_sf)
+#' b_matrix <- boundary_matrix(sim_pu_polygons)
 #'
 #' # standardize matrix values to lay between zero and one
 #' b_matrix[] <- rescale(b_matrix[])
@@ -195,7 +199,9 @@ NULL
 #' # create a symmetric connectivity matrix where the connectivity between
 #' # two planning units corresponds to their spatial proximity
 #' # i.e., planning units that are further apart share less connectivity
-#' centroids <- sf::st_coordinates(suppressWarnings(sf::st_centroid(sim_pu_sf)))
+#' centroids <- sf::st_coordinates(
+#'   suppressWarnings(sf::st_centroid(sim_pu_polygons))
+#' )
 #' d_matrix <- (1 / (Matrix::Matrix(as.matrix(dist(centroids))) + 1))
 #'
 #' # standardize matrix values to lay between zero and one
@@ -210,12 +216,12 @@ NULL
 #'
 #' # create a symmetric connectivity matrix where the connectivity
 #' # between adjacent two planning units corresponds to their combined
-#' # value in a field in the planning unit attribute data
-#' # for example, this field could describe the extent of native vegetation in
+#' # value in a column of the planning unit data
+#' # for example, this column could describe the extent of native vegetation in
 #' # each planning unit and we could use connectivity penalties to identify
 #' # solutions that cluster planning units together that both contain large
 #' # amounts of native vegetation
-#' c_matrix <- connectivity_matrix(sim_pu_sf, "cost")
+#' c_matrix <- connectivity_matrix(sim_pu_polygons, "cost")
 #'
 #' # standardize matrix values to lay between zero and one
 #' c_matrix[] <- rescale(c_matrix[])
@@ -223,45 +229,59 @@ NULL
 #' # visualize connectivity matrix
 #' image(c_matrix)
 #'
-#'
 #' # create penalties
 #' penalties <- c(10, 25)
 #'
 #' # create problems using the different connectivity matrices and penalties
-#' p2 <- list(p1,
-#'            p1 %>% add_connectivity_penalties(penalties[1], data = b_matrix),
-#'            p1 %>% add_connectivity_penalties(penalties[2], data = b_matrix),
-#'            p1 %>% add_connectivity_penalties(penalties[1], data = d_matrix),
-#'            p1 %>% add_connectivity_penalties(penalties[2], data = d_matrix),
-#'            p1 %>% add_connectivity_penalties(penalties[1], data = c_matrix),
-#'            p1 %>% add_connectivity_penalties(penalties[2], data = c_matrix))
+#' p2 <- list(
+#'   p1,
+#'   p1 %>% add_connectivity_penalties(penalties[1], data = b_matrix),
+#'   p1 %>% add_connectivity_penalties(penalties[2], data = b_matrix),
+#'   p1 %>% add_connectivity_penalties(penalties[1], data = d_matrix),
+#'   p1 %>% add_connectivity_penalties(penalties[2], data = d_matrix),
+#'   p1 %>% add_connectivity_penalties(penalties[1], data = c_matrix),
+#'   p1 %>% add_connectivity_penalties(penalties[2], data = c_matrix)
+#' )
 #'
 #' # assign names to the problems
-#' names(p2) <- c("basic problem",
-#'                paste0("b_matrix (", penalties,")"),
-#'                paste0("d_matrix (", penalties,")"),
-#'                paste0("c_matrix (", penalties,")"))
+#' names(p2) <- c(
+#'   "basic problem",
+#'   paste0("b_matrix (", penalties,")"),
+#'   paste0("d_matrix (", penalties,")"),
+#'   paste0("c_matrix (", penalties,")")
+#' )
 #'
 #' # solve problems
 #' s2 <- lapply(p2, solve)
 #'
+#' # create single object with all solutions
+#' s2 <- sf::st_sf(
+#'   tibble::tibble(
+#'     p2_1 = s2[[1]]$solution_1,
+#'     p2_2 = s2[[2]]$solution_1,
+#'     p2_3 = s2[[3]]$solution_1,
+#'     p2_4 = s2[[4]]$solution_1,
+#'     p2_5 = s2[[5]]$solution_1,
+#'     p2_6 = s2[[6]]$solution_1,
+#'     p2_7 = s2[[7]]$solution_1
+#'   ),
+#'   geometry = sf::st_geometry(s2[[1]])
+#' )
+#'
 #' # plot solutions
-#' par(mfrow = c(2, 2))
-#' for (i in seq_along(s2)) {
-#'   plot(s2[[i]], main = names(p2)[i], cex = 1.5, col = "white")
-#'   plot(s2[[i]][s2[[i]]$solution_1 == 1, ], col = "darkgreen", add = TRUE)
-#' }
+#' plot(s2, main = names(p2))
 #'
 #' # create minimal multi-zone problem and limit solver to one minute
 #' # to obtain solutions in a short period of time
-#' p3 <- problem(sim_pu_zones_stack, sim_features_zones) %>%
-#'       add_min_set_objective() %>%
-#'       add_relative_targets(matrix(0.15, nrow = 5, ncol = 3)) %>%
-#'       add_binary_decisions() %>%
-#'       add_default_solver(time_limit = 60, verbose = FALSE)
+#' p3 <-
+#'   problem(sim_zones_pu_raster, sim_zones_features) %>%
+#'   add_min_set_objective() %>%
+#'   add_relative_targets(matrix(0.15, nrow = 5, ncol = 3)) %>%
+#'   add_binary_decisions() %>%
+#'   add_default_solver(time_limit = 60, verbose = FALSE)
 #'
 #' # create matrix showing which planning units are adjacent to other units
-#' a_matrix <- adjacency_matrix(sim_pu_zones_stack)
+#' a_matrix <- adjacency_matrix(sim_zones_pu_raster)
 #'
 #' # visualize matrix
 #' image(a_matrix)
@@ -334,50 +354,49 @@ NULL
 #'   p3 %>% add_connectivity_penalties(penalties2[1], zm4, a_matrix),
 #'   p3 %>% add_connectivity_penalties(penalties2[2], zm4, a_matrix),
 #'   p3 %>% add_connectivity_penalties(penalties2[1], zm5, a_matrix),
-#'   p3 %>% add_connectivity_penalties(penalties2[2], zm5, a_matrix))
-#'
-#' # assign names to the problems
-#' names(p4) <- c("basic problem",
-#'                paste0("zm", rep(seq_len(5), each = 2), " (",
-#'                       rep(penalties2, 2), ")"))
+#'   p3 %>% add_connectivity_penalties(penalties2[2], zm5, a_matrix)
+#' )
 #'
 #' # solve problems
 #' s4 <- lapply(p4, solve)
 #' s4 <- lapply(s4, category_layer)
-#' s4 <- stack(s4)
+#' s4 <- terra::rast(s4)
+#' names(s4) <-  c(
+#'   "basic problem",
+#'   paste0("zm", rep(seq_len(5), each = 2), " (", rep(penalties2, 2), ")")
+#' )
 #'
 #' # plot solutions
-#' plot(s4, main = names(p4), axes = FALSE, box = FALSE)
-#'
+#' plot(s4, axes = FALSE)
 #'
 #' # create an array to manually specify the connectivities between
 #' # each planning unit when they are allocated to each different zone
 #' # for real-world problems, these connectivities would be generated using
 #' # data - but here these connectivity values are assigned as random
 #' # ones or zeros
-#' c_array <- array(0, c(rep(ncell(sim_pu_zones_stack[[1]]), 2), 3, 3))
+#' c_array <- array(0, c(rep(ncell(sim_zones_pu_raster[[1]]), 2), 3, 3))
 #' for (z1 in seq_len(3))
 #'   for (z2 in seq_len(3))
-#'     c_array[, , z1, z2] <- round(runif(ncell(sim_pu_zones_stack[[1]]) ^ 2,
-#'                                        0, 0.505))
+#'     c_array[, , z1, z2] <- round(
+#'       runif(ncell(sim_zones_pu_raster[[1]]) ^ 2, 0, 0.505)
+#'     )
 #'
 #' # create a problem with the manually specified connectivity array
 #' # note that the zones argument is set to NULL because the connectivity
 #' # data is an array
-#' p5 <- list(p3,
-#'            p3 %>% add_connectivity_penalties(15, zones = NULL, c_array))
-#'
-#'
-#' # assign names to the problems
-#' names(p5) <- c("basic problem", "connectivity array")
+#' p5 <- list(
+#'   p3,
+#'   p3 %>% add_connectivity_penalties(15, zones = NULL, c_array)
+#' )
 #'
 #' # solve problems
 #' s5 <- lapply(p5, solve)
 #' s5 <- lapply(s5, category_layer)
-#' s5 <- stack(s5)
+#' s5 <- terra::rast(s5)
+#' names(s5) <- c("basic problem", "connectivity array")
 #'
 #' # plot solutions
-#' plot(s5, main = names(p5), axes = FALSE, box = FALSE)
+#' plot(s5, axes = FALSE)
 #' }
 #' @name add_connectivity_penalties
 #'
@@ -389,8 +408,21 @@ NULL
 #' @export
 methods::setGeneric("add_connectivity_penalties",
   signature = methods::signature("x", "penalty", "zones", "data"),
-  function(x, penalty, zones = diag(number_of_zones(x)), data)
-    standardGeneric("add_connectivity_penalties"))
+  function(x, penalty, zones = diag(number_of_zones(x)), data) {
+    rlang::check_required(x)
+    rlang::check_required(penalty)
+    rlang::check_required(zones)
+    rlang::check_required(data)
+    assert(
+      is_conservation_problem(x),
+      is_inherits(
+        data,
+        c("dgCMatrix", "data.frame", "matrix", "Matrix", "array")
+      )
+    )
+    standardGeneric("add_connectivity_penalties")
+  }
+)
 
 #' @name add_connectivity_penalties
 #' @usage \S4method{add_connectivity_penalties}{ConservationProblem,ANY,ANY,matrix}(x, penalty, zones, data)
@@ -400,8 +432,9 @@ methods::setMethod("add_connectivity_penalties",
   function(x, penalty, zones, data) {
     add_connectivity_penalties(
       x, penalty, zones, as_Matrix(data, "dgCMatrix")
-   )
-})
+    )
+  }
+)
 
 #' @name add_connectivity_penalties
 #' @usage \S4method{add_connectivity_penalties}{ConservationProblem,ANY,ANY,Matrix}(x, penalty, zones, data)
@@ -409,9 +442,11 @@ methods::setMethod("add_connectivity_penalties",
 methods::setMethod("add_connectivity_penalties",
   methods::signature("ConservationProblem", "ANY", "ANY", "Matrix"),
   function(x, penalty, zones, data) {
-     add_connectivity_penalties(x, penalty, zones,
-       as_Matrix(data, "dgCMatrix"))
-})
+     add_connectivity_penalties(
+       x, penalty, zones, as_Matrix(data, "dgCMatrix")
+     )
+  }
+)
 
 #' @name add_connectivity_penalties
 #' @usage \S4method{add_connectivity_penalties}{ConservationProblem,ANY,ANY,data.frame}(x, penalty, zones, data)
@@ -420,14 +455,19 @@ methods::setMethod("add_connectivity_penalties",
   methods::signature("ConservationProblem", "ANY", "ANY", "data.frame"),
   function(x, penalty, zones, data) {
     # assert valid arguments
-    assertthat::assert_that(
-      inherits(x, "ConservationProblem"), assertthat::is.scalar(penalty),
-      is.finite(penalty), is.data.frame(data))
-  # add penalties to problem
-  add_connectivity_penalties(
-    x, penalty, zones,
-   marxan_connectivity_data_to_matrix(x, data, symmetric = TRUE))
-})
+    assert(
+      is_conservation_problem(x),
+      assertthat::is.number(penalty),
+      all_finite(penalty),
+      is.data.frame(data)
+    )
+    # add penalties to problem
+    add_connectivity_penalties(
+      x, penalty, zones,
+      marxan_connectivity_data_to_matrix(x, data, symmetric = TRUE)
+    )
+  }
+)
 
 #' @name add_connectivity_penalties
 #' @usage \S4method{add_connectivity_penalties}{ConservationProblem,ANY,ANY,dgCMatrix}(x, penalty, zones, data)
@@ -436,40 +476,34 @@ methods::setMethod("add_connectivity_penalties",
   methods::signature("ConservationProblem", "ANY", "ANY", "dgCMatrix"),
   function(x, penalty, zones, data) {
     # assert valid arguments
-    assertthat::assert_that(
-      inherits(x, "ConservationProblem"), assertthat::is.scalar(penalty),
-      isTRUE(all(is.finite(penalty))), inherits(zones, c("matrix", "Matrix")),
-      nrow(zones) == ncol(zones), is.numeric(as.vector(zones)),
-      all(is.finite(as.vector(zones))),
-      is.numeric(data@x), ncol(data) == nrow(data),
-      max(zones) <= 1, min(zones) >= -1,
+    assert(
+      is_conservation_problem(x),
+      assertthat::is.number(penalty),
+      all_finite(penalty),
+      is_matrix_ish(zones),
+      nrow(zones) == ncol(zones),
+      is_numeric_values(zones),
+      all_finite(zones),
+      is_numeric_values(data),
+      all_finite(data),
+      ncol(data) == nrow(data),
+      max(zones) <= 1,
+      min(zones) >= -1,
       number_of_total_units(x) == ncol(data),
-      number_of_zones(x) == ncol(zones),
-      all(is.finite(data@x)))
+      number_of_zones(x) == ncol(zones)
+    )
     # check for symmetry
-    if (!Matrix::isSymmetric(data)) {
-      stop(
-        paste0(
-          "argument to data does not contain symmetric connectivity values, ",
-          "use add_asym_connectivity_penalties()"
-        )
+    assert(
+      Matrix::isSymmetric(data),
+      msg = paste0(
+        "{.arg data} does not contain symmetric connectivity values, ",
+        "use {.fn add_asym_connectivity_penalties} instead."
       )
-    }
-    # coerce zones to matrix
-    zones <- as.matrix(zones)
-    indices <- x$planning_unit_indices()
-    data <- data[indices, indices, drop = FALSE]
-    # convert zones & dgCMatrix data to list of sparse matrices
-    m <- list()
-    for (z1 in seq_len(ncol(zones))) {
-      m[[z1]] <- list()
-      for (z2 in seq_len(nrow(zones))) {
-        m[[z1]][[z2]] <- data * zones[z1, z2]
-      }
-    }
+    )
     # add penalties
-    internal_add_connectivity_penalties(x, penalty, m)
-})
+    internal_add_connectivity_penalties(x, penalty, as.matrix(zones), data)
+  }
+)
 
 #' @name add_connectivity_penalties
 #' @usage \S4method{add_connectivity_penalties}{ConservationProblem,ANY,ANY,array}(x, penalty, zones, data)
@@ -478,53 +512,89 @@ methods::setMethod("add_connectivity_penalties",
   methods::signature("ConservationProblem", "ANY", "ANY", "array"),
   function(x, penalty, zones, data) {
     # assert valid arguments
-    assertthat::assert_that(inherits(x, "ConservationProblem"),
-      assertthat::is.scalar(penalty), is.finite(penalty), is.null(zones),
-      is.array(data), length(dim(data)) == 4,
+    assert(
+      is_conservation_problem(x),
+      assertthat::is.number(penalty),
+      all_finite(penalty),
+      is.null(zones),
+      is.array(data),
+      length(dim(data)) == 4,
       dim(data)[1] == number_of_total_units(x),
       dim(data)[2] == number_of_total_units(x),
       dim(data)[3] == number_of_zones(x),
       dim(data)[4] == number_of_zones(x),
-      all(is.finite(data)))
-    # generate indices for units that are planning units
-    indices <- x$planning_unit_indices()
-    # convert array to list of list of sparseMatrix objects
-    m <- list()
-    for (z1 in seq_len(dim(data)[3])) {
-      m[[z1]] <- list()
-      for (z2 in seq_len(dim(data)[4])) {
-        m[[z1]][[z2]] <-
-          as_Matrix(data[indices, indices, z1, z2], "dgCMatrix")
-      }
-    }
+      all_finite(data)
+    )
     # add penalties
-    internal_add_connectivity_penalties (x, penalty, m)
-})
+    internal_add_connectivity_penalties (x, penalty, zones, data)
+  }
+)
 
-internal_add_connectivity_penalties <- function(x, penalty, data) {
+internal_add_connectivity_penalties <- function(x, penalty, zones, data) {
   # assert valid arguments
-  assertthat::assert_that(
-    inherits(x, "ConservationProblem"),
-    assertthat::is.scalar(penalty), is.finite(penalty),
-    is.list(data))
-    # create new penalty object
-    x$add_penalty(pproto(
+  assert(
+    is_conservation_problem(x),
+    assertthat::is.number(penalty),
+    all_finite(penalty),
+    .internal = TRUE
+  )
+  # create new penalty object
+  x$add_penalty(
+    R6::R6Class(
       "ConnectivityPenalty",
-      Penalty,
-      name = "Connectivity penalties",
-      data = list(data = data),
-      parameters = parameters(numeric_parameter("penalty", penalty)),
-      apply = function(self, x, y) {
-        assertthat::assert_that(inherits(x, "OptimizationProblem"),
-                                inherits(y, "ConservationProblem"))
-        # coerce to symmetric connectivity data
-        cm <- self$get_data("data")
-        cm <- lapply(cm, function(x) {
-          lapply(x, function(y) as_Matrix(Matrix::tril(y), "dgCMatrix"))
-        })
-        # apply penalties
-        rcpp_apply_connectivity_penalties(
-          x$ptr, self$parameters$get("penalty"), cm)
-        invisible(TRUE)
-    }))
+      inherit = Penalty,
+      public = list(
+        name = "connectivity penalties",
+        data = list(penalty = penalty, zones = zones, data = data),
+        apply = function(x, y) {
+          # assert valid arguments
+          assert(
+            inherits(x, "OptimizationProblem"),
+            inherits(y, "ConservationProblem"),
+            .internal = TRUE
+          )
+          # extract data
+          d <- self$get_data("data")
+          z <- self$get_data("zones")
+          indices <- y$planning_unit_indices()
+          # process data
+          m <- list()
+          if (inherits(d, "dgCMatrix")) {
+            ## if data is a dgCMatrix...
+            d <- d[indices, indices, drop = FALSE]
+            for (z1 in seq_len(ncol(z))) {
+              m[[z1]] <- list()
+              for (z2 in seq_len(nrow(z))) {
+                m[[z1]][[z2]] <- d * z[z1, z2]
+              }
+            }
+          } else if (inherits(d, "array")) {
+            ## if data is an array...
+            for (z1 in seq_len(dim(d)[3])) {
+              m[[z1]] <- list()
+              for (z2 in seq_len(dim(d)[4])) {
+                m[[z1]][[z2]] <- as_Matrix(
+                  d[indices, indices, z1, z2],
+                  "dgCMatrix"
+                )
+              }
+            }
+          } else {
+            ## throw error if not recognized
+            cli::cli_abort(
+              "Failed calculations for {.fn add_connectivity_penalties}.",
+              .internal = TRUE
+            )
+          }
+          # coerce to symmetric connectivity data
+          m <- lapply(m, function(x) {
+            lapply(x, function(y) as_Matrix(Matrix::tril(y), "dgCMatrix"))
+          })
+          # apply penalties
+          rcpp_apply_connectivity_penalties(x$ptr, self$get_data("penalty"), m)
+          invisible(TRUE)
+        }
+      )
+    )$new()
+  )
 }
