@@ -1,13 +1,13 @@
-#' @include internal.R Constraint-proto.R
+#' @include internal.R Constraint-class.R
 NULL
 
 #' Add contiguity constraints
 #'
-#' Add constraints to a conservation planning [problem()] to ensure
+#' Add constraints to a conservation planning problem to ensure
 #' that all selected planning units are spatially connected with each other
 #' and form a single contiguous unit.
 #'
-#' @param x [problem()] (i.e., [`ConservationProblem-class`]) object.
+#' @param x [problem()] object.
 #'
 #' @param zones `matrix` or `Matrix` object describing the
 #'   connection scheme for different zones. Each row and column corresponds
@@ -57,12 +57,12 @@ NULL
 #'   matrix diagonal have no effect on the solution at all because each
 #'   planning unit cannot be a connected with itself.}
 #'
-#' \item{`data` as a `data.frame` object}{containing the fields (columns)
+#' \item{`data` as a `data.frame` object}{containing columns that are named
 #'   `"id1"`, `"id2"`, and `"boundary"`. Here, each row
 #'   denotes the connectivity between two planning units following the
-#'   *Marxan* format. The field `boundary` should contain
+#'   *Marxan* format. The `"boundary"` column should contain
 #'   binary `numeric` values that indicate if the two planning units
-#'   specified in the fields `"id1"` and `"id2"` are connected
+#'   specified in the `"id1"` and `"id2"` columns are connected
 #'   or not. This data can be used to describe symmetric or
 #'   asymmetric relationships between planning units. By default,
 #'   input data is assumed to be symmetric unless asymmetric data is
@@ -76,8 +76,7 @@ NULL
 #' In early versions, this function was named as the
 #' `add_connected_constraints()` function.
 #'
-#' @return Object (i.e., [`ConservationProblem-class`]) with the constraints
-#'  added to it.
+#' @return An updated [problem()] object with the constraints added to it.
 #'
 #' @seealso
 #' See [constraints] for an overview of all functions for adding constraints.
@@ -93,32 +92,37 @@ NULL
 #' @examples
 #' \dontrun{
 #' # load data
-#' data(sim_pu_raster, sim_features, sim_pu_zones_stack, sim_features_zones)
+#' sim_pu_raster <- get_sim_pu_raster()
+#' sim_features <- get_sim_features()
+#' sim_zones_pu_raster <- get_sim_zones_pu_raster()
+#' sim_zones_features <- get_sim_zones_features()
 #'
 #' # create minimal problem
-#' p1 <- problem(sim_pu_raster, sim_features) %>%
-#'       add_min_set_objective() %>%
-#'       add_relative_targets(0.2) %>%
-#'       add_binary_decisions() %>%
-#'       add_default_solver(verbose = FALSE)
+#' p1 <-
+#'   problem(sim_pu_raster, sim_features) %>%
+#'   add_min_set_objective() %>%
+#'   add_relative_targets(0.2) %>%
+#'   add_binary_decisions() %>%
+#'   add_default_solver(verbose = FALSE)
 #'
 #' # create problem with added connected constraints
 #' p2 <- p1 %>% add_contiguity_constraints()
 #'
 #' # solve problems
-#' s <- stack(solve(p1), solve(p2))
+#' s1 <- c(solve(p1), solve(p2))
+#' names(s1) <- c("basic solution", "connected solution")
 #'
 #' # plot solutions
-#' plot(s, main = c("basic solution", "connected solution"), axes = FALSE,
-#'      box = FALSE)
+#' plot(s1, axes = FALSE)
 #'
 #' # create minimal problem with multiple zones, and limit the solver to
 #' # 30 seconds to obtain solutions in a feasible period of time
-#' p3 <- problem(sim_pu_zones_stack, sim_features_zones) %>%
-#'       add_min_set_objective() %>%
-#'       add_relative_targets(matrix(0.2, ncol = 3, nrow = 5)) %>%
-#'       add_binary_decisions() %>%
-#'       add_default_solver(time_limit = 30, verbose = FALSE)
+#' p3 <-
+#'   problem(sim_zones_pu_raster, sim_zones_features) %>%
+#'   add_min_set_objective() %>%
+#'   add_relative_targets(matrix(0.2, ncol = 3, nrow = 5)) %>%
+#'   add_binary_decisions() %>%
+#'   add_default_solver(time_limit = 30, verbose = FALSE)
 #'
 #' # create problem with added constraints to ensure that the planning units
 #' # allocated to each zone form a separate contiguous unit
@@ -146,41 +150,45 @@ NULL
 #' # solve problems
 #' s2 <- lapply(list(p3, p4, p5, p6), solve)
 #' s2 <- lapply(s2, category_layer)
-#' s2 <- stack(s2)
+#' s2 <- terra::rast(s2)
+#' names(s2) <- c("basic solution", "p4", "p5", "p6")
 #'
 #' # plot solutions
-#' plot(s2, axes = FALSE, box = FALSE,
-#'      main = c("basic solution", "p4", "p5", "p6"))
+#' plot(s2, axes = FALSE)
 #'
 #' # create a problem that has a main "reserve zone" and a secondary
 #' # "corridor zone" to connect up import areas. Here, each feature has a
-#' # target of 30% of its distribution. If a planning unit is allocated to the
+#' # target of 50% of its distribution. If a planning unit is allocated to the
 #' # "reserve zone", then the prioritization accrues 100% of the amount of
 #' # each feature in the planning unit. If a planning unit is allocated to the
 #' # "corridor zone" then the prioritization accrues 40% of the amount of each
 #' # feature in the planning unit. Also, the cost of managing a planning unit
-#' # in the "corridor zone" is 45% of that when it is managed as the
+#' # in the "corridor zone" is 30% of that when it is managed as the
 #' # "reserve zone". Finally, the problem has constraints which
 #' # ensure that all of the selected planning units form a single contiguous
 #' # unit, so that the planning units allocated to the "corridor zone" can
 #' # link up the planning units allocated to the "reserve zone"
 #'
 #' # create planning unit data
-#' pus <- sim_pu_zones_stack[[c(1, 1)]]
-#' pus[[2]] <- pus[[2]] * 0.45
+#' pus <- sim_zones_pu_raster[[c(1, 1)]]
+#' pus[[2]] <- pus[[2]] * 0.3
 #' print(pus)
 #'
 #' # create biodiversity data
-#' fts <- zones(sim_features, sim_features * 0.4,
-#'              feature_names = names(sim_features),
-#'              zone_names = c("reserve zone", "corridor zone"))
+#' fts <- zones(
+#'   sim_features, sim_features * 0.4,
+#'   feature_names = names(sim_features),
+#'   zone_names = c("reserve zone", "corridor zone")
+#' )
 #' print(fts)
 #'
 #' # create targets
-#' targets <- tibble::tibble(feature = names(sim_features),
-#'                           zone = list(zone_names(fts))[rep(1, 5)],
-#'                           target = cellStats(sim_features, "sum") * 0.2,
-#'                           type = rep("absolute", 5))
+#' targets <- tibble::tibble(
+#'   feature = names(sim_features),
+#'   zone = list(zone_names(fts))[rep(1, 5)],
+#'   target = terra::global(sim_features, "sum", na.rm = TRUE)[[1]] * 0.5,
+#'   type = rep("absolute", 5)
+#' )
 #' print(targets)
 #'
 #' # create zones matrix
@@ -188,18 +196,19 @@ NULL
 #' print(z7)
 #'
 #' # create problem
-#' p7 <- problem(pus, fts) %>%
-#'       add_min_set_objective() %>%
-#'       add_manual_targets(targets) %>%
-#'       add_contiguity_constraints(z7) %>%
-#'       add_binary_decisions() %>%
-#'       add_default_solver(verbose = FALSE)
+#' p7 <-
+#'   problem(pus, fts) %>%
+#'   add_min_set_objective() %>%
+#'   add_manual_targets(targets) %>%
+#'   add_contiguity_constraints(z7) %>%
+#'   add_binary_decisions() %>%
+#'   add_default_solver(verbose = FALSE)
 #'
 #' # solve problems
 #' s7 <- category_layer(solve(p7))
 #'
 #' # plot solutions
-#' plot(s7, "solution", axes = FALSE, box = FALSE)
+#' plot(s7, main = "solution", axes = FALSE)
 #' }
 #' @name add_contiguity_constraints
 #'
@@ -210,8 +219,20 @@ NULL
 
 methods::setGeneric("add_contiguity_constraints",
   signature = methods::signature("x", "zones", "data"),
-  function(x, zones = diag(number_of_zones(x)), data = NULL)
-  standardGeneric("add_contiguity_constraints"))
+  function(x, zones = diag(number_of_zones(x)), data = NULL) {
+    rlang::check_required(x)
+    rlang::check_required(zones)
+    rlang::check_required(data)
+    assert(
+      is_conservation_problem(x),
+      is_inherits(
+        data,
+        c("NULL",  "dgCMatrix", "data.frame", "matrix", "Matrix")
+      )
+    )
+    standardGeneric("add_contiguity_constraints")
+  }
+)
 
 #' @name add_contiguity_constraints
 #' @usage \S4method{add_contiguity_constraints}{ConservationProblem,ANY,ANY}(x, zones, data)
@@ -220,79 +241,110 @@ methods::setMethod("add_contiguity_constraints",
   methods::signature("ConservationProblem", "ANY", "ANY"),
   function(x, zones, data) {
     # assert valid arguments
-    assertthat::assert_that(inherits(x, "ConservationProblem"),
-     inherits(zones, c("matrix", "Matrix")),
-     inherits(data, c("NULL", "Matrix")))
+    assert(
+      is_conservation_problem(x),
+      is_matrix_ish(zones)
+    )
     if (!is.null(data)) {
       # check argument to data if not NULL
+      assert(is_matrix_ish(data))
       data <- as_Matrix(data, "dgCMatrix")
-      assertthat::assert_that(all(data@x %in% c(0, 1)),
-        ncol(data) == nrow(data), number_of_total_units(x) == ncol(data),
-        all(is.finite(data@x)), Matrix::isSymmetric(data))
-      d <- list(matrix = data)
+      assert(
+        is_numeric_values(data),
+        all_finite(data),
+        all_binary(data),
+        ncol(data) == nrow(data),
+        number_of_total_units(x) == ncol(data),
+        Matrix::isSymmetric(data)
+      )
     } else {
       # check that planning unit data is spatially referenced
-      assertthat::assert_that(inherits(x$data$cost,
-                                       c("Spatial", "Raster", "sf")),
-        msg = paste("argument to data must be supplied because planning unit",
-                    "data are not in a spatially referenced format"))
-      d <- list()
+      assert(
+        is_pu_spatially_explicit(x),
+        msg = c(
+          paste(
+            "{.arg data} must be manually specified (e.g., as a {.cls Matrix})."
+          ),
+          "i" = paste(
+            "This is because {.arg x} has planning unit data that are not",
+            "spatially explicit",
+            "(e.g., {.cls sf}, or {.cls SpatRaster} objects)."
+          )
+        )
+      )
     }
     # convert zones to matrix
     zones <- as.matrix(zones)
-    assertthat::assert_that(
-      isSymmetric(zones), ncol(zones) == number_of_zones(x),
-      is.numeric(zones), all(zones %in% c(0, 1)),
-      all(colMeans(zones) <= diag(zones)), all(rowMeans(zones) <= diag(zones)))
+    assert(
+      is.matrix(zones),
+      isSymmetric(zones),
+      ncol(zones) == number_of_zones(x),
+      is_numeric_values(zones),
+      all_finite(zones),
+      all_binary(zones),
+      all(colMeans(zones) <= diag(zones)),
+      all(rowMeans(zones) <= diag(zones))
+    )
     colnames(zones) <- x$zone_names()
     rownames(zones) <- colnames(zones)
     # add constraints
-    x$add_constraint(pproto(
-      "ContiguityConstraint",
-      Constraint,
-      data = d,
-      name = "Contiguity constraints",
-      parameters = parameters(
-        binary_parameter("apply constraints?", 1L),
-        binary_matrix_parameter("zones", zones, symmetric = TRUE)),
-      calculate = function(self, x) {
-        assertthat::assert_that(inherits(x, "ConservationProblem"))
-        # generate matrix if null
-        if (is.Waiver(self$get_data("matrix"))) {
-          # create matrix
-          data <- adjacency_matrix(x$data$cost)
-          # coerce matrix to full matrix
-          data <- as_Matrix(data, "dgCMatrix")
-          # store data
-          self$set_data("matrix", data)
-        }
-        # return success
-        invisible(TRUE)
-      },
-      apply = function(self, x, y) {
-        assertthat::assert_that(inherits(x, "OptimizationProblem"),
-          inherits(y, "ConservationProblem"))
-        if (as.logical(self$parameters$get("apply constraints?")[[1]])) {
-          # extract data and parameters
-          ind <- y$planning_unit_indices()
-          d <- self$get_data("matrix")[ind, ind, drop = FALSE]
-          z <- self$parameters$get("zones")
-          # extract clusters from z
-          z_cl <- igraph::graph_from_adjacency_matrix(z, diag = FALSE,
-            mode = "undirected", weighted = NULL)
-          z_cl <-  igraph::clusters(z_cl)$membership
-          # set cluster memberships to zero if constraints not needed
-          z_cl <- z_cl * diag(z)
-          # convert d to lower triangle sparse matrix
-          d <- Matrix::forceSymmetric(d, uplo = "L")
-          class(d) <- "dgCMatrix"
-          # apply constraints if any zones have contiguity constraints
-          if (max(z_cl) > 0)
-            rcpp_apply_contiguity_constraints(x$ptr, d, z_cl)
-        }
-        invisible(TRUE)
-      }))
-})
+    x$add_constraint(
+      R6::R6Class(
+        "ContiguityConstraint",
+        inherit = Constraint,
+        public = list(
+          name = "contiguity constraints",
+          data = list(data = data, zones = zones),
+          calculate = function(x) {
+            # assert valid arguments
+            assert(is_conservation_problem(x), .internal = TRUE)
+            # if needed, calculate adjacency matrix
+            if (
+              is.null(self$get_data("data")) &&
+              is.Waiver(x$get_data("adjacency"))
+            ) {
+              x$set_data("adjacency", adjacency_matrix(x$data$cost))
+            }
+            # return success
+            invisible(TRUE)
+          },
+          apply = function(x, y) {
+            assert(
+              inherits(x, "OptimizationProblem"),
+              inherits(y, "ConservationProblem"),
+              .internal = TRUE
+            )
+            # extract data
+            z <- self$get_data("zones")
+            d <- self$get_data("data")
+            if (is.Waiver(d) || is.null(d)) {
+              d <- y$get_data("adjacency")
+            }
+            # convert to dgCMatrix
+            d <- as_Matrix(d, "dgCMatrix")
+            # subset data by planning unit indices
+            ind <- y$planning_unit_indices()
+            d <- d[ind, ind, drop = FALSE]
+            # extract clusters from z
+            z_cl <- igraph::graph_from_adjacency_matrix(
+              z, diag = FALSE, mode = "undirected", weighted = NULL
+            )
+            z_cl <- igraph::clusters(z_cl)$membership
+            # set cluster memberships to zero if constraints not needed
+            z_cl <- z_cl * diag(z)
+            # convert d to lower triangle sparse matrix
+            d <- Matrix::forceSymmetric(d, uplo = "L")
+            d <- as_Matrix(Matrix::tril(d), "dgCMatrix")
+            # apply constraints if any zones have contiguity constraints
+            if (max(z_cl) > 0) rcpp_apply_contiguity_constraints(x$ptr, d, z_cl)
+            # return invisible
+            invisible(TRUE)
+          }
+        )
+      )$new()
+    )
+  }
+)
 
 #' @name add_contiguity_constraints
 #' @usage \S4method{add_contiguity_constraints}{ConservationProblem,ANY,data.frame}(x, zones, data)
@@ -301,12 +353,20 @@ methods::setMethod("add_contiguity_constraints",
   methods::signature("ConservationProblem", "ANY", "data.frame"),
   function(x, zones, data) {
     # assert that does not have zone1 and zone2 columns
-    assertthat::assert_that(inherits(data, "data.frame"),
+    assert(
+      is.data.frame(data),
+      assertthat::has_name(data, "id1"),
+      assertthat::has_name(data, "id2"),
+      assertthat::has_name(data, "boundary"),
       !assertthat::has_name(data, "zone1"),
-      !assertthat::has_name(data, "zone2"))
+      !assertthat::has_name(data, "zone2")
+    )
     # add constraints
-    add_contiguity_constraints(x, zones, marxan_boundary_data_to_matrix(x, data))
-})
+    add_contiguity_constraints(
+      x, zones, marxan_connectivity_data_to_matrix(x, data, TRUE)
+    )
+  }
+)
 
 #' @name add_contiguity_constraints
 #' @usage \S4method{add_contiguity_constraints}{ConservationProblem,ANY,matrix}(x, zones, data)
@@ -316,4 +376,5 @@ methods::setMethod("add_contiguity_constraints",
   function(x, zones, data) {
     # add constraints
     add_contiguity_constraints(x, zones, as_Matrix(data, "dgCMatrix"))
-})
+  }
+)
