@@ -1,9 +1,9 @@
-#' @include internal.R Constraint-proto.R
+#' @include internal.R Constraint-class.R
 NULL
 
 #' Add feature contiguity constraints
 #'
-#' Add constraints to a [problem()] to ensure that each feature is
+#' Add constraints to a problem to ensure that each feature is
 #' represented in a contiguous unit of dispersible habitat. These constraints
 #' are a more advanced version of those implemented in the
 #' [add_contiguity_constraints()] function, because they ensure that
@@ -13,7 +13,7 @@ NULL
 #' feature to ensure that all features can disperse through out the areas
 #' designated for their conservation.
 #'
-#' @param x [problem()] (i.e., [`ConservationProblem-class`]) object.
+#' @param x [problem()] object.
 #'
 #' @param zones `matrix`, `Matrix` or `list` object describing
 #'   the connection scheme for different zones. For `matrix` or
@@ -85,12 +85,12 @@ NULL
 #'   of connected planning units are treated as being potentially dispersible
 #'   for all features.}
 #'
-#' \item{`data` as a `data.frame` object}{containing the fields (columns)
+#' \item{`data` as a `data.frame` object}{containing columns that are named
 #'   `"id1"`, `"id2"`, and `"boundary"`. Here, each row
 #'   denotes the connectivity between two planning units following the
-#'   *Marxan* format. The field `boundary` should contain
+#'   *Marxan* format. The `"boundary"` column should contain
 #'   binary `numeric` values that indicate if the two planning units
-#'   specified in the fields `"id1"` and `"id2"` are connected
+#'   specified in the `"id1"` and `"id2"` columns are connected
 #'   or not. This data can be used to describe symmetric or
 #'   asymmetric relationships between planning units. By default,
 #'   input data is assumed to be symmetric unless asymmetric data is
@@ -137,14 +137,18 @@ NULL
 #' @examples
 #' \dontrun{
 #' # load data
-#' data(sim_pu_raster, sim_pu_zones_stack, sim_features, sim_features_zones)
+#' sim_pu_raster <- get_sim_pu_raster()
+#' sim_features <- get_sim_features()
+#' sim_zones_pu_raster <- get_sim_zones_pu_raster()
+#' sim_zones_features <- get_sim_zones_features()
 #'
 #' # create minimal problem
-#' p1 <- problem(sim_pu_raster, sim_features) %>%
-#'       add_min_set_objective() %>%
-#'       add_relative_targets(0.3) %>%
-#'       add_binary_decisions() %>%
-#'       add_default_solver(verbose = FALSE)
+#' p1 <-
+#'   problem(sim_pu_raster, sim_features) %>%
+#'   add_min_set_objective() %>%
+#'   add_relative_targets(0.3) %>%
+#'   add_binary_decisions() %>%
+#'   add_default_solver(verbose = FALSE)
 #'
 #' # create problem with contiguity constraints
 #' p2 <- p1 %>% add_contiguity_constraints()
@@ -155,12 +159,12 @@ NULL
 #'
 #' # create problem with constraints to represent features in contiguous
 #' # units that contain highly suitable habitat values
-#' # (specifically in the top 1.5th percentile)
-#' cm4 <- lapply(seq_len(nlayers(sim_features)), function(i) {
+#' # (specifically in the top 5th percentile)
+#' cm4 <- lapply(seq_len(terra::nlyr(sim_features)), function(i) {
 #'   # create connectivity matrix using the i'th feature's habitat data
 #'   m <- connectivity_matrix(sim_pu_raster, sim_features[[i]])
-#'   # convert matrix to 0/1 values denoting values in top 20th percentile
-#'   m <- round(m > quantile(as.vector(m), 1 - 0.015, names = FALSE))
+#'   # convert matrix to 0/1 values denoting values in top 5th percentile
+#'   m <- round(m > quantile(as.vector(m), 1 - 0.05, names = FALSE))
 #'   # remove 0s from the sparse matrix
 #'   m <- Matrix::drop0(m)
 #'   # return matrix
@@ -169,21 +173,23 @@ NULL
 #' p4 <- p1 %>% add_feature_contiguity_constraints(data = cm4)
 #'
 #' # solve problems
-#' s1 <- stack(solve(p1), solve(p2), solve(p3), solve(p4))
-#'
+#' s1 <- c(solve(p1), solve(p2), solve(p3), solve(p4))
+#' names(s1) <- c(
+#'   "basic solution", "contiguity constraints",
+#'   "feature contiguity constraints",
+#'   "feature contiguity constraints with data"
+#' )
 #' # plot solutions
-#' plot(s1,  axes = FALSE, box = FALSE,
-#'      main = c("basic solution", "contiguity constraints",
-#'               "feature contiguity constraints",
-#'               "feature contiguity constraints with data"))
+#' plot(s1, axes = FALSE)
 #'
 #' # create minimal problem with multiple zones, and limit the solver to
 #' # 30 seconds to obtain solutions in a feasible period of time
-#' p5 <- problem(sim_pu_zones_stack, sim_features_zones) %>%
-#'       add_min_set_objective() %>%
-#'       add_relative_targets(matrix(0.1, ncol = 3, nrow = 5)) %>%
-#'       add_binary_decisions() %>%
-#'       add_default_solver(time_limit = 30, verbose = FALSE)
+#' p5 <-
+#'   problem(sim_zones_pu_raster, sim_zones_features) %>%
+#'   add_min_set_objective() %>%
+#'   add_relative_targets(matrix(0.1, ncol = 3, nrow = 5)) %>%
+#'   add_binary_decisions() %>%
+#'   add_default_solver(time_limit = 30, verbose = FALSE)
 #'
 #' # create problem with contiguity constraints that specify that the
 #' # planning units used to conserve each feature in different management
@@ -204,17 +210,19 @@ NULL
 #' # the planning units in all three of the zones must conserve first feature
 #' # in a single contiguous unit but the planning units used to conserve the
 #' # remaining features do not need to be contiguous in any way
-#' zm8 <- lapply(seq_len(number_of_features(sim_features_zones)), function(i)
-#'   matrix(ifelse(i == 1, 1, 0), ncol = 3, nrow = 3))
+#' zm8 <- lapply(
+#'   seq_len(number_of_features(sim_zones_features)),
+#'   function(i) matrix(ifelse(i == 1, 1, 0), ncol = 3, nrow = 3)
+#' )
 #' print(zm8)
 #' p8 <- p5 %>% add_feature_contiguity_constraints(zm8)
 #'
 #' # solve problems
 #' s2 <- lapply(list(p5, p6, p7, p8), solve)
-#' s2 <- stack(lapply(s2, category_layer))
-#'
+#' s2 <- terra::rast(lapply(s2, category_layer))
+#' names(s2) <- c("p5", "p6", "p7", "p8")
 #' # plot solutions
-#' plot(s2, main = c("p5", "p6", "p7", "p8"), axes = FALSE, box = FALSE)
+#' plot(s2, axes = FALSE)
 #' }
 #' @name add_feature_contiguity_constraints
 #'
@@ -225,23 +233,20 @@ NULL
 
 methods::setGeneric("add_feature_contiguity_constraints",
   signature = methods::signature("x", "zones", "data"),
-  function(x, zones = diag(number_of_zones(x)), data = NULL)
-  standardGeneric("add_feature_contiguity_constraints"))
-
-#' @name add_feature_contiguity_constraints
-#' @usage \S4method{add_feature_contiguity_constraints}{ConservationProblem,ANY,Matrix}(x, zones, data)
-#' @rdname add_feature_contiguity_constraints
-methods::setMethod("add_feature_contiguity_constraints",
-  methods::signature("ConservationProblem", "ANY", "Matrix"),
-  function(x, zones, data) {
-    # assert valid arguments
-    assertthat::assert_that(inherits(x, "ConservationProblem"),
-     inherits(zones, c("matrix", "Matrix", "list")),
-     inherits(data, c("matrix", "Matrix")))
-    # apply constraints
-    data <- list(data)[rep(1, number_of_features(x))]
-    add_feature_contiguity_constraints(x, zones, data)
-})
+  function(x, zones = diag(number_of_zones(x)), data = NULL) {
+    rlang::check_required(x)
+    rlang::check_required(zones)
+    rlang::check_required(data)
+    assert(
+      is_conservation_problem(x),
+      is_inherits(
+        data,
+        c("NULL", "dgCMatrix", "matrix", "Matrix", "list")
+      )
+    )
+    standardGeneric("add_feature_contiguity_constraints")
+  }
+)
 
 #' @name add_feature_contiguity_constraints
 #' @usage \S4method{add_feature_contiguity_constraints}{ConservationProblem,ANY,data.frame}(x, zones, data)
@@ -250,14 +255,15 @@ methods::setMethod("add_feature_contiguity_constraints",
   methods::signature("ConservationProblem", "ANY", "data.frame"),
   function(x, zones, data) {
     # assert valid arguments
-    assertthat::assert_that(inherits(x, "ConservationProblem"),
-     inherits(zones, c("matrix", "Matrix", "list")),
-     inherits(data, "data.frame"))
+    assert(
+      is_conservation_problem(x),
+      is_inherits(zones, c("matrix", "Matrix", "list")),
+      is.data.frame(data)
+    )
     # apply constraints
-    data <- list(data)[rep(1, number_of_features(x))]
     add_feature_contiguity_constraints(x, zones, data)
-
-})
+  }
+)
 
 #' @name add_feature_contiguity_constraints
 #' @usage \S4method{add_feature_contiguity_constraints}{ConservationProblem,ANY,matrix}(x, zones, data)
@@ -267,7 +273,8 @@ methods::setMethod("add_feature_contiguity_constraints",
   function(x, zones, data) {
     # add constraints
     add_feature_contiguity_constraints(x, zones, as_Matrix(data, "dgCMatrix"))
-})
+  }
+)
 
 #' @name add_feature_contiguity_constraints
 #' @usage \S4method{add_feature_contiguity_constraints}{ConservationProblem,ANY,ANY}(x, zones, data)
@@ -276,111 +283,169 @@ methods::setMethod("add_feature_contiguity_constraints",
   methods::signature("ConservationProblem", "ANY", "ANY"),
   function(x, zones, data) {
     # assert valid arguments
-    assertthat::assert_that(inherits(x, "ConservationProblem"),
-      inherits(zones, c("matrix", "Matrix", "list")),
-      inherits(data, c("NULL", "list")))
+    assert(
+      is_conservation_problem(x),
+      is_inherits(zones, c("matrix", "Matrix", "list")),
+      is_inherits(data, c("NULL", "matrix", "Matrix", "data.frame", "list"))
+    )
     # format zones
-    if (inherits(zones, c("matrix", "Matrix"))) {
-      zones <- list(zones)[rep(1, x$number_of_features())]
+    if (inherits(zones, "list")) {
+      assert(
+        length(zones) == number_of_features(x),
+        all_elements_inherit(zones, c("matrix", "Matrix"))
+      )
+      names(zones) <- paste(x$feature_names(), "zones")
+      for (i in seq_along(zones)) {
+        zones[[i]] <- as.matrix(zones[[i]])
+        assert(
+          is.numeric(zones[[i]]),
+          all_binary(zones[[i]]),
+          all_finite(zones[[i]]),
+          isSymmetric(zones[[i]]),
+          ncol(zones[[i]]) == number_of_zones(x),
+          all(colMeans(zones[[i]]) <= diag(zones[[i]])),
+          all(rowMeans(zones[[i]]) <= diag(zones[[i]]))
+        )
+        colnames(zones[[i]]) <- x$zone_names()
+        rownames(zones[[i]]) <- colnames(zones[[i]])
+      }
     } else {
-      assertthat::assert_that(length(zones) == number_of_features(x),
-        all(vapply(zones, inherits, logical(1), c("matrix", "Matrix"))))
+      assert(
+        is_matrix_ish(zones),
+        all_binary(zones),
+        all_finite(zones),
+        isSymmetric(zones),
+        ncol(zones) == number_of_zones(x),
+        all(colMeans(zones) <= diag(zones)),
+        all(rowMeans(zones) <= diag(zones))
+      )
+      colnames(zones) <- x$zone_names()
+      rownames(zones) <- colnames(zones)
     }
     # format data
-    if (!is.null(data)) {
-      # check argument to data if not NULL
+    if (is.list(data)) {
       for (i in seq_along(data)) {
         # assert that element is valid
-        assertthat::assert_that(
-          inherits(data[[i]], c("matrix", "Matrix", "data.frame")),
-          msg = paste0("argument to data[[", i, "]] is not a matrix, Matrix",
-            "or data.frame"))
-        # coerce to correct format
+        assert(
+          is_inherits(data[[i]], c("matrix", "Matrix", "data.frame")),
+          msg = paste(
+            "{.arg data[[", i, "]]} is not a {.cls matrix}, {.cls Matrix}",
+            "or a data frame."
+          )
+        )
+        # coerce data to dgCMatrix
         if (is.matrix(data[[i]]))
           data[[i]] <- as_Matrix(data, "dgCMatrix")
         if (is.data.frame(data[[i]]))
-          data[[i]] <- marxan_boundary_data_to_matrix(x, data[[i]])
+          data[[i]] <- marxan_connectivity_data_to_matrix(x, data[[i]], TRUE)
         # run checks
-        assertthat::assert_that(all(data[[i]]@x %in% c(0, 1)),
+        assert(
+          all_binary(data[[i]]),
+          all_finite(data[[i]]),
           ncol(data[[i]]) == nrow(data[[i]]),
           number_of_total_units(x) == ncol(data[[i]]),
-          all(is.finite(data[[i]]@x)), Matrix::isSymmetric(data[[i]]))
+          Matrix::isSymmetric(data[[i]])
+        )
       }
-      # create list with data
-      d <- list(matrices = data)
-    } else {
-      # check that planning unit data is spatially referenced
-      assertthat::assert_that(inherits(x$data$cost,
-                                       c("Spatial", "Raster", "sf")),
-        msg = paste("argument to data must be supplied because planning unit",
-                    "data are not in a spatially referenced format"))
-      d <- list()
-    }
-    # convert zones to matrix
-    for (i in seq_along(zones)) {
-      zones[[i]] <- as.matrix(zones[[i]])
-      assertthat::assert_that(
-        isSymmetric(zones[[i]]), ncol(zones[[i]]) == number_of_zones(x),
-        is.numeric(zones[[i]]), all(zones[[i]] %in% c(0, 1)),
-        all(colMeans(zones[[i]]) <= diag(zones[[i]])),
-        all(rowMeans(zones[[i]]) <= diag(zones[[i]])))
-      colnames(zones[[i]]) <- x$zone_names()
-      rownames(zones[[i]]) <- colnames(zones[[i]])
-    }
-    # create list of parameters
-    p <- lapply(seq_along(zones), function(i) {
-      binary_matrix_parameter(paste(x$feature_names()[i], "zones"),
-                              zones[[i]], symmetric = TRUE)
-    })
-    # add constraint
-    x$add_constraint(pproto(
-    "FeatureContiguityConstraint",
-    Constraint,
-    name = "Feature contiguity constraints",
-    compressed_formulation = FALSE,
-    parameters = do.call(parameters, append(
-      list(binary_parameter("apply constraints?", 1L)), p)),
-    data = d,
-    calculate = function(self, x) {
-      # generate connectivity data
-      if (is.Waiver(self$get_data("matrices"))) {
-        # create matrix
-        data <- adjacency_matrix(x$data$cost)
-        # coerce matrix to full matrix
+    } else if (inherits(data, c("matrix", "dgCMatrix", "data.frame"))) {
+      # coerce data to dgCMatrix
+      if (is.matrix(data))
         data <- as_Matrix(data, "dgCMatrix")
-        # create list for each feature
-        data <- list(data)[rep(1, number_of_features(x))]
-        # store data
-        self$set_data("matrices", data)
-      }
-      # return success
-      invisible(TRUE)
-    },
-    apply = function(self, x, y) {
-      assertthat::assert_that(inherits(x, "OptimizationProblem"),
-        inherits(y, "ConservationProblem"))
-      if (as.logical(self$parameters$get("apply constraints?"))) {
-        # extract list of connectivity matrices
-        ind <- y$planning_unit_indices()
-        # format matrices
-        d <- self$get_data("matrices")
-        d <- lapply(d, `[`, ind, ind, drop = FALSE)
-        # extract clusters from z
-        z <- list()
-        for (i in seq_len(y$number_of_features()))
-        z[[i]] <- self$parameters$get(paste(y$feature_names()[i], "zones"))
-        z_cl <- lapply(seq_along(z), function(i) {
-          igraph::clusters(igraph::graph_from_adjacency_matrix(z[[i]],
-            diag = FALSE, mode = "undirected", weighted = NULL))$membership *
-            diag(z[[i]])
-        })
-        # convert d to lower triangle sparse matrix
-        d <- lapply(d, Matrix::forceSymmetric, uplo = "L")
-        d <- lapply(d, Matrix::tril)
-        d <- lapply(d, as_Matrix, "dgCMatrix")
-        # apply the constraints
-        if (max(vapply(z_cl, max, numeric(1))) > 0)
-          rcpp_apply_feature_contiguity_constraints(x$ptr, d, z_cl)
-      }
-    }))
-})
+      if (is.data.frame(data))
+        data <- marxan_connectivity_data_to_matrix(x, data[[i]], TRUE)
+      # run checks
+      assert(
+        all_binary(data),
+        all_finite(data),
+        ncol(data) == nrow(data),
+        number_of_total_units(x) == ncol(data),
+        Matrix::isSymmetric(data)
+      )
+    } else if (is.null(data)) {
+      # check that planning unit data is spatially explicit
+      assert(
+        is_pu_spatially_explicit(x),
+        msg =
+        c(
+          paste(
+            "{.arg data} must be manually specified (e.g., as a Matrix)."
+          ),
+          "i" = paste(
+            "This is because {.arg x} has planning unit data that are not",
+            "spatially explicit",
+            "(e.g., {.cls sf}, or {.cls SpatRaster} objects)."
+          )
+        )
+      )
+    } else {
+      cli::cli_abort("{.arg data} is not a recognized class.")
+    }
+    # add constraint
+    x$add_constraint(
+      R6::R6Class(
+        "FeatureContiguityConstraint",
+        inherit = Constraint,
+        public = list(
+          name = "feature contiguity constraints",
+          compressed_formulation = FALSE,
+          data = list(data = data, zones = zones),
+          calculate = function(x) {
+            # generate connectivity data
+            d <- self$get_data("data")
+            if (is.null(d)) {
+              if (is.Waiver(x$get_data("adjacency"))) {
+                data <- adjacency_matrix(x$data$cost)
+                data <- as_Matrix(data, "dgCMatrix")
+                x$set_data("adjacency", data)
+              }
+            }
+            # return success
+            invisible(TRUE)
+          },
+          apply = function(x, y) {
+            assert(
+              inherits(x, "OptimizationProblem"),
+              inherits(y, "ConservationProblem"),
+              .internal = TRUE
+            )
+            # extract data
+            zn <- self$get_data("zones")
+            d <- self$get_data("data")
+            if (is.null(d)) {
+              d <- y$get_data("adjacency")
+            }
+            # convert data to list format if needed
+            if (!is.list(d)) {
+              d <- list(d)[rep(1, number_of_features(y))]
+            }
+            if (!is.list(zn)) {
+              zn <- list(zn)[rep(1, number_of_features(y))]
+              names(zn) <- paste(y$feature_names(), "zones")
+            }
+            # convert to dgCMatrix
+            d <- lapply(d, as_Matrix, "dgCMatrix")
+            # subset matrix data to indices
+            ind <- y$planning_unit_indices()
+            d <- lapply(d, `[`, ind, ind, drop = FALSE)
+            # extract clusters from z
+            z_cl <- lapply(seq_along(zn), function(i) {
+              igraph::clusters(
+                igraph::graph_from_adjacency_matrix(
+                  zn[[i]], diag = FALSE, mode = "undirected", weighted = NULL
+                )
+              )$membership * diag(zn[[i]])
+            })
+            # convert d to lower triangle sparse matrix
+            d <- lapply(d, Matrix::forceSymmetric, uplo = "L")
+            d <- lapply(d, Matrix::tril)
+            d <- lapply(d, as_Matrix, "dgCMatrix")
+            # apply the constraints
+            if (max(vapply(z_cl, max, numeric(1))) > 0) {
+              rcpp_apply_feature_contiguity_constraints(x$ptr, d, z_cl)
+            }
+          }
+        )
+      )$new()
+    )
+  }
+)
