@@ -3,7 +3,7 @@ NULL
 
 #' Add linear constraints
 #'
-#' Add constraints to a conservation planning [problem()] to ensure
+#' Add constraints to a conservation planning problem to ensure
 #' that all selected planning units meet certain criteria.
 #'
 #' @inheritParams add_contiguity_constraints
@@ -16,7 +16,7 @@ NULL
 #'   per integer programming terminology.
 #'
 #' @param data `character`, `numeric`,
-#'   [`Raster-class`], `matrix`, or `Matrix` object
+#'   [terra::rast()], `matrix`, or `Matrix` object
 #'   containing the constraint values.
 #'   These constraint values are also known as constraint coefficients
 #'   per integer programming terminology.
@@ -68,20 +68,22 @@ NULL
 #' @examples
 #' \dontrun{
 #' # load data
-#' data(sim_pu_raster, sim_features)
+#' sim_pu_raster <- get_sim_pu_raster()
+#' sim_features <- get_sim_features()
 #'
 #' # create a baseline problem with minimum shortfall objective
-#' p0 <- problem(sim_pu_raster, sim_features) %>%
-#'       add_min_shortfall_objective(1800) %>%
-#'       add_relative_targets(0.2) %>%
-#'       add_binary_decisions() %>%
-#'       add_default_solver(verbose = FALSE)
+#' p0 <-
+#'   problem(sim_pu_raster, sim_features) %>%
+#'   add_min_shortfall_objective(1800) %>%
+#'   add_relative_targets(0.2) %>%
+#'   add_binary_decisions() %>%
+#'   add_default_solver(verbose = FALSE)
 #'
 #' # solve problem
 #' s0 <- solve(p0)
 #'
 #' # plot solution
-#' plot(s0, main = "solution", axes = FALSE, box = FALSE)
+#' plot(s0, main = "solution", axes = FALSE)
 #'
 #' # now let's create some modified versions of this baseline problem by
 #' # adding additional criteria using linear constraints
@@ -94,23 +96,26 @@ NULL
 #'
 #' # plot the primary cost dataset (sim_pu_raster) and
 #' # the secondary cost dataset (sim_pu_raster2)
-#' plot(stack(sim_pu_raster, sim_pu_raster2),
-#'      main = c("sim_pu_raster", "sim_pu_raster2"),
-#'      axes = FALSE, box = FALSE)
+#' plot(
+#'   c(sim_pu_raster, sim_pu_raster2),
+#'   main = c("sim_pu_raster", "sim_pu_raster2"),
+#'   axes = FALSE
+#' )
 #'
 #' # create a modified version of p0 with linear constraints that
 #' # specify that the planning units in the solution must not have
-#' # values in sim_pu_raster2 that sum to a total greater than 1600
-#' p1 <- p0 %>%
-#'       add_linear_constraints(threshold = 1600,
-#'                              sense = "<=",
-#'                              data = sim_pu_raster2)
+#' # values in sim_pu_raster2 that sum to a total greater than 500
+#' p1 <-
+#'   p0 %>%
+#'   add_linear_constraints(
+#'     threshold = 500, sense = "<=", data = sim_pu_raster2
+#'   )
 #'
 #' # solve problem
 #' s1 <- solve(p1)
 #'
 #' # plot solutions s1 and s2 to compare them
-#' plot(stack(s0, s1), main = c("s0", "s1"), axes = FALSE, box = FALSE)
+#' plot(c(s0, s1), main = c("s0", "s1"), axes = FALSE)
 #'
 #' # second, let's create a modified version of p0 that contains
 #' # additional constraints to ensure that each feature has
@@ -124,11 +129,14 @@ NULL
 #' # feature to specify that the planning units in the solution must
 #' # secure at least 8% of the total abundance for each feature
 #' p2 <- p0
-#' for (i in seq_len(nlayers(sim_features))) {
-#'   p2 <- p2 %>%
-#'         add_linear_constraints(threshold = feat_abund[i] * 0.08,
-#'                                sense = ">=",
-#'                                data = sim_features[[i]])
+#' for (i in seq_len(terra::nlyr(sim_features))) {
+#'   p2 <-
+#'     p2 %>%
+#'     add_linear_constraints(
+#'       threshold = feat_abund[i] * 0.08,
+#'       sense = ">=",
+#'       data = sim_features[[i]]
+#'     )
 #' }
 #'
 #' # overall, p2 could be described as an optimization problem
@@ -143,7 +151,7 @@ NULL
 #' s2 <- solve(p2)
 #'
 #' # plot solutions s0 and s2 to compare them
-#' plot(stack(s0, s2), main = c("s1", "s2"), axes = FALSE, box = FALSE)
+#' plot(c(s0, s2), main = c("s1", "s2"), axes = FALSE)
 #'
 #' # third, let's create a modified version of p0 that contains
 #' # additional constraints to ensure that the solution equitably
@@ -153,25 +161,25 @@ NULL
 #' # to begin with, we will simulate a dataset describing the spatial extent of
 #' # four different administrative areas across the study region
 #' sim_admin <- sim_pu_raster
-#' sim_admin <- aggregate(sim_admin, fact = 5)
-#' values(sim_admin) <- seq_along(values(sim_admin))
-#' sim_admin <- resample(sim_admin, sim_pu_raster, method = "ngb")
-#' sim_admin <- mask(sim_admin, sim_pu_raster)
+#' sim_admin <- terra::aggregate(sim_admin, fact = 5)
+#' sim_admin <- terra::setValues(sim_admin, seq_len(terra::ncell(sim_admin)))
+#' sim_admin <- terra::resample(sim_admin, sim_pu_raster, method = "near")
+#' sim_admin <- terra::mask(sim_admin, sim_pu_raster)
 #'
 #' # plot administrative areas layer,
 #' # we can see that the administrative areas subdivide
 #' # the study region into four quadrants, and the sim_admin object is a
-#' # RasterLayer with integer values denoting ids for the administrative areas
-#' plot(sim_admin)
+#' # SpatRaster with integer values denoting ids for the administrative areas
+#' plot(sim_admin, axes = FALSE)
 #'
-#' # next we will convert the sim_admin RasterLayer object into a RasterStack
+#' # next we will convert the sim_admin SpatRaster object into a SpatRaster
 #' # object (with a layer for each administrative area) indicating which
 #' # planning units belong to each administrative area using binary
 #' # (presence/absence) values
 #' sim_admin2 <- binary_stack(sim_admin)
 #'
-#' # plot administrative areas stack
-#' plot(sim_admin2)
+#' # plot binary stack of administrative areas
+#' plot(sim_admin2, axes = FALSE)
 #'
 #' # we will now calculate the total amount of planning units associated
 #' # with each administrative area, so that we can set the constraint threshold
@@ -180,39 +188,58 @@ NULL
 #' # accounting for the total area of each planning unit (because all
 #' # planning units have the same area in raster formats) -- but if we were
 #' # using vector data then we would need to account for the area of each unit
-#' admin_total <- rowSums(rij_matrix(sim_pu_raster, sim_admin2))
+#' admin_total <- Matrix::rowSums(rij_matrix(sim_pu_raster, sim_admin2))
 #'
 #' # create a modified version of p0 with additional constraints for each
 #' # administrative area to specify that the planning units in the solution must
 #' # not encompass more than 10% of the total extent of the administrative
 #' # area
 #' p3 <- p0
-#' for (i in seq_len(nlayers(sim_admin2))) {
-#'   p3 <- p3 %>%
-#'         add_linear_constraints(threshold = admin_total[i] * 0.1,
-#'                                sense = "<=",
-#'                                data = sim_admin2[[i]])
+#' for (i in seq_len(terra::nlyr(sim_admin2))) {
+#'   p3 <-
+#'     p3 %>%
+#'     add_linear_constraints(
+#'       threshold = admin_total[i] * 0.1,
+#'       sense = "<=",
+#'       data = sim_admin2[[i]]
+#'     )
 #' }
 #'
 #' # solve problem
 #' s3 <- solve(p3)
 #'
 #' # plot solutions s0 and s3 to compare them
-#' plot(stack(s0, s3), main = c("s0", "s3"), axes = FALSE, box = FALSE)
+#' plot(c(s0, s3), main = c("s0", "s3"), axes = FALSE)
 #' }
 #'
 #' @name add_linear_constraints
 #'
 #' @exportMethod add_linear_constraints
 #'
-#' @aliases add_linear_constraints,ConservationProblem,ANY,ANY,Matrix-method add_linear_constraints,ConservationProblem,ANY,ANY,matrix-method add_linear_constraints,ConservationProblem,ANY,ANY,dgCMatrix-method add_linear_constraints,ConservationProblem,ANY,ANY,character-method add_linear_constraints,ConservationProblem,ANY,ANY,numeric-method add_linear_constraints,ConservationProblem,ANY,ANY,Raster-method
+#' @aliases add_linear_constraints,ConservationProblem,ANY,ANY,Matrix-method add_linear_constraints,ConservationProblem,ANY,ANY,matrix-method add_linear_constraints,ConservationProblem,ANY,ANY,dgCMatrix-method add_linear_constraints,ConservationProblem,ANY,ANY,character-method add_linear_constraints,ConservationProblem,ANY,ANY,numeric-method add_linear_constraints,ConservationProblem,ANY,ANY,Raster-method add_linear_constraints,ConservationProblem,ANY,ANY,SpatRaster-method
 NULL
 
 #' @export
 methods::setGeneric("add_linear_constraints",
   signature = methods::signature("x", "threshold", "sense", "data"),
-  function(x, threshold, sense, data)
-    standardGeneric("add_linear_constraints"))
+  function(x, threshold, sense, data) {
+    rlang::check_required(x)
+    rlang::check_required(threshold)
+    rlang::check_required(sense)
+    rlang::check_required(data)
+    assert(
+      is_conservation_problem(x),
+      is_inherits(
+        data,
+        c(
+          "character", "numeric",  "dgCMatrix",
+          "matrix", "Matrix", "SpatRaster", "Raster"
+        )
+      )
+    )
+    standardGeneric("add_linear_constraints")
+  }
+)
 
 #' @name add_linear_constraints
 #' @usage \S4method{add_linear_constraints}{ConservationProblem,ANY,ANY,character}(x, threshold, sense, data)
@@ -221,23 +248,34 @@ methods::setMethod("add_linear_constraints",
   methods::signature("ConservationProblem", "ANY", "ANY", "character"),
   function(x, threshold, sense, data) {
     # validate arguments
-    assertthat::assert_that(
-      inherits(x$data$cost, c("data.frame", "Spatial", "sf")),
-      msg = paste("argument to data is a character and planning units",
-                  "specified in x are not a data.frame, Spatial, or sf object"))
-    assertthat::assert_that(
+    assert(
       assertthat::is.number(threshold),
       assertthat::noNA(threshold),
       assertthat::noNA(data),
-      number_of_zones(x) == length(data))
-    assertthat::assert_that(
+      number_of_zones(x) == length(data)
+    )
+    assert(
       assertthat::is.string(sense),
       assertthat::noNA(sense),
-      sense %in% c("<=", "=", ">="))
-    assertthat::assert_that(
+      is_match_of(sense, c("<=", "=", ">="))
+    )
+    assert(
+      is_inherits(x$data$cost, c("data.frame", "tbl_df", "sf", "Spatial")),
+      msg = c(
+        "{.arg data} cannot be a character vector.",
+        "i" = paste(
+          "This is because planning unit data for {.arg x} are not a",
+          "data frame or {.cls sf}."
+        )
+      )
+    )
+    assert(
       all(data %in% names(x$data$cost)),
-      msg = paste("argument to data is not a field name in the planning units",
-                  "specified in x"))
+      msg = paste(
+        "{.arg data} must contain character values that refer to column",
+        "names of the planning unit data for {.arg x}."
+      )
+    )
     # extract planning unit data
     d <- x$data$cost
     if (inherits(d, "Spatial")) {
@@ -248,17 +286,25 @@ methods::setMethod("add_linear_constraints",
       d <- as.matrix(d[, data, drop = FALSE])
     }
     # additional checks
-    assertthat::assert_that(
+    assert(
       is.numeric(d),
-      msg = paste("argument to data correspond to non-numeric fields in",
-                  "the planning unit data associated with x"))
-    assertthat::assert_that(
+      msg = paste(
+        "{.arg data} must contain character values that refer to numeric",
+        "columns of the planning unit data for {.arg x}."
+      )
+    )
+    assert(
       assertthat::noNA(d),
-      msg = paste("argument to data correspond to fields with NA values in",
-                  "the planning unit data associated with x"))
+      msg = paste(
+        "{.arg data} must not refer to columns",
+        "of the planning unit data for {.arg} with missing",
+        "({.val {NA}) values."
+      )
+    )
     # add penalties
     add_linear_constraints(x, threshold, sense, d)
-})
+  }
+)
 
 #' @name add_linear_constraints
 #' @usage \S4method{add_linear_constraints}{ConservationProblem,ANY,ANY,numeric}(x, threshold, sense, data)
@@ -266,16 +312,9 @@ methods::setMethod("add_linear_constraints",
 methods::setMethod("add_linear_constraints",
   methods::signature("ConservationProblem", "ANY", "ANY", "numeric"),
   function(x, threshold, sense, data) {
-    assertthat::assert_that(
-      assertthat::is.number(threshold),
-      assertthat::noNA(threshold),
-      assertthat::is.string(sense),
-      assertthat::noNA(sense),
-      sense %in% c("<=", "=", ">="),
-      assertthat::noNA(data),
-      number_of_total_units(x) == length(data))
     add_linear_constraints(x, threshold, sense, matrix(data, ncol = 1))
-})
+  }
+)
 
 #' @name add_linear_constraints
 #' @usage \S4method{add_linear_constraints}{ConservationProblem,ANY,ANY,matrix}(x, threshold, sense, data)
@@ -283,17 +322,9 @@ methods::setMethod("add_linear_constraints",
 methods::setMethod("add_linear_constraints",
   methods::signature("ConservationProblem", "ANY", "ANY", "matrix"),
   function(x, threshold, sense, data) {
-    assertthat::assert_that(
-      assertthat::is.number(threshold),
-      assertthat::noNA(threshold),
-      assertthat::is.string(sense),
-      assertthat::noNA(sense),
-      sense %in% c("<=", "=", ">="),
-      assertthat::noNA(c(data)),
-      number_of_total_units(x) == nrow(data),
-      number_of_zones(x) == ncol(data))
     add_linear_constraints(x, threshold, sense, as_Matrix(data, "dgCMatrix"))
-})
+  }
+)
 
 #' @name add_linear_constraints
 #' @usage \S4method{add_linear_constraints}{ConservationProblem,ANY,ANY,Matrix}(x, threshold, sense, data)
@@ -301,46 +332,51 @@ methods::setMethod("add_linear_constraints",
 methods::setMethod("add_linear_constraints",
   methods::signature("ConservationProblem", "ANY", "ANY", "Matrix"),
   function(x, threshold, sense, data) {
-    assertthat::assert_that(
-      assertthat::is.number(threshold),
-      assertthat::noNA(threshold),
-      assertthat::is.string(sense),
-      assertthat::noNA(sense),
-      sense %in% c("<=", "=", ">="),
-      assertthat::noNA(c(data)),
-      number_of_total_units(x) == nrow(data),
-      number_of_zones(x) == ncol(data))
     add_linear_constraints(x, threshold, sense, as_Matrix(data, "dgCMatrix"))
-})
+  }
+)
 
 #' @name add_linear_constraints
 #' @usage \S4method{add_linear_constraints}{ConservationProblem,ANY,ANY,Raster}(x, threshold, sense, data)
 #' @rdname add_linear_constraints
 methods::setMethod("add_linear_constraints",
   methods::signature("ConservationProblem", "ANY", "ANY", "Raster"),
+  function(x, threshold, sense, data) {
+    cli_warning(raster_pkg_deprecation_notice)
+    add_linear_constraints(x, threshold, sense, terra::rast(data, "SpatRaster"))
+  }
+)
+
+#' @name add_linear_constraints
+#' @usage \S4method{add_linear_constraints}{ConservationProblem,ANY,ANY,SpatRaster}(x, threshold, sense, data)
+#' @rdname add_linear_constraints
+methods::setMethod("add_linear_constraints",
+  methods::signature("ConservationProblem", "ANY", "ANY", "SpatRaster"),
   function(x, threshold, sense, data) {
     # assert valid arguments
-    assertthat::assert_that(
-      inherits(x, "ConservationProblem"),
-      inherits(data, "Raster"),
+    assert(
+      is_conservation_problem(x),
+      is_pu_spatially_explicit(x),
+      inherits(data, "SpatRaster"),
       assertthat::is.number(threshold),
       assertthat::noNA(threshold),
       assertthat::is.string(sense),
       assertthat::noNA(sense),
-      sense %in% c("<=", "=", ">="),
-      number_of_zones(x) == raster::nlayers(data),
-      inherits(x$data$cost, c("sf", "Spatial", "Raster")))
+      is_match_of(sense, c("<=", "=", ">=")),
+      number_of_zones(x) == terra::nlyr(data)
+    )
     # extract constraint data
     if (inherits(x$data$cost, c("sf", "Spatial"))) {
       d <- fast_extract(data, x$data$cost, fun = "sum")
     } else {
-      assertthat::assert_that(is_comparable_raster(x$data$cost, data[[1]]))
-      d <- as.matrix(raster::as.data.frame(data))
+      assert(is_pu_comparable_raster(x, data[[1]]))
+      d <- as.matrix(terra::as.data.frame(data, na.rm = FALSE))
     }
     d[is.na(d)] <- 0
     # add constraints
     add_linear_constraints(x, threshold, sense, as_Matrix(d, "dgCMatrix"))
-})
+  }
+)
 
 #' @name add_linear_constraints
 #' @usage \S4method{add_linear_constraints}{ConservationProblem,ANY,ANY,dgCMatrix}(x, threshold, sense, data)
@@ -349,44 +385,57 @@ methods::setMethod("add_linear_constraints",
   methods::signature("ConservationProblem", "ANY", "ANY", "dgCMatrix"),
   function(x, threshold, sense, data) {
     # assert valid arguments
-    assertthat::assert_that(
-      inherits(x, "ConservationProblem"),
+    assert(
+      is_conservation_problem(x),
       assertthat::is.number(threshold),
       assertthat::noNA(threshold),
       assertthat::is.string(sense),
       assertthat::noNA(sense),
-      sense %in% c("<=", "=", ">="),
-      is.numeric(data@x),
-      all(is.finite(data@x),
+      is_match_of(sense, c("<=", "=", ">=")),
+      is_numeric_values(data),
+      all_finite(data),
       number_of_total_units(x) == nrow(data),
-      number_of_zones(x) == ncol(data)))
+      number_of_zones(x) == ncol(data)
+    )
     # assert that constraint is even remotely possible
     if (identical(sense, ">=")) {
-      assertthat::assert_that(
+      assert(
         any(Matrix::colSums(data) >= threshold),
-        msg = paste("linear constraint cannot achieve threshold even if all",
-                    "planning units selected.")
+        msg = paste(
+          "The linear constraint cannot be meet {.arg threshold} even if all",
+          "planning units selected in the solution."
+        )
       )
     }
     # add penalties
-    x$add_constraint(pproto(
-      "LinearConstraint",
-      Constraint,
-      name = "Linear constraints",
-      data = list(data = data, sense = sense),
-      parameters = parameters(numeric_parameter("threshold", threshold)),
-      apply = function(self, x, y) {
-        assertthat::assert_that(inherits(x, "OptimizationProblem"),
-                                inherits(y, "ConservationProblem"))
-        # extract parameters
-        th <- self$parameters$get("threshold")
-        # extract data
-        indices <- y$planning_unit_indices()
-        d <- self$get_data("data")[indices, , drop = FALSE]
-        s <- self$get_data("sense")
-        # apply constraints
-        rcpp_apply_linear_constraints(x$ptr, th, s, d)
-        # return success
-        invisible(TRUE)
-    }))
-})
+    x$add_constraint(
+      R6::R6Class(
+        "LinearConstraint",
+        inherit = Constraint,
+        public = list(
+          name = "linear constraints",
+          data = list(threshold = threshold, sense = sense, data = data),
+          apply = function(x, y) {
+            assert(
+              inherits(x, "OptimizationProblem"),
+              inherits(y, "ConservationProblem"),
+              .internal = TRUE
+            )
+            # extract data
+            indices <- y$planning_unit_indices()
+            d <- self$get_data("data")[indices, , drop = FALSE]
+            # apply constraints
+            rcpp_apply_linear_constraints(
+              x$ptr,
+              self$get_data("threshold"),
+              self$get_data("sense"),
+              d
+            )
+            # return success
+            invisible(TRUE)
+          }
+        )
+      )$new()
+    )
+  }
+)
