@@ -1,16 +1,16 @@
-#' @include internal.R Penalty-proto.R marxan_boundary_data_to_matrix.R
+#' @include internal.R Penalty-class.R
 NULL
 
 #' Add linear penalties
 #'
-#' Add penalties to a conservation planning [problem()] to penalize
+#' Add penalties to a conservation planning problem to penalize
 #' solutions that select planning units with higher values from a specific
 #' data source (e.g., anthropogenic impact). These penalties assume
 #' a linear trade-off between the penalty values and the primary
-#' objective of the conservation planning [problem()] (e.g.,
+#' objective of the conservation planning problem (e.g.,
 #' solution cost for minimum set problems; [add_min_set_objective()].
 #'
-#' @param x [problem()] (i.e., [`ConservationProblem-class`]) object.
+#' @param x [problem()] object.
 #'
 #' @param penalty `numeric` penalty value that is used to scale the
 #'   importance not selecting planning units with high `data` values.
@@ -25,7 +25,7 @@ NULL
 #'   must have a value for each zone.
 #'
 #' @param data `character`, `numeric`,
-#'   [`Raster-class`], `matrix`, or `Matrix` object
+#'   [terra::rast()], `matrix`, or `Matrix` object
 #'   containing the values used to penalize solutions. Planning units that are
 #'   associated with higher data values are penalized more strongly
 #'   in the solution. See the Data format section for more information.
@@ -41,16 +41,15 @@ NULL
 #'
 #' \describe{
 #'
-#' \item{`data` as `character` vector}{containing field (column) name(s) that
+#' \item{`data` as `character` vector}{containing column name(s) that
 #'   contain penalty values for planning units. This format is only
 #'   compatible if the planning units in the argument to `x` are a
-#'   [`Spatial-class`], [sf::sf()], or
-#'   `data.frame` object. The fields (columns) must have `numeric`
+#'   [sf::sf()] or `data.frame` object. The column(s) must have `numeric`
 #'   values, and must not contain any missing (`NA`) values.
 #'   For problems that contain a single zone, the argument to `data` must
-#'   contain a single field name. Otherwise, for problems that
+#'   contain a single column name. Otherwise, for problems that
 #'   contain multiple zones, the argument to `data` must
-#'   contain a field name for each zone.}
+#'   contain a column name for each zone.}
 #'
 #' \item{`data` as a `numeric` vector}{containing values for
 #'   planning units. These values must not contain any missing
@@ -63,18 +62,16 @@ NULL
 #'   zone, and each cell indicates the data for penalizing a planning unit
 #'   when it is allocated to a given zone.}
 #'
-#' \item{`data` as a [`Raster-class`] object}{containing values for planning
+#' \item{`data` as a [terra::rast()] object}{containing values for planning
 #'   units. This format is only
 #'   compatible if the planning units in the argument to `x` are
-#'   [`Spatial-class`], [sf::sf()], or
-#'   [`Raster-class`] objects.
-#'   If the planning unit data are a [`Spatial-class`] or
-#'   [sf::sf()] object, then the values are calculated by overlaying the
+#'   [sf::sf()], or [terra::rast()] objects.
+#'   If the planning unit data are a [sf::sf()] object,
+#'   then the values are calculated by overlaying the
 #'   planning units with the argument to `data` and calculating the sum of the
 #'   values associated with each planning unit.
-#'   If the planning unit data are a
-#'   [`Raster-class`] object then the values are calculated by extracting the
-#'   cell
+#'   If the planning unit data are a [terra::rast()] object, then the values
+#'   are calculated by extracting the cell
 #'   values (note that the planning unit data and the argument to `data` must
 #'   have exactly the same dimensionality, extent, and missingness).
 #'   For problems involving multiple zones, the argument to `data` must
@@ -119,23 +116,26 @@ NULL
 #' set.seed(600)
 #'
 #' # load data
-#' data(sim_pu_polygons, sim_pu_zones_stack, sim_features, sim_features_zones)
+#' sim_pu_polygons <- get_sim_pu_polygons()
+#' sim_features <- get_sim_features()
+#' sim_zones_pu_raster <- get_sim_zones_pu_raster()
+#' sim_zones_features <- get_sim_zones_features()
 #'
 #' # add a column to contain the penalty data for each planning unit
 #' # e.g., these values could indicate the level of habitat
 #' sim_pu_polygons$penalty_data <- runif(nrow(sim_pu_polygons))
 #'
 #' # plot the penalty data to visualise its spatial distribution
-#' spplot(sim_pu_polygons, zcol = "penalty_data", main = "penalty data",
-#'        axes = FALSE, box = FALSE)
+#' plot(sim_pu_polygons[, "penalty_data"], axes = FALSE)
 #'
 #' # create minimal problem with minimum set objective,
 #' # this does not use the penalty data
-#' p1 <- problem(sim_pu_polygons, sim_features, cost_column = "cost") %>%
-#'       add_min_set_objective() %>%
-#'       add_relative_targets(0.1) %>%
-#'       add_binary_decisions() %>%
-#'       add_default_solver(verbose = FALSE)
+#' p1 <-
+#'   problem(sim_pu_polygons, sim_features, cost_column = "cost") %>%
+#'   add_min_set_objective() %>%
+#'   add_relative_targets(0.1) %>%
+#'   add_binary_decisions() %>%
+#'   add_default_solver(verbose = FALSE)
 #'
 #' # print problem
 #' print(p1)
@@ -151,14 +151,21 @@ NULL
 #' s1 <- solve(p1)
 #' s2 <- solve(p2)
 #'
+#' # create a new object with both solutions
+#' s3 <- sf::st_sf(
+#'   tibble::tibble(
+#'     s1 = s1$solution_1,
+#'     s2 = s2$solution_1
+#'   ),
+#'   geometry = sf::st_geometry(s1)
+#' )
+#'
+#'
 #' # plot the solutions and compare them,
 #' # since we supplied a very high penalty value (i.e., 100), relative
 #' # to the range of values in the penalty data and the objective function,
 #' # the solution in s2 is very sensitive to values in the penalty data
-#' spplot(s1, zcol = "solution_1", main = "solution without penalties",
-#'        axes = FALSE, box = FALSE)
-#' spplot(s2, zcol = "solution_1", main = "solution with penalties",
-#'        axes = FALSE, box = FALSE)
+#' plot(s3, axes = FALSE)
 #'
 #' # for real conservation planning exercises,
 #' # it would be worth exploring a range of penalty values (e.g., ranging
@@ -170,49 +177,68 @@ NULL
 #' # create targets for each feature within each zone,
 #' # these targets indicate that each zone needs to represent 10% of the
 #' # spatial distribution of each feature
-#' targ <- matrix(0.1, ncol = number_of_zones(sim_features_zones),
-#'                nrow = number_of_features(sim_features_zones))
+#' targ <- matrix(
+#'   0.1, ncol = number_of_zones(sim_zones_features),
+#'   nrow = number_of_features(sim_zones_features)
+#' )
 #'
 #' # create penalty data for allocating each planning unit to each zone,
 #' # these data will be generated by simulating values
-#' penalty_stack <- simulate_cost(sim_pu_zones_stack[[1]],
-#'                                n = number_of_zones(sim_features_zones))
+#' penalty_raster <- simulate_cost(
+#'   sim_zones_pu_raster[[1]],
+#'   n = number_of_zones(sim_zones_features)
+#' )
 #'
 #' # plot the penalty data, each layer corresponds to a different zone
-#' plot(penalty_stack, main = "penalty data", axes = FALSE, box = FALSE)
+#' plot(penalty_raster, main = "penalty data", axes = FALSE)
 #'
 #' # create a multi-zone problem with the minimum set objective
 #' # and penalties for allocating planning units to each zone,
 #' # with a penalty scaling factor of 1 for each zone
-#' p3 <- problem(sim_pu_zones_stack, sim_features_zones) %>%
-#'       add_min_set_objective() %>%
-#'       add_relative_targets(targ) %>%
-#'       add_linear_penalties(c(1, 1, 1), penalty_stack) %>%
-#'       add_binary_decisions() %>%
-#'       add_default_solver(verbose = FALSE)
+#' p4 <-
+#'   problem(sim_zones_pu_raster, sim_zones_features) %>%
+#'   add_min_set_objective() %>%
+#'   add_relative_targets(targ) %>%
+#'   add_linear_penalties(c(1, 1, 1), penalty_raster) %>%
+#'   add_binary_decisions() %>%
+#'   add_default_solver(verbose = FALSE)
 #'
 #' # print problem
-#' print(p3)
+#' print(p4)
 #'
 #' # solve problem
-#' s3 <- solve(p3)
+#' s4 <- solve(p4)
 #'
 #' # plot solution
-#' plot(category_layer(s3), main = "multi-zone solution",
-#'      axes = FALSE, box = FALSE)
+#' plot(category_layer(s4), main = "multi-zone solution", axes = FALSE)
 #' }
 #' @name add_linear_penalties
 #'
 #' @exportMethod add_linear_penalties
 #'
-#' @aliases add_linear_penalties,ConservationProblem,ANY,Matrix-method add_linear_penalties,ConservationProblem,ANY,matrix-method add_linear_penalties,ConservationProblem,ANY,dgCMatrix-method add_linear_penalties,ConservationProblem,ANY,character-method add_linear_penalties,ConservationProblem,ANY,numeric-method add_linear_penalties,ConservationProblem,ANY,Raster-method
+#' @aliases add_linear_penalties,ConservationProblem,ANY,Matrix-method add_linear_penalties,ConservationProblem,ANY,matrix-method add_linear_penalties,ConservationProblem,ANY,dgCMatrix-method add_linear_penalties,ConservationProblem,ANY,character-method add_linear_penalties,ConservationProblem,ANY,numeric-method add_linear_penalties,ConservationProblem,ANY,Raster-method add_linear_penalties,ConservationProblem,ANY,SpatRaster-method
 NULL
 
 #' @export
 methods::setGeneric("add_linear_penalties",
   signature = methods::signature("x", "penalty", "data"),
-  function(x, penalty, data)
-    standardGeneric("add_linear_penalties"))
+  function(x, penalty, data) {
+    rlang::check_required(x)
+    rlang::check_required(penalty)
+    rlang::check_required(data)
+    assert(
+      is_conservation_problem(x),
+      is_inherits(
+        data,
+        c(
+          "character", "numeric", "dgCMatrix",
+          "matrix", "Matrix", "SpatRaster", "Raster"
+        )
+      )
+    )
+    standardGeneric("add_linear_penalties")
+  }
+)
 
 #' @name add_linear_penalties
 #' @usage \S4method{add_linear_penalties}{ConservationProblem,ANY,character}(x, penalty, data)
@@ -221,18 +247,27 @@ methods::setMethod("add_linear_penalties",
   methods::signature("ConservationProblem", "ANY", "character"),
   function(x, penalty, data) {
     # validate arguments
-    assertthat::assert_that(
-      inherits(x$data$cost, c("data.frame", "Spatial", "sf")),
-      msg = paste("argument to data is a character and planning units",
-                  "specified in x are not a data.frame, Spatial, or sf object"))
-    assertthat::assert_that(
-      is.numeric(penalty), assertthat::noNA(penalty),
+    assert(
+      is.numeric(penalty),
+      assertthat::noNA(penalty),
       number_of_zones(x) == length(penalty),
-      assertthat::noNA(data), number_of_zones(x) == length(data))
-    assertthat::assert_that(
+      assertthat::noNA(data),
+      number_of_zones(x) == length(data)
+    )
+    assert(
+      is_inherits(x$data$cost, c("matrix", "data.frame", "sf", "Spatial")),
+      msg = paste(
+        "The planning unit data for {.arg x} does not have columns,",
+        "and so {.arg data} cannot be a character value.",
+      )
+    )
+    assert(
       all(data %in% names(x$data$cost)),
-      msg = paste("argument to data is not a field name in the planning units",
-                  "specified in x"))
+      msg = paste(
+        "{.arg data} must contain character values that refer to column",
+        "names of the planning unit data for {.arg x}."
+      )
+    )
     # extract planning unit data
     d <- x$data$cost
     if (inherits(d, "Spatial")) {
@@ -243,17 +278,25 @@ methods::setMethod("add_linear_penalties",
       d <- as.matrix(d[, data, drop = FALSE])
     }
     # additional checks
-    assertthat::assert_that(
+    assert(
       is.numeric(d),
-      msg = paste("argument to data correspond to non-numeric fields in",
-                  "the planning unit data associated with x"))
-    assertthat::assert_that(
+      msg = paste(
+        "{.arg data} must contain character values that refer to numeric",
+        "columns of the planning unit data for {.arg x}."
+      )
+    )
+    assert(
       assertthat::noNA(d),
-      msg = paste("argument to data correspond to fields with NA values in",
-                  "the planning unit data associated with x"))
+      msg = paste(
+        "{.arg data} must not refer to columns",
+        "of the planning unit data for {.arg} with missing",
+        "({.val {NA}) values."
+      )
+    )
     # add penalties
     add_linear_penalties(x, penalty, d)
-})
+  }
+)
 
 #' @name add_linear_penalties
 #' @usage \S4method{add_linear_penalties}{ConservationProblem,ANY,numeric}(x, penalty, data)
@@ -261,12 +304,9 @@ methods::setMethod("add_linear_penalties",
 methods::setMethod("add_linear_penalties",
   methods::signature("ConservationProblem", "ANY", "numeric"),
   function(x, penalty, data) {
-    assertthat::assert_that(
-      is.numeric(penalty), assertthat::noNA(penalty),
-      number_of_zones(x) == length(penalty),
-      assertthat::noNA(data), number_of_total_units(x) == length(data))
-      add_linear_penalties(x, penalty, matrix(data, ncol = 1))
-})
+    add_linear_penalties(x, penalty, matrix(data, ncol = 1))
+  }
+)
 
 #' @name add_linear_penalties
 #' @usage \S4method{add_linear_penalties}{ConservationProblem,ANY,matrix}(x, penalty, data)
@@ -274,14 +314,9 @@ methods::setMethod("add_linear_penalties",
 methods::setMethod("add_linear_penalties",
   methods::signature("ConservationProblem", "ANY", "matrix"),
   function(x, penalty, data) {
-    assertthat::assert_that(
-      is.numeric(penalty), assertthat::noNA(penalty),
-      number_of_zones(x) == length(penalty),
-      assertthat::noNA(c(data)),
-      number_of_total_units(x) == nrow(data),
-      number_of_zones(x) == ncol(data))
     add_linear_penalties(x, penalty, as_Matrix(data, "dgCMatrix"))
-})
+  }
+)
 
 #' @name add_linear_penalties
 #' @usage \S4method{add_linear_penalties}{ConservationProblem,ANY,Matrix}(x, penalty, data)
@@ -289,40 +324,62 @@ methods::setMethod("add_linear_penalties",
 methods::setMethod("add_linear_penalties",
   methods::signature("ConservationProblem", "ANY", "Matrix"),
   function(x, penalty, data) {
-    assertthat::assert_that(
-      is.numeric(penalty), assertthat::noNA(penalty),
-      number_of_zones(x) == length(penalty),
-      assertthat::noNA(c(data)),
-      number_of_total_units(x) == nrow(data),
-      number_of_zones(x) == ncol(data))
     add_linear_penalties(x, penalty, as_Matrix(data, "dgCMatrix"))
-})
+  }
+)
 
 #' @name add_linear_penalties
 #' @usage \S4method{add_linear_penalties}{ConservationProblem,ANY,Raster}(x, penalty, data)
 #' @rdname add_linear_penalties
 methods::setMethod("add_linear_penalties",
   methods::signature("ConservationProblem", "ANY", "Raster"),
+  function(x, penalty, data) {
+    cli_warning(raster_pkg_deprecation_notice)
+    add_linear_penalties(x, penalty, terra::rast(data))
+  }
+)
+
+#' @name add_linear_penalties
+#' @usage \S4method{add_linear_penalties}{ConservationProblem,ANY,SpatRaster}(x, penalty, data)
+#' @rdname add_linear_penalties
+methods::setMethod("add_linear_penalties",
+  methods::signature("ConservationProblem", "ANY", "SpatRaster"),
   function(x, penalty, data) {
     # assert valid arguments
-    assertthat::assert_that(
-      inherits(x, "ConservationProblem"),
-      inherits(data, "Raster"),
-      is.numeric(penalty), assertthat::noNA(penalty),
-      number_of_zones(x) == raster::nlayers(data),
-      number_of_zones(x) == length(penalty),
-      inherits(x$data$cost, c("sf", "Spatial", "Raster")))
+    assert(
+      is_conservation_problem(x),
+      inherits(data, "SpatRaster"),
+      is.numeric(penalty),
+      all_finite(penalty),
+      number_of_zones(x) == terra::nlyr(data),
+      number_of_zones(x) == length(penalty)
+    )
+    assert(
+      is_pu_spatially_explicit(x),
+      msg =
+      c(
+        paste(
+          "{.arg x} has planning unit data that are not spatially explicit",
+          "(e.g., {.cls sf} or {.cls SpatRaster})."
+        ),
+        "i" = paste(
+          "{.arg data} must be a {.cls matrix}, {.cls Matrix},",
+          "or {.cls character} vector."
+        )
+      )
+    )
     # extract penalty data
     if (inherits(x$data$cost, c("sf", "Spatial"))) {
       d <- fast_extract(data, x$data$cost, fun = "sum")
     } else {
-      assertthat::assert_that(is_comparable_raster(x$data$cost, data[[1]]))
-      d <- as.matrix(raster::as.data.frame(data))
+      assert(is_pu_comparable_raster(x, data[[1]]))
+      d <- as.matrix(terra::as.data.frame(data, na.rm = FALSE))
     }
     d[is.na(d)] <- 0
     # add penalties
     add_linear_penalties(x, penalty, as_Matrix(d, "dgCMatrix"))
-})
+  }
+)
 
 #' @name add_linear_penalties
 #' @usage \S4method{add_linear_penalties}{ConservationProblem,ANY,dgCMatrix}(x, penalty, data)
@@ -331,39 +388,43 @@ methods::setMethod("add_linear_penalties",
   methods::signature("ConservationProblem", "ANY", "dgCMatrix"),
   function(x, penalty, data) {
     # assert valid arguments
-    assertthat::assert_that(
-      inherits(x, "ConservationProblem"),
-      is.numeric(penalty), isTRUE(all(is.finite(penalty))),
+    assert(
+      is_conservation_problem(x),
+      is.numeric(penalty),
+      all_finite(penalty),
       number_of_zones(x) == length(penalty),
-      is.numeric(data@x), all(is.finite(data@x),
+      is_numeric_values(data),
+      all_finite(data),
       number_of_total_units(x) == nrow(data),
-      number_of_zones(x) == ncol(data)))
-    # create parameters
-    if (number_of_zones(x) == 1) {
-      p <- numeric_parameter("penalty", penalty)
-    } else {
-      p <- numeric_parameter_array("penalty", penalty, x$zone_names())
-    }
+      number_of_zones(x) == ncol(data)
+    )
     # add penalties
-    x$add_penalty(pproto(
-      "LinearPenalty",
-      Penalty,
-      name = "Linear penalties",
-      data = list(data = data),
-      parameters = parameters(p),
-      apply = function(self, x, y) {
-        assertthat::assert_that(inherits(x, "OptimizationProblem"),
-                                inherits(y, "ConservationProblem"))
-        # extract parameters
-        p <- self$parameters$get("penalty")
-        if (inherits(p, "data.frame")) p <- p[[1]]
-        if (min(abs(p)) > 1e-50) {
-          # extract data
-          indices <- y$planning_unit_indices()
-          d <- self$get_data("data")[indices, , drop = FALSE]
-          # apply penalties
-          rcpp_apply_linear_penalties(x$ptr, p, d)
-        }
-        invisible(TRUE)
-    }))
-})
+    x$add_penalty(
+      R6::R6Class(
+        "LinearPenalty",
+        inherit = Penalty,
+        public = list(
+          name = "linear penalties",
+          data = list(penalty = penalty, data = data),
+          apply = function(x, y) {
+            assert(
+              inherits(x, "OptimizationProblem"),
+              inherits(y, "ConservationProblem"),
+              .internal = TRUE
+            )
+            # extract parameters
+            p <- self$get_data("penalty")
+            if (min(abs(p)) > 1e-50) {
+              # extract data
+              indices <- y$planning_unit_indices()
+              d <- self$get_data("data")[indices, , drop = FALSE]
+              # apply penalties
+              rcpp_apply_linear_penalties(x$ptr, p, d)
+            }
+            invisible(TRUE)
+          }
+        )
+      )$new()
+    )
+  }
+)
