@@ -1,9 +1,9 @@
-#' @include internal.R Constraint-proto.R intersecting_units.R
+#' @include internal.R Constraint-class.R intersecting_units.R
 NULL
 
 #' Add locked in constraints
 #'
-#' Add constraints to a conservation planning [problem()] to ensure
+#' Add constraints to a conservation planning problem to ensure
 #' that specific planning units are selected (or allocated
 #' to a specific zone) in the solution. For example, it may be desirable to
 #' lock in planning units that are inside existing protected areas so that the
@@ -16,7 +16,7 @@ NULL
 #'
 #' @usage add_locked_in_constraints(x, locked_in)
 #'
-#' @param x [problem()] (i.e., [`ConservationProblem-class`]) object.
+#' @param x [problem()] object.
 #'
 #' @param locked_in Object that determines which planning units that should be
 #'   locked in. See the Data format section for more information.
@@ -26,9 +26,9 @@ NULL
 #' The locked planning units can be specified using the following formats.
 #' Generally, the locked data should correspond to the planning units
 #' in the argument to `x.` To help make working with
-#' [`Raster-class`] planning unit data easier,
+#' [terra::rast()] planning unit data easier,
 #' the locked data should correspond to cell indices in the
-#' [`Raster-class`] data. For example, `integer` arguments
+#' [terra::rast()] data. For example, `integer` arguments
 #' should correspond to cell indices and `logical` arguments should have
 #' a value for each cell---regardless of which planning unit cells contain
 #' `NA` values.
@@ -52,35 +52,35 @@ NULL
 #'   zone. Thus each row should only contain at most a single `TRUE`
 #'   value.}
 #'
-#' \item{`data` as a `character` vector}{containing field (column) name(s)
-#'   that indicate if planning units should be locked for the solution.
+#' \item{`data` as a `character` vector}{containing column name(s)
+#'   that indicates if planning units should be locked for the solution.
 #'   This format is only
 #'   compatible if the planning units in the argument to `x` are a
-#'   [`Spatial-class`], [sf::sf()], or
-#'   `data.frame` object. The fields
-#'   (columns) must have `logical` (i.e., `TRUE` or `FALSE`)
+#'   [sf::sf()] or `data.frame` object. The columns
+#'   must have `logical` (i.e., `TRUE` or `FALSE`)
 #'   values indicating if the planning unit is to be locked for the solution.
 #'   For problems that contain a single zone, the argument to `data` must
-#'   contain a single field name. Otherwise, for problems that
+#'   contain a single column name. Otherwise, for problems that
 #'   contain multiple zones, the argument to `data` must
-#'   contain a field name for each zone.}
+#'   contain a column name for each zone.}
 #'
-#' \item{`data` as a [`Spatial-class`] or [sf::sf()] object}{
+#' \item{`data` as a [sf::sf()] object}{
 #'   containing geometries that will be used to lock planning units for
 #'   the solution. Specifically, planning units in `x` that spatially
 #'   intersect with `y` will be locked (per [intersecting_units()]).
 #'   Note that this option is only available
 #'   for problems that contain a single management zone.}
 #'
-#' \item{`data` as a [`Raster-class`] object}{containing cells used to lock
-#'   planning units for the solution. Specifically, planning units in `x`
+#' \item{`data` as a [terra::rast()] object}{
+#'   containing cells used to lock planning units for the solution.
+#'   Specifically, planning units in `x`
 #'   that intersect with cells that have non-zero and non-`NA` values are
 #'   locked.
 #'   For problems that contain multiple zones, the
-#'   [`Raster-class`] object must contain a layer
+#'   `data` object must contain a layer
 #'   for each zone. Note that for multi-band arguments, each pixel must
 #'   only contain a non-zero value in a single band. Additionally, if the
-#'   cost data in `x` is a [`Raster-class`] object, we
+#'   cost data in `x` is a [terra::rast()] object, we
 #'   recommend standardizing `NA` values in this dataset with the cost
 #'   data. In other words, the pixels in `x` that have `NA` values
 #'   should also have `NA` values in the locked data.}
@@ -99,19 +99,25 @@ NULL
 #' set.seed(500)
 #'
 #' # load data
-#' data(sim_pu_polygons, sim_features, sim_locked_in_raster)
+#' sim_pu_polygons <- get_sim_pu_polygons()
+#' sim_features <- get_sim_features()
+#' sim_locked_in_raster <- get_sim_locked_in_raster()
+#' sim_zones_pu_raster <- get_sim_zones_pu_raster()
+#' sim_zones_pu_polygons <- get_sim_zones_pu_polygons()
+#' sim_zones_features <- get_sim_zones_features()
 #'
 #' # create minimal problem
-#' p1 <- problem(sim_pu_polygons, sim_features, "cost") %>%
-#'       add_min_set_objective() %>%
-#'       add_relative_targets(0.2) %>%
-#'       add_binary_decisions() %>%
-#'       add_default_solver(verbose = FALSE)
+#' p1 <-
+#'   problem(sim_pu_polygons, sim_features, "cost") %>%
+#'   add_min_set_objective() %>%
+#'   add_relative_targets(0.2) %>%
+#'   add_binary_decisions() %>%
+#'   add_default_solver(verbose = FALSE)
 #'
 #' # create problem with added locked in constraints using integers
 #' p2 <- p1 %>% add_locked_in_constraints(which(sim_pu_polygons$locked_in))
 #'
-#' # create problem with added locked in constraints using a field name
+#' # create problem with added locked in constraints using a column name
 #' p3 <- p1 %>% add_locked_in_constraints("locked_in")
 #'
 #' # create problem with added locked in constraints using raster data
@@ -128,111 +134,132 @@ NULL
 #' s4 <- solve(p4)
 #' s5 <- solve(p5)
 #'
+#' # create single object with all solutions
+#' s6 <- sf::st_sf(
+#'   tibble::tibble(
+#'     s1 = s1$solution_1,
+#'     s2 = s2$solution_1,
+#'     s3 = s3$solution_1,
+#'     s4 = s4$solution_1,
+#'     s5 = s5$solution_1
+#'   ),
+#'   geometry = sf::st_geometry(s1)
+#' )
+#'
 #' # plot solutions
-#' par(mfrow = c(3, 2), mar = c(0, 0, 4.1, 0))
-#' plot(s1, main = "none locked in")
-#' plot(s1[s1$solution_1 == 1, ], col = "darkgreen", add = TRUE)
-#'
-#' plot(s2, main = "locked in (integer input)")
-#' plot(s2[s2$solution_1 == 1, ], col = "darkgreen", add = TRUE)
-#'
-#' plot(s3, main = "locked in (character input)")
-#' plot(s3[s3$solution_1 == 1, ], col = "darkgreen", add = TRUE)
-#'
-#' plot(s4, main = "locked in (raster input)")
-#' plot(s4[s4$solution_1 == 1, ], col = "darkgreen", add = TRUE)
-#'
-#' plot(s5, main = "locked in (polygon input)")
-#' plot(s5[s5$solution_1 == 1, ], col = "darkgreen", add = TRUE)
-#'
-#' # reset plot
-#' par(mfrow = c(1, 1))
+#' plot(
+#'   s6,
+#'   main = c(
+#'     "none locked in", "locked in (integer input)",
+#'     "locked in (character input)", "locked in (raster input)",
+#'     "locked in (polygon input)"
+#'   )
+#' )
 #'
 #' # create minimal multi-zone problem with spatial data
-#' p6 <- problem(sim_pu_zones_polygons, sim_features_zones,
-#'               cost_column = c("cost_1", "cost_2", "cost_3")) %>%
-#'       add_min_set_objective() %>%
-#'       add_absolute_targets(matrix(rpois(15, 1), nrow = 5,
-#'                                   ncol = 3)) %>%
-#'       add_binary_decisions() %>%
-#'       add_default_solver(verbose = FALSE)
+#' p7 <-
+#'   problem(
+#'     sim_zones_pu_polygons, sim_zones_features,
+#'     cost_column = c("cost_1", "cost_2", "cost_3")
+#'   ) %>%
+#'   add_min_set_objective() %>%
+#'   add_absolute_targets(matrix(rpois(15, 1), nrow = 5, ncol = 3)) %>%
+#'   add_binary_decisions() %>%
+#'   add_default_solver(verbose = FALSE)
 #'
 #' # create multi-zone problem with locked in constraints using matrix data
-#' locked_matrix <- sim_pu_zones_polygons@data[, c("locked_1", "locked_2",
-#'                                                 "locked_3")]
-#' locked_matrix <- as.matrix(locked_matrix)
+#' locked_matrix <- as.matrix(sf::st_drop_geometry(
+#'   sim_zones_pu_polygons[, c("locked_1", "locked_2", "locked_3")]
+#' ))
 #'
-#' p7 <- p6 %>% add_locked_in_constraints(locked_matrix)
+#' p8 <- p7 %>% add_locked_in_constraints(locked_matrix)
 #'
 #' # solve problem
-#' s6 <- solve(p6)
+#' s7 <- solve(p7)
 #'
 #' # create new column representing the zone id that each planning unit
 #' # was allocated to in the solution
-#' s6$solution <- category_vector(s6@data[, c("solution_1_zone_1",
-#'                                            "solution_1_zone_2",
-#'                                            "solution_1_zone_3")])
-#' s6$solution <- factor(s6$solution)
+#' s7$solution <- category_vector(sf::st_drop_geometry(
+#'   s7[, c("solution_1_zone_1", "solution_1_zone_2", "solution_1_zone_3")]
+#' ))
+#' s7$solution <- factor(s7$solution)
 #'
 #' # plot solution
-#' spplot(s6, zcol = "solution", main = "solution", axes = FALSE, box = FALSE)
+#' plot(s7[ "solution"], axes = FALSE)
 #'
-#' # create multi-zone problem with locked in constraints using field names
-#' p8 <- p6 %>% add_locked_in_constraints(c("locked_1", "locked_2", "locked_3"))
-#'
-#' # solve problem
-#' s8 <- solve(p8)
-#'
-#' # create new column representing the zone id that each planning unit
-#' # was allocated to in the solution
-#' s8$solution <- category_vector(s8@data[, c("solution_1_zone_1",
-#'                                            "solution_1_zone_2",
-#'                                            "solution_1_zone_3")])
-#' s8$solution[s8$solution == 1 & s8$solution_1_zone_1 == 0] <- 0
-#' s8$solution <- factor(s8$solution)
-#'
-#' # plot solution
-#' spplot(s8, zcol = "solution", main = "solution", axes = FALSE, box = FALSE)
-#'
-#' # create multi-zone problem with raster planning units
-#' p9 <- problem(sim_pu_zones_stack, sim_features_zones) %>%
-#'       add_min_set_objective() %>%
-#'       add_absolute_targets(matrix(rpois(15, 1), nrow = 5, ncol = 3)) %>%
-#'       add_binary_decisions() %>%
-#'       add_default_solver(verbose = FALSE)
-#'
-#' # create raster stack with locked in units
-#' locked_in_stack <- sim_pu_zones_stack[[1]]
-#' locked_in_stack[!is.na(locked_in_stack)] <- 0
-#' locked_in_stack <- locked_in_stack[[c(1, 1, 1)]]
-#' locked_in_stack[[1]][1] <- 1
-#' locked_in_stack[[2]][2] <- 1
-#' locked_in_stack[[3]][3] <- 1
-#'
-#' # plot locked in stack
-#' plot(locked_in_stack)
-#' # add locked in raster units to problem
-#' p9 <- p9 %>% add_locked_in_constraints(locked_in_stack)
+#' # create multi-zone problem with locked in constraints using column names
+#' p9 <- p7 %>% add_locked_in_constraints(c("locked_1", "locked_2", "locked_3"))
 #'
 #' # solve problem
 #' s9 <- solve(p9)
 #'
+#' # create new column representing the zone id that each planning unit
+#' # was allocated to in the solution
+#' s9$solution <- category_vector(sf::st_drop_geometry(
+#'   s9[, c("solution_1_zone_1", "solution_1_zone_2", "solution_1_zone_3")]
+#' ))
+#' s9$solution[s9$solution == 1 & s9$solution_1_zone_1 == 0] <- 0
+#' s9$solution <- factor(s9$solution)
+#'
 #' # plot solution
-#' plot(category_layer(s9), main = "solution", axes = FALSE, box = FALSE)
+#' plot(s9[, "solution"], axes = FALSE)
+#'
+#' # create multi-zone problem with raster planning units
+#' p10 <-
+#'   problem(sim_zones_pu_raster, sim_zones_features) %>%
+#'   add_min_set_objective() %>%
+#'   add_absolute_targets(matrix(rpois(15, 1), nrow = 5, ncol = 3)) %>%
+#'   add_binary_decisions() %>%
+#'   add_default_solver(verbose = FALSE)
+#'
+#' # create multi-layer raster with locked in units
+#' locked_in_raster <- sim_zones_pu_raster[[1]]
+#' locked_in_raster[!is.na(locked_in_raster)] <- 0
+#' locked_in_raster <- locked_in_raster[[c(1, 1, 1)]]
+#' locked_in_raster[[1]][1] <- 1
+#' locked_in_raster[[2]][2] <- 1
+#' locked_in_raster[[3]][3] <- 1
+#'
+#' # plot locked in raster
+#' plot(locked_in_raster)
+#'
+#' # add locked in raster units to problem
+#' p10 <- p10 %>% add_locked_in_constraints(locked_in_raster)
+#'
+#' # solve problem
+#' s10 <- solve(p10)
+#'
+#' # plot solution
+#' plot(category_layer(s10), main = "solution", axes = FALSE)
 #' }
 #'
 #' @name add_locked_in_constraints
 #'
 #' @exportMethod add_locked_in_constraints
 #'
-#' @aliases add_locked_in_constraints,ConservationProblem,numeric-method add_locked_in_constraints,ConservationProblem,logical-method add_locked_in_constraints,ConservationProblem,matrix-method add_locked_in_constraints,ConservationProblem,character-method  add_locked_in_constraints,ConservationProblem,Raster-method add_locked_in_constraints,ConservationProblem,Spatial-method add_locked_in_constraints,ConservationProblem,sf-method
+#' @aliases add_locked_in_constraints,ConservationProblem,numeric-method add_locked_in_constraints,ConservationProblem,logical-method add_locked_in_constraints,ConservationProblem,matrix-method add_locked_in_constraints,ConservationProblem,character-method  add_locked_in_constraints,ConservationProblem,Raster-method add_locked_in_constraints,ConservationProblem,SpatRaster-method add_locked_in_constraints,ConservationProblem,Spatial-method add_locked_in_constraints,ConservationProblem,sf-method
 
 #'
 #' @export
-methods::setGeneric("add_locked_in_constraints",
-                    signature = methods::signature("x", "locked_in"),
-                    function(x, locked_in)
-                      standardGeneric("add_locked_in_constraints"))
+methods::setGeneric(
+  "add_locked_in_constraints",
+  signature = methods::signature("x", "locked_in"),
+  function(x, locked_in) {
+    rlang::check_required(x)
+    rlang::check_required(locked_in)
+    assert(
+      is_conservation_problem(x),
+      is_inherits(
+        locked_in,
+        c(
+          "character", "numeric", "logical",
+          "matrix", "sf", "SpatRaster", "Spatial", "Raster"
+        )
+      )
+    )
+    standardGeneric("add_locked_in_constraints")
+  }
+)
 
 #' @name add_locked_in_constraints
 #' @usage \S4method{add_locked_in_constraints}{ConservationProblem,numeric}(x, locked_in)
@@ -241,19 +268,22 @@ methods::setMethod("add_locked_in_constraints",
   methods::signature("ConservationProblem", "numeric"),
   function(x, locked_in) {
     # assert valid arguments
-    assertthat::assert_that(inherits(x, "ConservationProblem"),
-      inherits(locked_in, c("integer", "numeric")),
+    assert(
+      is_conservation_problem(x),
       x$number_of_zones() == 1,
-      isTRUE(all(is.finite(locked_in))),
-      isTRUE(all(round(locked_in) == locked_in)),
-      isTRUE(max(locked_in) <= number_of_total_units(x)),
-      isTRUE(min(locked_in) >= 1))
+      is.numeric(locked_in),
+      all_finite(locked_in),
+      is_count_vector(locked_in),
+      max(locked_in) <= number_of_total_units(x),
+      min(locked_in) >= 1
+    )
     # create matrix with locked in constraints
     m <- matrix(FALSE, ncol = 1, nrow = x$number_of_total_units())
     m[locked_in, 1] <- TRUE
     # add constraints
     add_locked_in_constraints(x, m)
-})
+  }
+)
 
 #' @name add_locked_in_constraints
 #' @usage \S4method{add_locked_in_constraints}{ConservationProblem,logical}(x, locked_in)
@@ -262,14 +292,17 @@ methods::setMethod("add_locked_in_constraints",
   methods::signature("ConservationProblem", "logical"),
   function(x, locked_in) {
     # assert valid arguments
-    assertthat::assert_that(inherits(x, "ConservationProblem"),
-      inherits(locked_in, "logical"),
+    assert(
+      is_conservation_problem(x),
+      is.logical(locked_in),
+      all_finite(locked_in),
       x$number_of_zones() == 1,
-      x$number_of_total_units() == length(locked_in),
-      isTRUE(all(is.finite(locked_in))))
+      x$number_of_total_units() == length(locked_in)
+    )
     # add constraints
     add_locked_in_constraints(x, matrix(locked_in, ncol = 1))
-})
+  }
+)
 
 #' @name add_locked_in_constraints
 #' @usage \S4method{add_locked_in_constraints}{ConservationProblem,matrix}(x, locked_in)
@@ -278,24 +311,31 @@ methods::setMethod("add_locked_in_constraints",
   methods::signature("ConservationProblem", "matrix"),
   function(x, locked_in) {
     # assert valid arguments
-    assertthat::assert_that(
-      inherits(x, "ConservationProblem"),
-      inherits(locked_in, "matrix"),
+    assert(
+      is_conservation_problem(x),
+      is.matrix(locked_in),
       is.logical(locked_in),
+      all_finite(locked_in),
       x$number_of_zones() == ncol(locked_in),
       x$number_of_total_units() == nrow(locked_in),
-      all(is.finite(locked_in)),
-      all(rowSums(locked_in) <= 1))
-    assertthat::assert_that(
+      all(rowSums(locked_in) <= 1)
+    )
+    assert(
       sum(locked_in, na.rm = TRUE) > 0,
-      msg = "at least one planning unit must be locked in")
+      msg = "{.arg locked_in} must lock in at least one planning unit."
+    )
     # create data.frame with statuses
     ind <- which(locked_in, arr.ind = TRUE)
-    y <- data.frame(pu = ind[, 1], zone = x$zone_names()[ind[, 2]], status = 1,
-                    stringsAsFactors = FALSE)
+    y <- data.frame(
+      pu = ind[, 1],
+      zone = x$zone_names()[ind[, 2]],
+      status = 1,
+      stringsAsFactors = FALSE
+    )
     # add constraints
     add_manual_locked_constraints(x, y)
-})
+  }
+)
 
 #' @name add_locked_in_constraints
 #' @usage \S4method{add_locked_in_constraints}{ConservationProblem,character}(x, locked_in)
@@ -304,20 +344,45 @@ methods::setMethod("add_locked_in_constraints",
   methods::signature("ConservationProblem", "character"),
   function(x, locked_in) {
     # assert valid arguments
-    assertthat::assert_that(inherits(x, "ConservationProblem"),
-      is.character(locked_in), !anyNA(locked_in),
+    assert(
+      is_conservation_problem(x),
+      is.character(locked_in),
+      assertthat::noNA(locked_in),
+      x$number_of_zones() == length(locked_in)
+    )
+    assert(
       inherits(x$data$cost, c("data.frame", "Spatial", "sf")),
-      x$number_of_zones() == length(locked_in),
-      all(locked_in %in% names(x$data$cost)))
-    assertthat::assert_that(
-      all(vapply(as.data.frame(x$data$cost)[, locked_in, drop = FALSE],
-                 inherits, logical(1), "logical")),
-      msg = paste("argument to locked_in refers to a column with data",
-                  "that are not logical (i.e TRUE/FALSE)"))
+      msg = paste(
+        "{.arg locked_in} can only be a character vector, if the",
+        "planning unit data for {.arg x} is a",
+        "{.cls sf}, {.cls Spatial}, or data frame."
+      )
+    )
+    assert(
+      all_match_of(locked_in, names(x$data$cost)),
+      msg = paste(
+        "{.arg locked_in} must contain character values that refer to",
+        "column names of the planning unit data for {.arg x}."
+      )
+    )
+    assert(
+      all_columns_inherit(
+        as.data.frame(x$data$cost)[, locked_in, drop = FALSE],
+        "logical"
+      ),
+      msg = paste(
+        ".arg{locked_in} must refer to columns of the",
+        "planning unit data for {.arg x} that contain",
+        "logical ({.code TRUE}/{.code FALSE}) values."
+      )
+    )
     # add constraints
-    add_locked_in_constraints(x,
-      as.matrix(as.data.frame(x$data$cost)[, locked_in, drop = FALSE]))
-})
+    add_locked_in_constraints(
+      x,
+      as.matrix(as.data.frame(x$data$cost)[, locked_in, drop = FALSE])
+    )
+  }
+)
 
 #' @name add_locked_in_constraints
 #' @usage \S4method{add_locked_in_constraints}{ConservationProblem,Spatial}(x, locked_in)
@@ -325,13 +390,13 @@ methods::setMethod("add_locked_in_constraints",
 methods::setMethod("add_locked_in_constraints",
   methods::signature("ConservationProblem", "Spatial"),
   function(x, locked_in) {
-    # assert valid arguments
-    assertthat::assert_that(inherits(x, "ConservationProblem"),
-      inherits(locked_in, "Spatial"),
-      x$number_of_zones() == 1)
-    # add constraints
-    add_locked_in_constraints(x, intersecting_units(x$data$cost, locked_in))
-})
+    cli_warning(sp_pkg_deprecation_notice)
+    add_locked_in_constraints(
+      x,
+      intersecting_units(x$data$cost, sf::st_as_sf(locked_in))
+    )
+  }
+)
 
 #' @name add_locked_in_constraints
 #' @usage \S4method{add_locked_in_constraints}{ConservationProblem,sf}(x, locked_in)
@@ -340,45 +405,82 @@ methods::setMethod("add_locked_in_constraints",
   methods::signature("ConservationProblem", "sf"),
   function(x, locked_in) {
     # assert valid arguments
-    assertthat::assert_that(inherits(x, "ConservationProblem"),
+    assert(
+      is_conservation_problem(x),
       inherits(locked_in, "sf"),
-      x$number_of_zones() == 1)
+      x$number_of_zones() == 1
+    )
     # add constraints
     add_locked_in_constraints(x, intersecting_units(x$data$cost, locked_in))
-})
+  }
+)
 
 #' @name add_locked_in_constraints
 #' @usage \S4method{add_locked_in_constraints}{ConservationProblem,Raster}(x, locked_in)
 #' @rdname add_locked_in_constraints
 methods::setMethod("add_locked_in_constraints",
   methods::signature("ConservationProblem", "Raster"),
+  function(x, locked_in) {
+    # add constraints
+    cli_warning(raster_pkg_deprecation_notice)
+    add_locked_in_constraints(x, terra::rast(locked_in))
+  }
+)
+
+#' @name add_locked_in_constraints
+#' @usage \S4method{add_locked_in_constraints}{ConservationProblem,SpatRaster}(x, locked_in)
+#' @rdname add_locked_in_constraints
+methods::setMethod("add_locked_in_constraints",
+  methods::signature("ConservationProblem", "SpatRaster"),
   function(x, locked_in) {
     # assert valid arguments
-    assertthat::assert_that(inherits(x, "ConservationProblem"),
-      inherits(locked_in, "Raster"),
-      x$number_of_zones() == raster::nlayers(locked_in),
-      all(max(raster::cellStats(locked_in, "sum")) > 0))
-    if (raster::nlayers(locked_in) > 1)
-      assertthat::assert_that(raster::cellStats(sum(locked_in, na.rm = TRUE),
-                                                "max") <= 1)
+    assert(
+      is_conservation_problem(x),
+      inherits(locked_in, "SpatRaster"),
+      x$number_of_zones() == terra::nlyr(locked_in)
+    )
     # create matrix with statuses
-    if (inherits(x$data$cost, "Raster") && x$number_of_zones() > 1) {
-      status <- vapply(seq_len(x$number_of_zones()),
-                       FUN.VALUE = logical(x$number_of_total_units()),
-                       function(i) replace(rep(FALSE,
-                                               x$number_of_total_units()),
-                                           intersecting_units(x$data$cost[[i]],
-                                                              locked_in[[i]]),
-                                           TRUE))
+    if (
+      inherits(x$data$cost, c("SpatRaster", "Raster")) &&
+      isTRUE(x$number_of_zones() > 1)
+    ) {
+      status <- vapply(
+        seq_len(x$number_of_zones()),
+        FUN.VALUE = logical(x$number_of_total_units()),
+        function(i) {
+          replace(
+            rep(FALSE, x$number_of_total_units()),
+            intersecting_units(x$data$cost[[i]], locked_in[[i]]),
+            TRUE
+          )
+        }
+      )
     } else {
-      status <- vapply(seq_len(x$number_of_zones()),
-                       FUN.VALUE = logical(x$number_of_total_units()),
-                       function(i) replace(rep(FALSE,
-                                               x$number_of_total_units()),
-                                           intersecting_units(x$data$cost,
-                                                              locked_in[[i]]),
-                                           TRUE))
+      status <- vapply(
+        seq_len(x$number_of_zones()),
+        FUN.VALUE = logical(x$number_of_total_units()),
+        function(i) {
+          replace(
+            rep(FALSE, x$number_of_total_units()),
+            intersecting_units(x$data$cost, locked_in[[i]]),
+            TRUE
+          )
+        }
+      )
     }
+    # additional checks
+    assert(
+      all(rowSums(status) <= 1),
+      msg = paste(
+        "{.arg locked_in} must not specify that contain a",
+        "single planning unit should be locked to multiple zones."
+      )
+    )
+    assert(
+      sum(status) >= 1,
+      msg = "{.arg locked_in} must lock in at least one planning unit."
+    )
     # add constraints
     add_locked_in_constraints(x, status)
-})
+  }
+)
