@@ -1,9 +1,9 @@
-#' @include internal.R pproto.R Objective-proto.R
+#' @include internal.R Objective-class.R
 NULL
 
 #' Add maximum utility objective
 #'
-#' Set the objective of a conservation planning [problem()] to
+#' Set the objective of a conservation planning problem to
 #' secure as much of the features as possible without exceeding a budget.
 #' This objective does not use targets, and feature
 #' weights should be used instead to increase the representation of certain
@@ -12,7 +12,7 @@ NULL
 #' possible, and so often results in solutions that are heavily biased towards
 #' just a few features.
 #'
-#' @param x [problem()] (i.e., [`ConservationProblem-class`]) object.
+#' @param x [problem()] object.
 #'
 #' @param budget `numeric` value specifying the maximum expenditure of
 #'   the prioritization. For problems with multiple zones, the argument
@@ -26,7 +26,8 @@ NULL
 #' within a fixed budget.
 #' Additionally, weights can be used to favor the
 #' representation of certain features over other features (see
-#' [add_feature_weights()]).
+#' [add_feature_weights()]). It is essentially calculated as a weighted
+#' sum of the feature data inside the selected planning units.
 #'
 #' @section Mathematical formulation:
 #' This objective can be expressed mathematically for a set of planning units
@@ -69,46 +70,51 @@ NULL
 #' @examples
 #' \dontrun{
 #' # load data
-#' data(sim_pu_raster, sim_pu_zones_stack, sim_features, sim_features_zones)
+#' sim_pu_raster <- get_sim_pu_raster()
+#' sim_features <- get_sim_features()
+#' sim_zones_pu_raster <- get_sim_zones_pu_raster()
+#' sim_zones_features <- get_sim_zones_features()
 #'
 #' # create problem with maximum utility objective
-#' p1 <- problem(sim_pu_raster, sim_features) %>%
-#'       add_max_utility_objective(5000) %>%
-#'       add_binary_decisions() %>%
-#'       add_default_solver(gap = 0, verbose = FALSE)
+#' p1 <-
+#'   problem(sim_pu_raster, sim_features) %>%
+#'   add_max_utility_objective(5000) %>%
+#'   add_binary_decisions() %>%
+#'   add_default_solver(gap = 0, verbose = FALSE)
 #'
 #' # solve problem
 #' s1 <- solve(p1)
 #'
 #' # plot solution
-#' plot(s1, main = "solution", axes = FALSE, box = FALSE)
+#' plot(s1, main = "solution", axes = FALSE)
 #'
 #' # create multi-zone problem with maximum utility objective that
 #' # has a single budget for all zones
-#' p2 <- problem(sim_pu_zones_stack, sim_features_zones) %>%
-#'       add_max_utility_objective(5000) %>%
-#'       add_binary_decisions() %>%
-#'       add_default_solver(gap = 0, verbose = FALSE)
+#' p2 <-
+#'   problem(sim_zones_pu_raster, sim_zones_features) %>%
+#'   add_max_utility_objective(5000) %>%
+#'   add_binary_decisions() %>%
+#'   add_default_solver(gap = 0, verbose = FALSE)
 #'
 #' # solve problem
 #' s2 <- solve(p2)
 #'
 #' # plot solution
-#' plot(category_layer(s2), main = "solution", axes = FALSE, box = FALSE)
-#'
+#' plot(category_layer(s2), main = "solution", axes = FALSE)
 #'
 #' # create multi-zone problem with maximum utility objective that
 #' # has separate budgets for each zone
-#' p3 <- problem(sim_pu_zones_stack, sim_features_zones) %>%
-#'       add_max_utility_objective(c(1000, 2000, 3000)) %>%
-#'       add_binary_decisions() %>%
-#'       add_default_solver(gap = 0, verbose = FALSE)
+#' p3 <-
+#'   problem(sim_zones_pu_raster, sim_zones_features) %>%
+#'   add_max_utility_objective(c(1000, 2000, 3000)) %>%
+#'   add_binary_decisions() %>%
+#'   add_default_solver(gap = 0, verbose = FALSE)
 #'
 #' # solve problem
 #' s3 <- solve(p3)
 #'
 #' # plot solution
-#' plot(category_layer(s3), main = "solution", axes = FALSE, box = FALSE)
+#' plot(category_layer(s3), main = "solution", axes = FALSE)
 #' }
 #' @name add_max_utility_objective
 NULL
@@ -117,35 +123,39 @@ NULL
 #' @export
 add_max_utility_objective <- function(x, budget) {
   # assert argument is valid
-  assertthat::assert_that(inherits(x, "ConservationProblem"),
-                          is.numeric(budget),
-                          all(is.finite(budget)),
-                          all(budget >= 0.0),
-                          isTRUE(min(budget) > 0),
-                          length(budget) == 1 ||
-                            length(budget) == number_of_zones(x))
-  # make parameter
-  if (length(budget) == 1) {
-    p <- numeric_parameter("budget", budget, lower_limit = 0,
-                           upper_limit = sum(x$planning_unit_costs(),
-                                             na.rm = TRUE))
-  } else {
-    p <- numeric_parameter_array("budget", budget, x$zone_names(),
-                                 lower_limit = 0,
-                                 upper_limit = colSums(x$planning_unit_costs(),
-                                                       na.rm = TRUE))
-  }
+  rlang::check_required(x)
+  rlang::check_required(budget)
+  assert(
+    is_conservation_problem(x),
+    is.numeric(budget),
+    all_finite(budget),
+    all_positive(budget),
+    is_budget_length(x, budget)
+  )
   # add objective to problem
-  x$add_objective(pproto(
-    "MaximumUtilityObjective",
-    Objective,
-    name = "Maximum utility objective",
-    parameters = parameters(p),
-    apply = function(self, x, y) {
-      assertthat::assert_that(inherits(x, "OptimizationProblem"),
-                              inherits(y, "ConservationProblem"))
-      invisible(rcpp_apply_max_utility_objective(x$ptr,
-        unname(y$feature_abundances_in_planning_units()),
-        y$planning_unit_costs(), self$parameters$get("budget")[[1]]))
-    }))
+  x$add_objective(
+    R6::R6Class(
+      "MaximumUtilityObjective",
+      inherit = Objective,
+      public = list(
+        name = "maximum utility objective",
+        data = list(budget = budget),
+        apply = function(x, y) {
+          assert(
+            inherits(x, "OptimizationProblem"),
+            inherits(y, "ConservationProblem"),
+            .internal = TRUE
+          )
+          invisible(
+            rcpp_apply_max_utility_objective(
+              x$ptr,
+              unname(y$feature_abundances_in_planning_units()),
+              y$planning_unit_costs(),
+              self$get_data("budget")
+            )
+          )
+        }
+      )
+    )$new()
+  )
 }
