@@ -9,42 +9,35 @@ NULL
 #' planning units that each have higher a conductance and share a greater
 #' boundary are associated with greater connectivity.
 #'
-#' @param x [`Raster-class`],
-#'   [`SpatialPolygonsDataFrame-class`],
-#'   [`SpatialLinesDataFrame-class`],
-#'   or [sf::sf()] object
-#'   representing planning units.
-#'   If `x` is a [`Raster-class`] object then it must
-#'   contain a single layer.
+#' @param x [terra::rast()] or [sf::sf()] object representing planning units.
 #'
-#' @param y [`Raster-class`] object showing the conductance
+#' @param y [terra::rast()] object showing the conductance
 #'   of different areas across the study area, or a `character` object
 #'   denoting a column name in the attribute table of `x` that contains
 #'   the conductance values. Note that argument to `y` can only be a
-#'   `character` object if the argument to `x` is a
-#'   [`Spatial-class`] or [sf::sf()] object.
+#'   `character` object if the argument to `x` is a [sf::sf()] object.
 #'   Also, note that if the argument to `x` is a
-#'   [`Raster-class`] object then
+#'   [terra::rast()] object then
 #'   argument to `y` must have the same spatial properties as it
 #'   (i.e., coordinate system, extent, resolution).
 #'
 #' @param ... additional arguments passed to [fast_extract()] for
 #'   extracting and calculating the conductance values for each planning unit.
 #'   These arguments are only used if argument to `x` is a
-#'   \code{link[sp]{Spatial-class}} or [sf::sf()] object and argument
-#'   to `y` is a [`Raster-class`] object.
+#'   [sf::sf()] object and argument to `y` is a [terra::rast()] object.
 #'
 #' @details Shared boundary calculations are performed using
 #'   [boundary_matrix()].
 #'
-#' @return [`dsCMatrix-class`] symmetric sparse matrix object.
-#'   Each row and column represents a planning unit.
-#'   Cells values indicate the connectivity between different pairs of planning
-#'   units.
-#'   To reduce computational burden, cells among the matrix diagonal are
-#'   set to zero. Furthermore, if the argument to `x` is a
-#'   [`Raster-class`] object, then cells with `NA`
-#'   values are set to zero too.
+#' @return
+#' A [`dsCMatrix-class`] symmetric sparse matrix object.
+#' Each row and column represents a planning unit.
+#' Cells values indicate the connectivity between different pairs of planning
+#' units.
+#' To reduce computational burden, cells among the matrix diagonal are
+#' set to zero. Furthermore, if the argument to `x` is a
+#' [terra::rast()] object, then cells with missing (`NA`)
+#' values are set to zero too.
 #'
 #' @name connectivity_matrix
 #'
@@ -53,41 +46,39 @@ NULL
 #' @examples
 #' \dontrun{
 #' # load data
-#' data(sim_pu_raster, sim_pu_sf, sim_features)
+#' sim_pu_raster <- get_sim_pu_raster()
+#' sim_pu_polygons <- get_sim_pu_polygons()
+#' sim_features <- get_sim_features()
 #'
 #' # create connectivity matrix using raster planning unit data using
 #' # the raster cost values to represent conductance
 #' ## extract 9 planning units
-#' r <- crop(sim_pu_raster, c(0, 0.3, 0, 0.3))
+#' r <- terra::crop(sim_pu_raster, terra::ext(c(0, 0.3, 0, 0.3)))
 #'
 #' ## extract conductance data for the 9 planning units
-#' cd <- crop(sim_features, r)
+#' cd <- terra::crop(sim_features, r)
 #'
 #' ## make connectivity matrix using the habitat suitability data for the
 #' ## second feature to represent the planning unit conductance data
 #' cm_raster <- connectivity_matrix(r, cd[[2]])
 #'
 #' ## plot data and matrix
-#' par(mfrow = c(1,3))
-#' plot(r, main = "planning units (raster)", axes = FALSE, box = FALSE)
-#' plot(cd[[2]], main = "conductivity", axes = FALSE, box = FALSE)
-#' plot(clamp(raster(as.matrix(cm_raster)), lower = 1e-5, useValues = FALSE),
-#'      main = "connectivity", axes = FALSE, box = FALSE)
+#' plot(r, main = "planning units (raster)", axes = FALSE)
+#' plot(cd[[2]], main = "conductivity", axes = FALSE)
+#' Matrix::image(cm_raster,  main = "connectivity matrix")
 #'
 #' # create connectivity matrix using polygon planning unit data using
 #' # the habitat suitability data for the second feature to represent
 #' # planning unit conductances
 #' ## subset data to 9 polygons
-#' ply <- sim_pu_sf[c(1:2, 10:12, 20:22), ]
+#' ply <- sim_pu_polygons[c(1:2, 10:12, 20:22), ]
 #'
 #' ## make connectivity matrix
 #' cm_ply <- connectivity_matrix(ply, sim_features[[2]])
 #'
 #' ## plot data and matrix
-#' par(mfrow = c(1, 2))
-#' plot(sf::st_geometry(ply), main = "planning units (sf)")
-#' plot(clamp(raster(as.matrix(cm_ply)), lower = 1e-5, useValues = FALSE),
-#'      main = "connectivity", axes = FALSE, box = FALSE)
+#' plot(sf::st_geometry(ply), main = "planning units (polygons)")
+#' Matrix::image(cm_ply, main = "connectivity matrix")
 #'
 #' # create connectivity matrix using habitat suitability data for each feature,
 #' # this could be useful if prioritisations should spatially clump
@@ -102,10 +93,8 @@ NULL
 #' cm_sum <- Reduce("+", cm_sum) # sum matrices together
 #'
 #' ## plot data and matrix
-#' par(mfrow = c(1, 2))
-#' plot(r, main = "planning units (raster)", axes = FALSE, box = FALSE)
-#' plot(clamp(raster(as.matrix(cm_sum)), lower = 1e-5, useValues = FALSE),
-#'      main = "connectivity", axes = FALSE, box = FALSE)
+#' plot(r, main = "planning units (raster)", axes = FALSE)
+#' Matrix::image(cm_sum, main = "connectivity matrix")
 #'
 #' ## we could take this example one step further, and use weights to indicate
 #' ## relative importance of maintaining functional connectivity
@@ -121,10 +110,8 @@ NULL
 #' cm_wsum <- Reduce("+", cm_wsum) # sum matrices together
 #'
 #' ## plot data and matrix
-#' par(mfrow = c(1, 2))
-#' plot(r, main = "planning units (raster)", axes = FALSE, box = FALSE)
-#' plot(clamp(raster(as.matrix(cm_wsum)), lower = 1e-5, useValues = FALSE),
-#'      main = "connectivity", axes = FALSE, box = FALSE)
+#' plot(r, main = "planning units (raster)", axes = FALSE)
+#' Matrix::image(cm_wsum, main = "connectivity matrix")
 #'
 #' ## since the statistical distribution of the connectivity values
 #' ## for each feature (e.g., the mean and standard deviation of the
@@ -150,10 +137,8 @@ NULL
 #' cm_lwsum <- Reduce("+", cm_lwsum) # sum matrices together
 #'
 #' ## plot data and matrix
-#' par(mfrow = c(1, 2))
-#' plot(r, main = "planning units (raster)", axes = FALSE, box = FALSE)
-#' plot(clamp(raster(as.matrix(cm_lwsum)), lower = 1e-5, useValues = FALSE),
-#'      main = "connectivity", axes = FALSE, box = FALSE)
+#' plot(r, main = "planning units (raster)", axes = FALSE)
+#' Matrix::image(cm_lwsum, main = "connectivity matrix")
 #'
 #' ## another approach for normalizing the data could be using z-scores
 #' ## note that after normalizing the data we would need to add a constant
@@ -176,19 +161,26 @@ NULL
 #' cm_zwsum <- Reduce("+", cm_zwsum) # sum matrices together
 #'
 #' ## plot data and matrix
-#' par(mfrow = c(1, 2))
-#' plot(r, main = "planning units (raster)", axes = FALSE, box = FALSE)
-#' plot(clamp(raster(as.matrix(cm_zwsum)), lower = 1e-5, useValues = FALSE),
-#'      main = "connectivity", axes = FALSE, box = FALSE)
+#' plot(r, main = "planning units (raster)", axes = FALSE)
+#' Matrix::image(cm_zwsum, main = "connectivity matrix")
 #' }
 #'
-#' @aliases connectivity_matrix,Spatial,character-method connectivity_matrix,Spatial,Raster-method connectivity_matrix,Raster,Raster-method connectivity_matrix,sf,character-method connectivity_matrix,sf,Raster-method
+#' @aliases connectivity_matrix,Spatial,character-method connectivity_matrix,Spatial,Raster-method connectivity_matrix,Raster,Raster-method connectivity_matrix,sf,character-method connectivity_matrix,sf,Raster-method connectivity_matrix,sf,SpatRaster-method connectivity_matrix,SpatRaster,SpatRaster-method
 #'
 #' @export
 methods::setGeneric(
   "connectivity_matrix",
   signature = methods::signature("x", "y"),
-  function(x, y, ...) standardGeneric("connectivity_matrix"))
+  function(x, y, ...) {
+    rlang::check_required(x)
+    rlang::check_required(y)
+    assert(
+      is_spatially_explicit(x),
+      is_inherits(y, c("character", "sf", "SpatRaster", "Spatial", "Raster"))
+    )
+    standardGeneric("connectivity_matrix")
+  }
+)
 
 #' @name connectivity_matrix
 #' @usage \S4method{connectivity_matrix}{Spatial,Raster}(x, y, ...)
@@ -197,9 +189,11 @@ methods::setMethod(
   "connectivity_matrix",
   signature(x = "Spatial", y = "Raster"),
   function(x, y, ...) {
-    assertthat::assert_that(inherits(x, "Spatial"))
-    connectivity_matrix(sf::st_as_sf(x), y, ...)
-  })
+    cli_warning(sp_pkg_deprecation_notice)
+    cli_warning(raster_pkg_deprecation_notice)
+    connectivity_matrix(sf::st_as_sf(x), terra::rast(y), ...)
+  }
+)
 
 #' @name connectivity_matrix
 #' @usage \S4method{connectivity_matrix}{Spatial,character}(x, y, ...)
@@ -208,9 +202,11 @@ methods::setMethod(
   "connectivity_matrix",
   signature(x = "Spatial", y = "character"),
   function(x, y, ...) {
-    assertthat::assert_that(inherits(x, "Spatial"))
+    assert(inherits(x, "Spatial"))
+    cli_warning(sp_pkg_deprecation_notice)
     connectivity_matrix(sf::st_as_sf(x), y, ...)
-  })
+  }
+)
 
 #' @name connectivity_matrix
 #' @usage \S4method{connectivity_matrix}{sf,character}(x, y, ...)
@@ -220,19 +216,31 @@ methods::setMethod(
   signature(x = "sf", y = "character"),
   function(x, y, ...) {
     # validate that arguments are valid
-    assertthat::assert_that(inherits(x, "sf"),
+    assert(
+      inherits(x, "sf"),
       assertthat::is.string(y),
-      assertthat::has_name(x, y))
-    assertthat::assert_that(is.numeric(x[[y]]), assertthat::noNA(x[[y]]))
+      assertthat::has_name(x, y),
+      is_valid_geometries(x)
+    )
+    assert(
+      is.numeric(x[[y]]),
+      assertthat::noNA(x[[y]])
+    )
     # generate connectivity data for each pair of connected units
     bd <- matrix_to_triplet_dataframe(boundary_matrix(x))
-    bd <- bd[bd[[1]] != bd[[2]], ]
+    bd <- bd[bd[[1]] != bd[[2]], , drop = FALSE]
     bd$x <- bd$x * ( (x[[y]][bd$i] + x[[y]][bd$j]) * 0.5)
     bd <- bd[which(bd$x > 0), ]
     # generate connectivity matrix
-    Matrix::sparseMatrix(i = bd$i, j = bd$j, x = bd$x, symmetric = TRUE,
-                         dims = rep(nrow(x), 2))
-  })
+    Matrix::sparseMatrix(
+      i = bd$i,
+      j = bd$j,
+      x = bd$x,
+      symmetric = TRUE,
+      dims = rep(nrow(x), 2)
+    )
+  }
+)
 
 #' @name connectivity_matrix
 #' @usage \S4method{connectivity_matrix}{sf,Raster}(x, y, ...)
@@ -240,15 +248,29 @@ methods::setMethod(
 methods::setMethod(
   "connectivity_matrix",
   signature(x = "sf", y = "Raster"),
+  function(x, y, ...) {
+    assert(inherits(x, "sf"))
+    cli_warning(raster_pkg_deprecation_notice)
+    connectivity_matrix(x, terra::rast(y), ...)
+  }
+)
+
+#' @name connectivity_matrix
+#' @usage \S4method{connectivity_matrix}{sf,SpatRaster}(x, y, ...)
+#' @rdname connectivity_matrix
+methods::setMethod(
+  "connectivity_matrix",
+  signature(x = "sf", y = "SpatRaster"),
   function(x, y, ...) {
     # assert that arguments are valid
-    assertthat::assert_that(
+    assert(
       inherits(x, "sf"),
-      inherits(y, "Raster"),
-      raster::nlayers(y) == 1)
-    assertthat::assert_that(
-      sf::st_crs(x) == sf::st_crs(y@crs),
-      intersecting_extents(x, y))
+      inherits(y, "SpatRaster"),
+      terra::nlyr(y) == 1,
+      is_same_crs(x, y),
+      is_spatial_extents_overlap(x, y),
+      is_valid_geometries(x)
+    )
     # extract conductance values
     cv <- fast_extract(y, x, ...)
     # generate connectivity data for each pair of connected units
@@ -257,9 +279,15 @@ methods::setMethod(
     bd$x <- bd$x * ( (cv[bd$i] + cv[bd$j]) * 0.5)
     bd <- bd[which(bd$x > 0), ]
     # connectivity matrix
-    Matrix::sparseMatrix(i = bd$i, j = bd$j, x = bd$x, symmetric = TRUE,
-                         dims = rep(nrow(x), 2))
-  })
+    Matrix::sparseMatrix(
+      i = bd$i,
+      j = bd$j,
+      x = bd$x,
+      symmetric = TRUE,
+      dims = rep(nrow(x), 2)
+    )
+  }
+)
 
 #' @name connectivity_matrix
 #' @usage \S4method{connectivity_matrix}{Raster,Raster}(x, y, ...)
@@ -267,20 +295,42 @@ methods::setMethod(
 methods::setMethod(
   "connectivity_matrix",
   signature(x = "Raster", y = "Raster"),
+  function(x, y, ...) {
+    cli_warning(raster_pkg_deprecation_notice)
+    connectivity_matrix(terra::rast(x), terra::rast(y), ...)
+  }
+)
+
+#' @name connectivity_matrix
+#' @usage \S4method{connectivity_matrix}{SpatRaster,SpatRaster}(x, y, ...)
+#' @rdname connectivity_matrix
+methods::setMethod(
+  "connectivity_matrix",
+  signature(x = "SpatRaster", y = "SpatRaster"),
   function(x, y, ...) {
     # validate that arguments are valid
-    assertthat::assert_that(inherits(x, "Raster"), inherits(y, "Raster"),
-      raster::nlayers(x) == 1, raster::nlayers(y) == 1,
-      is_comparable_raster(x, y))
+    assert(
+      inherits(x, "SpatRaster"),
+      inherits(y, "SpatRaster"),
+      terra::nlyr(x) == 1,
+      terra::nlyr(y) == 1,
+      is_comparable_raster(x, y)
+    )
     # extract data from first bands in x and y
     x <- x[[1]]
     y <- y[[1]]
     # generate connectivity data for each pair of connected units
     bd <- matrix_to_triplet_dataframe(boundary_matrix(x))
-    bd <- bd[bd[[1]] != bd[[2]], ]
-    bd$x <- bd$x * ( (y[bd$i] + y[bd$j]) * 0.5)
+    bd <- bd[bd[[1]] != bd[[2]], , drop = TRUE]
+    bd$x <- bd$x * ( (y[bd$i][[1]] + y[bd$j][[1]]) * 0.5)
     bd <- bd[which(bd$x > 0), ]
     # connectivity matrix
-    Matrix::sparseMatrix(i = bd$i, j = bd$j, x = bd$x, symmetric = TRUE,
-                         dims = rep(raster::ncell(x), 2))
-  })
+    Matrix::sparseMatrix(
+      i = bd$i,
+      j = bd$j,
+      x = bd$x,
+      symmetric = TRUE,
+      dims = rep(terra::ncell(x), 2)
+    )
+  }
+)
