@@ -1,4 +1,4 @@
-#' @include internal.R pproto.R ConservationProblem-proto.R
+#' @include internal.R ConservationProblem-class.R
 NULL
 
 #' Log-linear interpolation
@@ -21,7 +21,7 @@ NULL
 #'   define the line. Values lesser or greater than these numbers are assigned
 #'   the minimum and maximum *y* coordinates.
 #'
-#' @return `numeric` values.
+#' @return A `numeric` vector.
 #'
 #' @examples
 #' \dontrun{
@@ -47,48 +47,68 @@ NULL
 #' # we can now use this function to calculate representation targets
 #' # (expressed as a percentage of the species' range sizes) using
 #' # the thresholds and cap sizes reported by Rodrigues et al. 2014
-#' spp_target_percentage_rodrigues <-
-#'   loglinear_interpolation(
+#' spp_target_percentage_rodrigues <- loglinear_interpolation(
 #'     x = spp_range_size_km2,
 #'     coordinate_one_x = 1000,
 #'     coordinate_one_y = 1,
 #'     coordinate_two_x = 250000,
-#'     coordinate_two_y = 0.1) * 100
+#'     coordinate_two_y = 0.1
+#' ) * 100
 #'
 #' # it is also common to apply a cap to the representation targets,
 #' # so let's apply the cap these targets following Butchart et al. (2015)
 #' spp_target_percentage_butchart <- ifelse(
 #'   spp_range_size_km2 >= 10000000,
 #'   (1000000 / spp_range_size_km2) * 100,
-#'   spp_target_percentage_rodrigues)
+#'   spp_target_percentage_rodrigues
+#' )
 #'
 #' # plot species range sizes and representation targets
-#' plot(spp_target_percentage_butchart ~ spp_range_size_km2,
-#'   xlab = "Range size km^2" , ylab = "Representation target (%)", type = "l")
+#' plot(
+#'   spp_target_percentage_butchart ~ spp_range_size_km2,
+#'   xlab = "Range size km^2" , ylab = "Representation target (%)", type = "l"
+#' )
 #'
 #' # plot species range sizes and representation targets on a log10 scale
-#' plot(spp_target_percentage_butchart ~ log10(spp_range_size_km2),
+#' plot(
+#'   spp_target_percentage_butchart ~ log10(spp_range_size_km2),
 #'   xlab = "Range size km^2" , ylab = "Representation target (%)",
-#'   type = "l", xaxt = "n")
-#' axis(1, pretty(log10(spp_range_size_km2)),
-#'      10^pretty(log10(spp_range_size_km2)))
+#'   type = "l", xaxt = "n"
+#' )
+#' axis(
+#'   1, pretty(log10(spp_range_size_km2)),
+#'   10^pretty(log10(spp_range_size_km2))
+#' )
 #' }
 #' @export
 loglinear_interpolation <- function(x, coordinate_one_x, coordinate_one_y,
-                                  coordinate_two_x, coordinate_two_y) {
-  assertthat::assert_that(is.numeric(x), isTRUE(all(is.finite(x))),
-                          assertthat::is.scalar(coordinate_one_x),
-                          assertthat::is.scalar(coordinate_one_y),
-                          assertthat::is.scalar(coordinate_two_x),
-                          assertthat::is.scalar(coordinate_two_y),
-                          coordinate_one_x < coordinate_two_x)
+                                    coordinate_two_x, coordinate_two_y) {
+  rlang::check_required(x)
+  rlang::check_required(coordinate_one_x)
+  rlang::check_required(coordinate_one_y)
+  rlang::check_required(coordinate_two_x)
+  rlang::check_required(coordinate_two_y)
+  assert(
+    is.numeric(x),
+    all_finite(x),
+    assertthat::is.number(coordinate_one_x),
+    assertthat::is.number(coordinate_one_y),
+    assertthat::is.number(coordinate_two_x),
+    assertthat::is.number(coordinate_two_y),
+    assertthat::noNA(coordinate_one_x),
+    assertthat::noNA(coordinate_one_y),
+    assertthat::noNA(coordinate_two_x),
+    assertthat::noNA(coordinate_two_y),
+    coordinate_one_x < coordinate_two_x
+  )
   out <- rep(NA_real_, length(x))
   out[x <= coordinate_one_x] <- coordinate_one_y
   out[x >= coordinate_two_x] <- coordinate_two_y
   between.pos <- which(is.na(out))
   out[between.pos] <- stats::approx(
-                             x = log10(c(coordinate_one_x, coordinate_two_x)),
-                             y = c(coordinate_one_y, coordinate_two_y),
-                             xout = log10(x[between.pos]), method = "linear")$y
-  return(out)
+    x = log10(c(coordinate_one_x, coordinate_two_x)),
+    y = c(coordinate_one_y, coordinate_two_y),
+    xout = log10(x[between.pos]), method = "linear"
+  )$y
+  out
 }
