@@ -1,4 +1,4 @@
-#' @include internal.R ConservationProblem-proto.R ConservationModifier-proto.R
+#' @include internal.R ConservationProblem-class.R ConservationModifier-class.R
 NULL
 
 #' Run calculations
@@ -9,26 +9,31 @@ NULL
 #' the same pre-processing steps (e.g., calculating boundary data), because
 #' means that the same calculations will not be run multiple times.
 #'
-#' @param x [problem()] (i.e., [`ConservationProblem-class`]) object.
+#' @param x [problem()] object.
 #'
 #' @details This function is used for the effect of modifying the input
 #'   [`ConservationProblem-class`] object. As such, it does not return
 #'   anything. To use this function with [pipe()] operators, use the
 #'   `%T>%` operator and not the `%>%` operator.
 #'
-#' @return Invisible `TRUE` indicating success.
+#' @return An invisible `TRUE` indicating success.
 #'
 #' @examples
 #' \dontrun{
-#' # Let us imagine a scenario where we wanted to understand the effect of
+#' # load data
+#' sim_pu_raster <- get_sim_pu_raster()
+#' sim_features <- get_sim_features()
+#'
+#' # let us imagine a scenario where we wanted to understand the effect of
 #' # setting different targets on our solution.
 #'
 #' # create a conservation problem with no targets
-#' p <- problem(sim_pu_raster, sim_features) %>%
-#'      add_min_set_objective() %>%
-#'      add_boundary_penalties(10, 0.5) %>%
-#'      add_binary_decisions() %>%
-#'      add_default_solver(verbose = FALSE)
+#' p <-
+#'   problem(sim_pu_raster, sim_features) %>%
+#'   add_min_set_objective() %>%
+#'   add_boundary_penalties(10, 0.5) %>%
+#'   add_binary_decisions() %>%
+#'   add_default_solver(verbose = FALSE)
 #'
 #' # create a copies of p and add targets
 #' p1 <- p %>% add_relative_targets(0.1)
@@ -47,12 +52,13 @@ NULL
 #'
 #' # recreate a conservation problem with no targets and tell R run the
 #' # preliminary calculations. Note how we use the %T>% operator here.
-#' p <- problem(sim_pu_raster, sim_features) %>%
-#'      add_min_set_objective() %>%
-#'      add_boundary_penalties(10, 0.5) %>%
-#'      add_binary_decisions() %>%
-#'      add_default_solver(verbose = FALSE) %T>%
-#'      run_calculations()
+#' p <-
+#'   problem(sim_pu_raster, sim_features) %>%
+#'   add_min_set_objective() %>%
+#'   add_boundary_penalties(10, 0.5) %>%
+#'   add_binary_decisions() %>%
+#'   add_default_solver(verbose = FALSE) %T>%
+#'   run_calculations()
 #'
 #' # create a copies of p and add targets just like before
 #' p1 <- p %>% add_relative_targets(0.1)
@@ -75,14 +81,15 @@ NULL
 #' }
 #' @export
 run_calculations <- function(x) {
-  assertthat::assert_that(inherits(x, "ConservationProblem"))
+  rlang::check_required(x)
+  assert(is_conservation_problem(x))
   if (!is.Waiver(x$decisions))
     x$decisions$calculate(x)
   if (!is.Waiver(x$objective))
     x$objective$calculate(x)
-  for (i in x$penalties$ids())
+  for (i in seq_along(x$penalties))
     x$penalties[[i]]$calculate(x)
-  for (i in x$constraints$ids())
+    for (i in seq_along(x$constraints))
     x$constraints[[i]]$calculate(x)
   invisible(TRUE)
 }
