@@ -1,15 +1,13 @@
-#' @include internal.R ConservationProblem-proto.R OptimizationProblem-proto.R compile.R presolve_check.R
+#' @include internal.R ConservationProblem-class.R OptimizationProblem-class.R compile.R presolve_check.R
 NULL
 
 #' Solve
 #'
-#' Solve a conservation planning [problem()].
+#' Solve a conservation planning problem.
 #'
-#' @param a [problem()] (i.e., [`ConservationProblem-class`]) or
-#'   [`OptimizationProblem-class`] object.
+#' @param a [problem()] object.
 #'
-#' @param b [`Solver-class`] object. Not used if `a` is an
-#'   [`ConservationProblem-class`] object.
+#' @param b missing.
 #'
 #' @param ... arguments passed to [compile()].
 #'
@@ -41,14 +39,10 @@ NULL
 #' `"Warning: Model contains large matrix coefficient range"` and
 #' `"Warning: Model contains large rhs"`).
 #'
-#' The object returned from this function depends on the argument to
-#' `a`. If the argument to `a` is an
-#' [`OptimizationProblem-class`] object, then the
-#' solution is returned as a `logical` vector showing the status
-#' of each planning unit in each zone. However, in most cases, the argument
-#' to `a` will be a [`ConservationProblem-class`] object, and so
-#' the type of object returned depends on the number of solutions
-#' generated and the data format used to specify the planning units:
+#' @section Output format:
+#' This function will output solutions in a similar format to the
+#' planning units associated with `a`. Specifically, it will return
+#' solutions based on the following types of planning units.
 #'
 #'   \describe{
 #'
@@ -61,50 +55,42 @@ NULL
 #'   \item{`a` has `matrix` planning units}{The solution will be
 #'     returned as a `matrix` object.
 #'     Here, rows correspond to different planning units,
-#'     and fields (columns) correspond to different  management zones.
+#'     and columns correspond to different  management zones.
 #'     Note that if a portfolio is used to generate multiple solutions,
 #'     then a `list` of such `matrix` objects will be returned.}
 #'
-#'   \item{`a` has [`Raster-class`] planning units}{The solution
-#'     will be returned as a [`Raster-class`] object.
-#'     If the argument to `x` contains a single
-#'     management zone, then a [`RasterLayer-class`] object will be returned.
-#'     Otherwise, if the argument to `x` contains multiple zones, then a
-#'     [`RasterStack-class`] object
-#'     will be returned containing a different layer for each management zone.
+#'   \item{`a` has [terra::rast()] planning units}{The solution
+#'     will be returned as a [terra::rast()] object.
+#'     If the argument to `x` contains multiple zones, then the object
+#'     will be have a different layer for each management zone.
 #'     Note that if a portfolio is used to generate multiple solutions,
-#'     then a `list` of such [`Raster-class`] objects will be returned.}
+#'     then a `list` of [terra::rast()] objects will be returned.}
 #'
-#'   \item{`a` has [`Spatial-class`], [sf::sf()], or `data.frame`
-#'     planning units}{
+#'   \item{`a` has [sf::sf()], or `data.frame` planning units}{
 #'     The solution will be returned in the same data format as the planning
 #'     units.
 #'     Here, each row corresponds to a different planning unit,
-#'     and fields contain solutions.
+#'     and columns contain solutions.
 #'     If the argument to `a` contains a single zone, then the solution object
-#'     will contain fields (columns) that solution the values.
-#'     Specifically, the field name(s) containing the solution values
+#'     will contain columns that solution the values.
+#'     Specifically, the column names containing the solution values
 #'     be will named as `"solution_XXX"` where `"XXX"` corresponds to a solution
 #'     identifier (e.g., `"solution_1"`).
-#'     If the argument to `a` contains multiple zones, then the fields
+#'     If the argument to `a` contains multiple zones, then the columns
 #'     containing solutions will be named as `"solution_XXX_YYY"` where
 #'     `"XXX"` corresponds to the solution identifier and `"YYY"` is the name
 #'     of the management zone (e.g., `"solution_1_zone1"`).}
 #'
 #'   }
 #'
-#' @return A `numeric`, `matrix`,
-#'   [`RasterLayer-class`], [`Spatial-class`],
-#'   or [sf::sf()] object containing the solution to the problem.
-#'   Additionally, the returned object will have the following additional
-#'   attributes: `"objective"` containing the solution's objective,
-#'   `"runtime"` denoting the number of seconds that elapsed while solving
-#'   the problem, and `"status"` describing the status of the solution
-#'   (e.g., `"OPTIMAL"` indicates that the optimal solution was found).
-#'   In most cases, the first solution (e.g., `"solution_1"`)
-#'   will contain the best solution found by the solver (note that this
-#'   may not be an optimal solution depending on the gap used to solve
-#'   the problem and noting that the default gap is 0.1).
+#' @return
+#' A `numeric`, `matrix`, `data.frame`, [sf::st_sf()], or
+#' [terra::rast()] object containing the solution to the problem.
+#' Additionally, the returned object will have the following additional
+#' attributes: `"objective"` containing the solution's objective,
+#' `"runtime"` denoting the number of seconds that elapsed while solving
+#' the problem, and `"status"` describing the status of the solution
+#' (e.g., `"OPTIMAL"` indicates that the optimal solution was found).
 #'
 #' @seealso
 #' See [problem()] to create conservation planning problems, and
@@ -118,15 +104,20 @@ NULL
 #' set.seed(500)
 #'
 #' # load data
-#' data(sim_pu_raster, sim_pu_polygons, sim_pu_sf, sim_features,
-#'      sim_pu_zones_stack, sim_pu_zones_sf, sim_features_zones)
+#' sim_pu_raster <- get_sim_pu_raster()
+#' sim_pu_polygons <- get_sim_pu_polygons()
+#' sim_features <- get_sim_features()
+#' sim_zones_pu_raster <- get_sim_zones_pu_raster()
+#' sim_zones_pu_polygons <- get_sim_zones_pu_polygons()
+#' sim_zones_features <- get_sim_zones_features()
 #'
 #' # build minimal conservation problem with raster data
-#' p1 <- problem(sim_pu_raster, sim_features) %>%
-#'       add_min_set_objective() %>%
-#'       add_relative_targets(0.1) %>%
-#'       add_binary_decisions() %>%
-#'       add_default_solver(verbose = FALSE)
+#' p1 <-
+#'   problem(sim_pu_raster, sim_features) %>%
+#'   add_min_set_objective() %>%
+#'   add_relative_targets(0.1) %>%
+#'   add_binary_decisions() %>%
+#'   add_default_solver(verbose = FALSE)
 #'
 #' # solve the problem
 #' s1 <- solve(p1)
@@ -144,41 +135,42 @@ NULL
 #' print(r1)
 #'
 #' # plot solution
-#' plot(s1, main = "solution", axes = FALSE, box = FALSE)
+#' plot(s1, main = "solution", axes = FALSE)
 #'
-#' # build minimal conservation problem with polygon (Spatial) data
-#' p2 <- problem(sim_pu_polygons, sim_features, cost_column = "cost") %>%
-#'       add_min_set_objective() %>%
-#'       add_relative_targets(0.1) %>%
-#'       add_binary_decisions() %>%
-#'       add_default_solver(verbose = FALSE)
+#' # build minimal conservation problem with polygon data
+#' p2 <-
+#'   problem(sim_pu_polygons, sim_features, cost_column = "cost") %>%
+#'   add_min_set_objective() %>%
+#'   add_relative_targets(0.1) %>%
+#'   add_binary_decisions() %>%
+#'   add_default_solver(verbose = FALSE)
 #'
 #' # solve the problem
 #' s2 <- solve(p2)
 #'
-#' # print first six rows of the attribute table
-#' print(head(s2))
+#' # print solution
+#' print(s2)
 #'
 #' # calculate feature representation in the solution
 #' r2 <- eval_feature_representation_summary(p2, s2[, "solution_1"])
 #' print(r2)
 #'
 #' # plot solution
-#' spplot(s2, zcol = "solution_1", main = "solution", axes = FALSE, box = FALSE)
+#' plot(s2[, "solution_1"], main = "solution", axes = FALSE)
 #'
-#'
-#' # build minimal conservation problem with polygon (sf) data
-#' p3 <- problem(sim_pu_sf, sim_features, cost_column = "cost") %>%
-#'       add_min_set_objective() %>%
-#'       add_relative_targets(0.1) %>%
-#'       add_binary_decisions() %>%
-#'       add_default_solver(verbose = FALSE)
+#' # build minimal conservation problem with polygon data
+#' p3 <-
+#'   problem(sim_pu_polygons, sim_features, cost_column = "cost") %>%
+#'   add_min_set_objective() %>%
+#'   add_relative_targets(0.1) %>%
+#'   add_binary_decisions() %>%
+#'   add_default_solver(verbose = FALSE)
 #'
 #' # solve the problem
 #' s3 <- solve(p3)
 #'
-#' # print first six rows of the attribute table
-#' print(head(s3))
+#' # print solution
+#' print(s3)
 #'
 #' # calculate feature representation in the solution
 #' r3 <- eval_feature_representation_summary(p3, s3[, "solution_1"])
@@ -187,14 +179,13 @@ NULL
 #' # plot solution
 #' plot(s3[, "solution_1"])
 #'
-#'
 #' # build multi-zone conservation problem with raster data
-#' p4 <- problem(sim_pu_zones_stack, sim_features_zones) %>%
-#'       add_min_set_objective() %>%
-#'       add_relative_targets(matrix(runif(15, 0.1, 0.2), nrow = 5,
-#'                                   ncol = 3)) %>%
-#'       add_binary_decisions() %>%
-#'       add_default_solver(verbose = FALSE)
+#' p4 <-
+#'   problem(sim_zones_pu_raster, sim_zones_features) %>%
+#'   add_min_set_objective() %>%
+#'   add_relative_targets(matrix(runif(15, 0.1, 0.2), nrow = 5, ncol = 3)) %>%
+#'   add_binary_decisions() %>%
+#'   add_default_solver(verbose = FALSE)
 #'
 #' # solve the problem
 #' s4 <- solve(p4)
@@ -207,33 +198,36 @@ NULL
 #' print(r4)
 #'
 #' # plot solution
-#' plot(category_layer(s4), main = "solution", axes = FALSE, box = FALSE)
+#' plot(category_layer(s4), main = "solution", axes = FALSE)
 #'
-#' # build multi-zone conservation problem with polygon (sf) data
-#' p5 <- problem(sim_pu_zones_sf, sim_features_zones,
-#'               cost_column = c("cost_1", "cost_2", "cost_3")) %>%
-#'       add_min_set_objective() %>%
-#'       add_relative_targets(matrix(runif(15, 0.1, 0.2), nrow = 5,
-#'                                   ncol = 3)) %>%
-#'       add_binary_decisions() %>%
-#'       add_default_solver(verbose = FALSE)
+#' # build multi-zone conservation problem with polygon data
+#' p5 <-
+#'   problem(
+#'     sim_zones_pu_polygons, sim_zones_features,
+#'     cost_column = c("cost_1", "cost_2", "cost_3")
+#'   ) %>%
+#'   add_min_set_objective() %>%
+#'   add_relative_targets(matrix(runif(15, 0.1, 0.2), nrow = 5, ncol = 3)) %>%
+#'   add_binary_decisions() %>%
+#'   add_default_solver(verbose = FALSE)
 #'
 #' # solve the problem
 #' s5 <- solve(p5)
 #'
-#' # print first six rows of the attribute table
-#' print(head(s5))
+#' # print solution
+#' print(s5)
 #'
 #' # calculate feature representation in the solution
 #' r5 <- eval_feature_representation_summary(
-#'   p5, s5[, c("solution_1_zone_1", "solution_1_zone_2", "solution_1_zone_3")])
+#'   p5, s5[, c("solution_1_zone_1", "solution_1_zone_2", "solution_1_zone_3")]
+#' )
 #' print(r5)
 #'
 #' # create new column representing the zone id that each planning unit
 #' # was allocated to in the solution
-#' s5$solution <- category_vector(s5[, c("solution_1_zone_1",
-#'                                       "solution_1_zone_2",
-#'                                       "solution_1_zone_3")])
+#' s5$solution <- category_vector(
+#'   s5[, c("solution_1_zone_1", "solution_1_zone_2", "solution_1_zone_3")]
+#' )
 #' s5$solution <- factor(s5$solution)
 #'
 #' # plot solution
@@ -245,20 +239,11 @@ NULL
 #'
 #' @exportMethod solve
 #'
-#' @aliases solve,OptimizationProblem,Solver-method solve,ConservationProblem,missing-method
+#' @aliases solve,ConservationProblem,missing-method
 #'
 #' @export
 NULL
 
-#' @name solve
-#'
-#' @rdname solve
-methods::setMethod(
-  "solve",
-  signature(a = "OptimizationProblem", b = "Solver"),
-  function(a, b, ...) b$solve(a)
-)
-
 #' @name solve
 #'
 #' @rdname solve
@@ -267,67 +252,111 @@ methods::setMethod(
   signature(a = "ConservationProblem", b = "missing"),
   function(a, b, ..., run_checks = TRUE, force = FALSE) {
     # assert arguments are valid
-    assertthat::assert_that(
-      assertthat::is.flag(run_checks), assertthat::noNA(run_checks),
-      assertthat::is.flag(force), assertthat::noNA(force))
-    # assign default solver and portfolio
-    if (inherits(a$solver, "Waiver"))
-      a <- add_default_solver(a)
-    default_portfolio <- inherits(a$portfolio, "Waiver")
-    if (inherits(a$portfolio, "Waiver"))
-      a <- add_default_portfolio(a) #nocov
+    rlang::check_required(a)
+    assert(
+      assertthat::is.flag(run_checks),
+      assertthat::noNA(run_checks),
+      assertthat::is.flag(force),
+      assertthat::noNA(force)
+    )
+    # compile optimization problem
+    opt <- compile.ConservationProblem(a, ...)
     # run presolve check to try to identify potential problems
     if (run_checks) {
-      ch <- presolve_check(a)
-      if (!isTRUE(force) && !isTRUE(ch))
-        stop(paste("problem failed presolve checks, for more information see",
-                   "?presolve_check"))
+      ## run checks
+      presolve_res <- internal_presolve_check(opt)
+      ## prepare message
+      msg <- presolve_res$msg
+      if (!isTRUE(force)) {
+        msg <- c(
+          msg,
+          "i" = paste(
+            "To ignore checks and attempt optimization anyway,",
+            "use {.code solve(force = TRUE)}."
+          )
+        )
+      }
+      ## determine if error or warning should be thrown
+      if (!isTRUE(force)) {
+        f <- assert
+      } else {
+        f <- verify
+      }
+      ## throw error or warning if checks failed
+      f(isTRUE(presolve_res$pass), call = parent.frame(), msg = msg)
     }
-    # compile optimisation problem
-    opt <- compile.ConservationProblem(a, ...)
     # solve problem
     sol <- a$portfolio$run(opt, a$solver)
+    # prepare message
+    msg <- c(
+      "Can't find a solution!",
+      "i" = paste(
+        "This is because it is impossible to meet the",
+        "targets, budgets, or constraints."
+      )
+    )
+    if (isTRUE(a$solver$data$time_limit < 1e5)) {
+      msg <- c(
+        msg,
+        "i" = paste(
+          "It could also be because the {.arg time_limit}",
+          "is too low."
+        )
+      )
+    }
     # check that solution is valid
-    #nocov start
-    if (is.null(sol) || is.null(sol[[1]]$x)) {
-      stop("no solution found (e.g., due to problem infeasibility or time ",
-           "limits)")
+    assert(
+      !is.null(sol) && !is.null(sol[[1]]$x),
+      msg = msg
+    )
+    # check that desired number of solutions were found
+    portfolio_number_solutions <- a$portfolio$get_data("number_solutions")
+    if (!is.Waiver(portfolio_number_solutions)) {
+      if (length(sol) != portfolio_number_solutions) {
+        cli_warning(
+          paste(
+            "Portfolio could only find",
+            "{.val {length(sol)}} out of",
+            "{.val {portfolio_number_solutions}}",
+            "solution{?s}."
+          )
+        )
+      }
     }
-    #nocov end
     ## format solutions
     # format solutions into planning unit by zones matrix
     na_pos <- which(is.na(a$planning_unit_costs()), arr.ind = TRUE)
     sol_status <- lapply(sol, function(x) {
-      m <- matrix(x[[1]][seq_len(a$number_of_planning_units() *
-                                 a$number_of_zones())],
-                  nrow = a$number_of_planning_units(),
-                  ncol = a$number_of_zones())
+      m <- matrix(
+        x[[1]][seq_len(a$number_of_planning_units() * a$number_of_zones())],
+        nrow = a$number_of_planning_units(),
+        ncol = a$number_of_zones()
+      )
       m[na_pos] <- NA_real_
-      return(m)
+      m
     })
     # create solution data
     pu <- a$data$cost
-    if (inherits(pu, "Raster")) {
-      # RasterLayer planning units
-      if (raster::nlayers(pu) == 1) {
-        pos <- raster::Which(!is.na(pu), cells = TRUE)
-      } else {
-        pos <- raster::Which(max(!is.na(pu)) > 0, cells = TRUE)
-      }
-      pu <- suppressWarnings(raster::setValues(pu[[1]], NA))
+    if (inherits(pu, c("SpatRaster", "Raster"))) {
+      # SpatRaster or Raster planning units
+      pos <- a$planning_unit_indices()
+      pu <- terra::rast(pu)
+      pu <- suppressWarnings(terra::setValues(pu[[1]], NA))
       ret <- lapply(sol_status, function(s) {
         ret <- lapply(seq_len(ncol(s)), function(z) {
           pu[pos] <- s[, z]
-          return(pu)
+          pu
         })
-        if (length(ret) > 1) {
-          ret <- raster::stack(ret)
-        } else {
-          ret <- ret[[1]]
-        }
+        ret <- terra::rast(ret)
         names(ret) <- a$zone_names()
-        return(ret)
+        ret
       })
+      # convert to RasterStack or RasterLayer if needed
+      if (inherits(a$data$cost, c("RasterStack", "RasterBrick"))) {
+        ret <- lapply(ret, raster::stack)
+      } else if (inherits(a$data$cost, "RasterLayer")) {
+        ret <- lapply(ret, raster::raster)
+      }
       names(ret) <- paste0("solution_", seq_along(sol))
     } else if (inherits(pu, c("data.frame", "Spatial", "sf"))) {
       # Spatial* or data.frame planning units
@@ -335,19 +364,23 @@ methods::setMethod(
       if (a$number_of_zones() == 1) {
         colnames(sol_status) <- paste0("solution_", seq_along(sol))
       } else {
-        colnames(sol_status) <- paste0("solution_",
-                                       rep(seq_along(sol),
-                                          each = a$number_of_zones()), "_",
-                                       rep(a$zone_names(), length(sol)))
+        colnames(sol_status) <- paste0(
+          "solution_",
+          rep(seq_along(sol), each = a$number_of_zones()),
+          "_",
+          rep(a$zone_names(), length(sol))
+        )
       }
       # add in NA values for planning units that contained NA values in
       # all zones that were discarded from the mathematical formulation
       # to reduce overheads
-      pos <- which(rowSums(!is.na(as.matrix(
-               as.data.frame(pu)[, a$data$cost_column, drop = FALSE]))) > 0)
+      pos <- a$planning_unit_indices()
       if (!identical(pos, seq_len(a$number_of_total_units()))) {
-        sol_status2 <- matrix(NA_real_, nrow = a$number_of_total_units(),
-                              ncol = ncol(sol_status))
+        sol_status2 <- matrix(
+          NA_real_,
+          nrow = a$number_of_total_units(),
+          ncol = ncol(sol_status)
+        )
         sol_status2[pos, ] <- sol_status
         dimnames(sol_status2) <- dimnames(sol_status)
       } else {
@@ -359,43 +392,56 @@ methods::setMethod(
         ret <- pu
         ret@data <- cbind(ret@data, sol_status2)
       } else if (inherits(pu, "sf")) {
-        ret <- sf::st_sf(data.frame(pu, sol_status2))
+        ret <- tibble::as_tibble(data.frame(pu, sol_status2))
+        sf_col <- attr(pu, "sf_column")
+        ret <- ret[, c(setdiff(names(ret), sf_col), sf_col), drop = FALSE]
+        ret <- ret <- sf::st_sf(ret)
       } else {
         ret <- cbind(pu, sol_status2)
+        if (inherits(pu, "tbl_df")) {
+          pu <- tibble::as_tibble(pu)
+        }
       }
     } else if (is.matrix(pu)) {
       # matrix planning units
       # add in NA values for planning units that contained NA values in
       # all zones that were discarded from the mathematical formulation
       # to reduce overheads
-      pos <- which(rowSums(!is.na(pu)) > 0)
+      pos <- a$planning_unit_indices()
       pu[] <- NA
       colnames(pu) <- a$zone_names()
       ret <- lapply(sol_status, function(s) {
         pu[pos, ] <- s
-        return(pu)
+        pu
       })
       names(ret) <- paste0("solution_", seq_along(sol))
     } else {
-      stop("planning unit data is of an unrecognized class") # nocov
+      # nocov start
+      cli::cli_abort(
+        "Planning unit data is of an unrecognized class.",
+        .internal = TRUE
+      )
+      # nocov end
     }
     # if ret is a list of matrices with a single column then convert to numeric
-    if (is.matrix(ret[[1]]) && ncol(ret[[1]]) == 1)
+    if (is.matrix(ret[[1]]) && ncol(ret[[1]]) == 1) {
       ret <- lapply(ret, as.numeric)
+    }
     # if ret is a list with a single element then extract the element
-    if (length(ret) == 1 && default_portfolio)
+    if (length(ret) == 1) {
       ret <- ret[[1]]
+    }
     # add attributes
-    attr(ret, "objective") <-
-      stats::setNames(
-        vapply(sol, `[[`, numeric(1), 2), paste0("solution_", seq_along(sol)))
-    attr(ret, "status") <-
-      stats::setNames(
-        vapply(sol, `[[`, character(1), 3), paste0("solution_", seq_along(sol)))
-    attr(ret, "runtime") <-
-      stats::setNames(
-        vapply(sol, `[[`, numeric(1), 4), paste0("solution_", seq_along(sol)))
+    attr(ret, "objective") <- stats::setNames(
+      vapply(sol, `[[`, numeric(1), 2), paste0("solution_", seq_along(sol))
+    )
+    attr(ret, "status") <- stats::setNames(
+      vapply(sol, `[[`, character(1), 3), paste0("solution_", seq_along(sol))
+    )
+    attr(ret, "runtime") <- stats::setNames(
+      vapply(sol, `[[`, numeric(1), 4), paste0("solution_", seq_along(sol))
+    )
     # return object
-    return(ret)
+    ret
   }
 )
