@@ -1,10 +1,10 @@
 #include "package.h"
 #include "optimization_problem.h"
-#include "rcpp_boundary_data.h"
+#include "functions.h"
 
 // [[Rcpp::export]]
 bool rcpp_apply_connectivity_penalties(SEXP x, double penalty,
-                                       Rcpp::List data) {
+                                       const Rcpp::List data) {
 
   /* The following code makes the following critical assumptions
    *
