@@ -2,14 +2,17 @@ context("eval_ferrier_importance")
 
 test_that("numeric", {
   # create data
-  pu <- data.frame(id = seq_len(4),
-                   cost = c(10, 2, NA, 3),
-                   spp1 = c(0, 0, 0, 1),
-                   spp2 = c(10, 5, 10, 6))
+  pu <- data.frame(
+    id = seq_len(4), cost = c(10, 2, NA, 3),
+    spp1 = c(0, 0, 0, 1), spp2 = c(10, 5, 10, 6)
+  )
   # create problem
   p <-
-    problem(pu$cost, data.frame(id = seq_len(2), name = c("spp1", "spp2")),
-            as.matrix(t(pu[, 3:4]))) %>%
+    problem(
+      pu$cost,
+      data.frame(id = seq_len(2), name = c("spp1", "spp2")),
+      as.matrix(t(pu[, 3:4]))
+    ) %>%
     add_min_set_objective() %>%
     add_absolute_targets(c(1, 10)) %>%
     add_binary_decisions() %>%
@@ -37,15 +40,17 @@ test_that("numeric", {
 
 test_that("matrix (single zone)", {
   # create data
-  pu <- data.frame(id = seq_len(4),
-                   cost = c(10, 2, NA, 3),
-                   spp1 = c(0, 0, 0, 1),
-                   spp2 = c(10, 5, 10, 6))
+  pu <- data.frame(
+    id = seq_len(4), cost = c(10, 2, NA, 3),
+    spp1 = c(0, 0, 0, 1), spp2 = c(10, 5, 10, 6)
+  )
   # create problem
   p <-
-    problem(matrix(pu$cost, ncol = 1),
-            data.frame(id = seq_len(2), name = c("spp1", "spp2")),
-            as.matrix(t(pu[, 3:4]))) %>%
+    problem(
+      matrix(pu$cost, ncol = 1),
+      data.frame(id = seq_len(2), name = c("spp1", "spp2")),
+      as.matrix(t(pu[, 3:4]))
+    ) %>%
     add_min_set_objective() %>%
     add_absolute_targets(c(1, 10)) %>%
     add_binary_decisions() %>%
@@ -73,10 +78,10 @@ test_that("matrix (single zone)", {
 
 test_that("data.frame (single zone)", {
   # create data
-  pu <- data.frame(id = seq_len(4),
-                   cost = c(10, 2, NA, 3),
-                   spp1 = c(0, 0, 0, 1),
-                   spp2 = c(10, 5, 10, 6))
+  pu <- data.frame(
+    id = seq_len(4), cost = c(10, 2, NA, 3),
+    spp1 = c(0, 0, 0, 1), spp2 = c(10, 5, 10, 6)
+  )
   # create problem
   p <-
     problem(pu, c("spp1", "spp2"), cost_column = "cost") %>%
@@ -107,52 +112,13 @@ test_that("data.frame (single zone)", {
   expect_equal(r1, tibble::as_tibble(r2))
 })
 
-test_that("Spatial (single zone)", {
-  # create data
-  data(sim_pu_polygons)
-  pu <- sim_pu_polygons[1:4, ]
-  pu@data <- data.frame(id = seq_len(4),
-                        cost = c(10, 2, NA, 3),
-                        spp1 = c(0, 0, 0, 1),
-                        spp2 = c(10, 5, 10, 6))
-  # create problem
-  p <-
-    problem(pu, c("spp1", "spp2"), cost_column = "cost") %>%
-    add_min_set_objective() %>%
-    add_absolute_targets(c(1, 10)) %>%
-    add_binary_decisions() %>%
-    add_default_solver(gap = 0, verbose = FALSE)
-  # create a solution
-  pu$solution <- c(0, 1, NA, 1)
-  # calculate scores
-  r1 <- eval_ferrier_importance(p, pu[, "solution"])
-  # create correct total scores
-  r2 <- as.data.frame(
-    as.matrix(
-      ferrier_scores_r(
-        rij = t(as.matrix(pu@data[, c("spp1", "spp2")])),
-        targets = c(1, 10),
-        solution = pu$solution
-      )
-    )
-  )
-  # run tests
-  expect_is(r1, "SpatialPolygonsDataFrame")
-  expect_equal(ncol(r1), 3)
-  expect_equal(nrow(r1), 4)
-  expect_equal(names(r1), c("spp1", "spp2", "total"))
-  expect_equal(unname(rowSums(as.matrix(r1@data[, -3]))), r1@data[[3]])
-  expect_equal(tibble::as_tibble(r1@data), tibble::as_tibble(r2))
-})
-
 test_that("sf (single zone)", {
   # create data
-  data(sim_pu_sf)
-  pu <- sim_pu_sf[1:4, ]
-  pu$id <- seq_len(4)
-  pu$cost <- c(10, 2, NA, 3)
-  pu$spp1 <- c(0, 0, 0, 1)
-  pu$spp2 <- c(10, 5, 10, 6)
+  pu <- get_sim_pu_polygons()[1:4, ]
+  pu$id = seq_len(4)
+  pu$cost = c(10, 2, NA, 3)
+  pu$spp1 = c(0, 0, 0, 1)
+  pu$spp2 = c(10, 5, 10, 6)
   # create problem
   p <-
     problem(pu, c("spp1", "spp2"), cost_column = "cost") %>%
@@ -165,12 +131,14 @@ test_that("sf (single zone)", {
   # calculate scores
   r1 <- eval_ferrier_importance(p, pu[, "solution"])
   # create correct total scores
-  r2 <- as.data.frame(
-    as.matrix(
-      ferrier_scores_r(
-        rij = t(as.matrix(sf::st_drop_geometry(pu)[, c("spp1", "spp2")])),
-        targets = c(1, 10),
-        solution = pu$solution
+  r2 <- tibble::as_tibble(
+    as.data.frame(
+      as.matrix(
+        ferrier_scores_r(
+          rij = t(as.matrix(sf::st_drop_geometry(pu)[, c("spp1", "spp2")])),
+          targets = c(1, 10),
+          solution = pu$solution
+        )
       )
     )
   )
@@ -179,19 +147,24 @@ test_that("sf (single zone)", {
   expect_equal(ncol(sf::st_drop_geometry(r1)), 3)
   expect_equal(nrow(r1), 4)
   expect_equal(names(sf::st_drop_geometry(r1)), c("spp1", "spp2", "total"))
-  expect_equivalent(r1$spp1 + r1$spp2, r1$total)
   expect_equal(
-    tibble::as_tibble(sf::st_drop_geometry(r1)[, c("spp1", "spp2", "total")]),
-    tibble::as_tibble(r2)
+    unname(rowSums(as.matrix(sf::st_drop_geometry(r1)[, -3]))),
+    r1[[3]]
   )
+  expect_equal(sf::st_drop_geometry(r1), r2)
+  expect_equal(sf::st_geometry(r1), sf::st_geometry(pu))
 })
 
-test_that("Raster (single zone)", {
+test_that("SpatRaster (single zone)", {
   # create data
-  pu <- raster::raster(matrix(c(10, 2, NA, 3), nrow = 2))
-  features <- raster::stack(raster::raster(matrix(c(0, 0, 0, 1), nrow = 2)),
-                            raster::raster(matrix(c(10, 5, 10, 6), nrow = 2)))
+  pu <- terra::rast(matrix(c(10, 2, NA, 3), nrow = 2))
+  features <- c(
+    terra::rast(matrix(c(0, 0, 0, 1), nrow = 2)),
+    terra::rast(matrix(c(10, 5, 10, 6), nrow = 2))
+  )
   names(features) <- c("spp1", "spp2")
+  # create a solution
+  s <- terra::rast(matrix(c(0, 1, NA, 1), nrow = 2))
   # create problem
   p <-
     problem(pu, features) %>%
@@ -199,30 +172,102 @@ test_that("Raster (single zone)", {
     add_absolute_targets(c(1, 10)) %>%
     add_binary_decisions() %>%
     add_default_solver(gap = 0, verbose = FALSE)
-  # create a solution
-  s <- raster::raster(matrix(c(0, 1, NA, 1), nrow = 2))
   # calculate scores
   r1 <- eval_ferrier_importance(p, s)
   # create correct total scores
   r2 <- as.data.frame(
     as.matrix(
       ferrier_scores_r(
-        rij = t(as.matrix(raster::as.data.frame(features))),
+        rij = t(as.matrix(terra::as.data.frame(features))),
         targets = c(1, 10),
         solution = raster::values(s)
       )
     )
   )
   # run tests
-  expect_is(r1, "Raster")
-  expect_equal(raster::nlayers(r1), 3)
-  expect_equal(raster::ncell(r1), 4)
-  expect_equal(raster::ncol(r1), raster::ncol(pu))
-  expect_equal(raster::nrow(r1), raster::nrow(pu))
+  expect_is(r1, "SpatRaster")
+  expect_equal(terra::nlyr(r1), 3)
+  expect_equal(terra::ncell(r1), 4)
+  expect_equal(terra::ncol(r1), terra::ncol(pu))
+  expect_equal(terra::nrow(r1), terra::nrow(pu))
   expect_equal(names(r1), c("spp1", "spp2", "total"))
-  expect_equal(raster::getValues(r1[["spp1"]]), r2$spp1)
-  expect_equal(raster::getValues(r1[["spp2"]]), r2$spp2)
-  expect_equal(raster::getValues(r1[["total"]]), r2$total)
+  expect_equal(c(terra::values(r1[["spp1"]])), r2$spp1)
+  expect_equal(c(terra::values(r1[["spp2"]])), r2$spp2)
+  expect_equal(c(terra::values(r1[["total"]])), r2$total)
+})
+
+test_that("Spatial (single zone)", {
+  # create data
+  sim_pu_polygons <- get_sim_pu_polygons()
+  pu <- sim_pu_polygons[1:4, ]
+  pu$id <- seq_len(4)
+  pu$cost <- c(10, 2, NA, 3)
+  pu$spp1 <- c(0, 0, 0, 1)
+  pu$spp2 <- c(10, 5, 10, 6)
+  pu$solution <- c(0, 1, NA, 1)
+  # create problems
+  p1 <-
+    problem(pu, c("spp1", "spp2"), cost_column = "cost") %>%
+    add_min_set_objective() %>%
+    add_absolute_targets(c(1, 10)) %>%
+    add_binary_decisions() %>%
+    add_default_solver(gap = 0, verbose = FALSE)
+  expect_warning(
+    p2 <-
+      problem(sf::as_Spatial(pu), c("spp1", "spp2"), cost_column = "cost") %>%
+      add_min_set_objective() %>%
+      add_absolute_targets(c(1, 10)) %>%
+      add_binary_decisions() %>%
+      add_default_solver(gap = 0, verbose = FALSE),
+    "deprecated"
+  )
+  # calculate scores
+  r1 <- eval_ferrier_importance(p1, pu[, "solution"])
+  expect_warning(
+    r2 <- eval_ferrier_importance(p2, sf::as_Spatial(pu[, "solution"])),
+    "deprecated"
+  )
+  # run tests
+  expect_equal(
+    tibble::tibble(sf::st_drop_geometry(r1)),
+    tibble::tibble(r2@data)
+  )
+})
+
+test_that("Raster (single zone)", {
+  # create data
+  pu <- terra::rast(matrix(c(10, 2, NA, 3), nrow = 2))
+  features <- c(
+    terra::rast(matrix(c(0, 0, 0, 1), nrow = 2)),
+    terra::rast(matrix(c(10, 5, 10, 6), nrow = 2))
+  )
+  names(features) <- c("spp1", "spp2")
+  # create a solution
+  s <- terra::rast(matrix(c(0, 1, NA, 1), nrow = 2))
+  # create problems
+  p1 <-
+    problem(pu, features) %>%
+    add_min_set_objective() %>%
+    add_absolute_targets(c(1, 10)) %>%
+    add_binary_decisions() %>%
+    add_default_solver(gap = 0, verbose = FALSE)
+  expect_warning(
+    p2 <-
+      problem(raster::stack(pu), raster::stack(features)) %>%
+      add_min_set_objective() %>%
+      add_absolute_targets(c(1, 10)) %>%
+      add_binary_decisions() %>%
+      add_default_solver(gap = 0, verbose = FALSE),
+    "deprecated"
+  )
+  # calculate scores
+  r1 <- eval_ferrier_importance(p1, s)
+  expect_warning(
+    r2 <- eval_ferrier_importance(p2, raster::stack(s)),
+    "deprecated"
+  )
+  # run tests
+  expect_equal(terra::values(r1), terra::values(terra::rast(r2)))
 })
 
 test_that("data.frame (complex dataset)", {
@@ -232,13 +277,14 @@ test_that("data.frame (complex dataset)", {
   # - portfolio_size is based on a Marxan analyses
   # define raw data
   rij <- readRDS(system.file("testdata", "rij.rds", package = "prioritizr"))
-  targ <- readRDS(system.file("testdata", "targets.rds",
-                              package = "prioritizr"))
+  targ <- readRDS(
+    system.file("testdata", "targets.rds", package = "prioritizr")
+  )
   portfolio_size <- 803
   # prepare for prioritizr
   s <- sample.int(ncol(rij), portfolio_size)
   s <- tibble::tibble(solution = replace(rep(0, ncol(rij)), s, 1))
-  pu <- as.data.frame(as.matrix(t(rij)))
+  pu <- as.data.frame(as.matrix(Matrix::t(rij)))
   names(pu) <- targ$Name
   pu$cost <- 1
   # create problem
